/* 
 * cdlerrors.H
 *
 *   These are constants for the errors generated by code made by CDL.
 * These are defined as a big enum. If you need to change them, just 
 * change the value of CDLErrorBase (and tell everyone in the world
 * to recompile).
 *    Here, we also define CDLStatusToString, which takes a CDLStatus and
 * returns a string that explains it. It's in cdlerrorfuncs.C.
 *
 * cdlerrors.H,v 1.6 1994/04/07 19:30:57 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_ERROR_DEFS_H
#define CDL_ERROR_DEFS_H

extern "C"
{
#include <pthread.h>
}
#include <String.h>
#include <cdl/cdlbasedefs.H>

#define CDLErrorBase 999

typedef enum
{
  CDL_SUCCESS = 0,
  CDL_SFS_ERROR = CDLErrorBase + 0,
  CDL_END_OF_RANGE = CDLErrorBase + 2,
  CDL_FILE_NOT_FOUND = CDLErrorBase + 3,
  CDL_PERMISSION_DENIED = CDLErrorBase + 4,
  CDL_CRP_UNDEFINED = CDLErrorBase + 5,
  CDL_DISALLOWED_ON_PRIMARY_INDEX = CDLErrorBase + 6,
  CDL_FILE_NAME_EXISTS = CDLErrorBase + 7,
  CDL_INDEX_NOT_ACTIVE = CDLErrorBase + 8,
  CDL_INDEX_NOT_UNIQUE = CDLErrorBase + 9,
  CDL_FILE_SYSTEM_NOT_FOUND = CDLErrorBase + 10,
  CDL_KEY_NOT_FOUND = CDLErrorBase + 11,
  CDL_TIMED_OUT = CDLErrorBase + 12,
  CDL_TRANSACTION_REQUIRED = CDLErrorBase + 13,
  CDL_IO_ERROR = CDLErrorBase + 14,
  CDL_COMM_ERROR = CDLErrorBase + 15,
  CDL_VOLUME_FULL = CDLErrorBase + 16,
  CDL_DIRTY_READ = CDLErrorBase + 17,
  CDL_FILES_LEFT_OPEN = CDLErrorBase + 18,
  CDL_FILE_NOT_OPEN = CDLErrorBase + 19,
  CDL_FILE_ALREADY_OPEN = CDLErrorBase + 20,
  CDL_TRANSACTION_ABORTED = CDLErrorBase + 21,
  CDL_FILE_TOO_LARGE = CDLErrorBase + 22,
  CDL_SLOT_OCCUPIED = CDLErrorBase + 23,
  CDL_FILE_DAMAGED = CDLErrorBase + 24,
  CDL_BATCH_QUEUE_CORRUPTED = CDLErrorBase + 25,
  CDL_DUPLICATE_IN_UNIQUE_INDEX = CDLErrorBase + 26,
  CDL_END_OF_QUEUE = CDLErrorBase + 27,
  CDL_TYPE_MISMATCH = CDLErrorBase + 28,
  CDL_INVALID_INDEX = CDLErrorBase + 29,
  CDL_RECORD_NOT_FOUND = CDLErrorBase + 30,
  CDL_LIST_COUNT_MISMATCH = CDLErrorBase + 31,
  CDL_UNPACKING_ERROR = CDLErrorBase + 32,
  CDL_PACKING_ERROR = CDLErrorBase + 33
} CDLStatus;

extern void CDLStatusToString(const CDLStatus status,
			      String& out);

#endif
