    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef PACKDEFINED
#define PACKDEFINED

extern "C"
{
#include <pthread.h>
}
#include <dce/idlbase.h>
#include <cdl/rtsutil.H>
#include <String.h>

/* return the total of all field sizes in a record type specification */
unsigned long FieldSizeTotal(const rqs_typeSpec_t & rts);
unsigned long FieldSizeTotal(const rqs_typeSpec_t & rts, int key);
unsigned long FieldSizeTotal(const rqs_recordFieldSpec_t & rfs);
unsigned long FieldSizeTotal(const rqs_recordSpec_t & rs);

/* pack and unpack known types.  return value is ``next'' address in buffer.
 * the length parameter, where present, is used to specify the length of the
 * rqs ``packed' buffer.  The size of the length parameter (short or long) is
 * used to distinguish between varLenByteArray and shortVarLenByteArry.  Those
 * functions that have the length parameter pack it's value in prior to the
 * acutal data values.  In each case the return value of the routine is the
 * next address to pack/unpack into / out of.
 */
void *Pack(void * buffer, const unsigned char & src);
void *Pack(void * buffer, const signed char & src);
void *Pack(void * buffer, const unsigned short & src);
void *Pack(void * buffer, const signed short & src);
void *Pack(void * buffer, const unsigned long & src);
void *Pack(void * buffer, const signed long & src);
void *Pack(void * buffer, const ndr_hyper_int_rep & src);
void *Pack(void * buffer, const ndr_uhyper_int_rep & src);
void *Pack(void * buffer, const float & src);
void *Pack(void * buffer, const double & src);
void *Pack(void * buffer, const unsigned char * const src, 
	   const unsigned long len, 
	   const int = 0);
void *PackString(void * buffer, const unsigned char* s, 
		 const unsigned long maxLen);
void *PackStringKey(void * buffer, const unsigned char* s,
		 const unsigned long maxLen);
void * Unpack(const void * const buffer, unsigned char & src);
void * Unpack(const void * const buffer, signed char & src);
void * Unpack(const void * const buffer, unsigned short & src);
void * Unpack(const void * const buffer, signed short & src);
void * Unpack(const void * const buffer, unsigned long & src);
void * Unpack(const void * const buffer, signed long & src);
void * Unpack(const void * const buffer, ndr_hyper_int_rep & src);
void * Unpack(const void * const buffer, ndr_uhyper_int_rep & src);
void * Unpack(const void * const buffer, float & src);
void * Unpack(const void * const buffer, double & src);
void * Unpack(const void * const buffer, char * src,
	      idl_ulong_int* returnLength,
	      const unsigned long length,
	      const int IncludeLength = 0);
void * Unpack(const void * const buffer, char * src,
	      const unsigned long maxLen);
#endif

