    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef RQSUTILDEFINED
#define RQSUTILDEFINED

extern "C"
{
#include <pthread.h>
}
#include <Map.h>
#include <String.h>
#include <List.h>
#include <hyperint.H>
extern "C"
    {
#include <rqs/rqs.h>
    }

inline void Assign(rqs_unsignedInt64_t & dst , ndr_uhyper_int_rep & src)
    {
	dst.high = src.high;
	dst.low = src.low;
    }
inline void Assign(rqs_signedInt64_t & dst , ndr_hyper_int_rep & src)
    {
	dst.high = src.high;
	dst.low = src.low;
    }
inline void Assign(ndr_uhyper_int_rep & dst , rqs_unsignedInt64_t & src)
    {
	dst.high = src.high;
	dst.low = src.low;
    }
inline void Assign(ndr_hyper_int_rep & dst , rqs_signedInt64_t & src)
    {
	dst.high = src.high;
	dst.low = src.low;
    }

class RqsBinding
    {	/* manage the binding to the rqs server */
public:
	RqsBinding();
	RqsBinding(const String & newServer);
	~RqsBinding();
	int Bound() const; /* predicate, 0 == unbound, else == bound */
	void Boundless() ; /* bind,sleep,retry until bound */
	void SetServer(const String & newServer);// set server name and bind
						 //to it
	friend ostream & operator<<(ostream &, const RqsBinding &);
	rqs_serverHandle_t Bind(const String &);
	    /* lookup server, if it's not found then bind to it and add an
	     * entry to the map.  returns 0 on failure */
	const String Find(const rqs_serverHandle_t) const;
	    /* lookup handle and return server name.  returns empty string if
	     * not found. */
	void Remove(const String &);
	void Remove(const rqs_serverHandle_t);
	const rqs_serverHandle_t Handle() const { return handle; }
	const String Server() const { return server; }
private:
	rqs_serverHandle_t handle;
	String server;
	static Map<String, rqs_serverHandle_t>* bindings;

    };

class YQelementID 
    {
public:
	YQelementID();
	YQelementID(const rqs_elementId_t & eid) {id=eid;}
	YQelementID(const String & src);
	YQelementID & operator=(const String & src);
	YQelementID & operator=(const rqs_elementId_t& i) { id = i;
                                               return *this;}
	int operator == (const YQelementID& i) const
		{ int n = sizeof id;
                  return memcmp(&id, &i.id, n); }
	operator String() const;
	friend ostream & operator << (ostream & o, const YQelementID & src) ;
	virtual ~YQelementID();
private:
	rqs_elementId_t id;
    };

#endif
