/*
 * sfsbases:
 *
 *    This header file contains a few classes used internally by the
 * CDL-generated SFS code. These base classes abstract a particular SFS
 * data type. CDL generates subclasses of these classes that "know" how
 * to put the proper data into the sfs structs to represent a particular
 * type of SFS record.
 *
 * sfsbases.H,v 1.5 1994/04/07 19:31:21 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef SFS_BASES_H
#define SFS_BASES_H

extern "C"
{
#include <pthread.h>
}
#include <String.h>
extern "C" {
#include <sfs/sfs.h>
}
#include <cdl/cdlerrors.H>

class SFSBaseRecord

{
public:
  SFSBaseRecord();
  virtual ~SFSBaseRecord();
  sfs_record_t* Copy() const;			// Copy the contents
  static void Destroy(sfs_record_t*);		// Destroy that copy
  CDLStatus CheckPacking(unsigned long len);	// Check packing
  CDLStatus CheckUnpacking(unsigned long len);	// Check unpacking results	
  operator const sfs_record_t*() const;		// Return pointer to contents
  operator sfs_record_t*();
protected:
  sfs_record_t rec;
};

class SFSBaseKey

{
public:
  SFSBaseKey();
  virtual ~SFSBaseKey();
  const String& Name() const;			// Return reference to name
  sfs_key_t* Copy() const;			// Copy contents
  static void Destroy(sfs_key_t*);		// Destroy that copy
  CDLStatus CheckPacking(unsigned long len);	// Check packing
  operator const sfs_key_t*() const;		// Return pointer to contents
  operator sfs_key_t*();
protected:
  sfs_key_t key;
  String indexName;
};

class SFSBaseSecondary

{
public:
  SFSBaseSecondary();
  virtual ~SFSBaseSecondary();
  const String& Name() const;
  operator const sfs_secondaryIndexSpec_t*() const;
  operator sfs_secondaryIndexSpec_t*();
protected:
  sfs_secondaryIndexSpec_t spec;
  String indexName;
};

class SFSBaseFileSpec

{
public:
  SFSBaseFileSpec();
  SFSBaseFileSpec(const String& typeName);
  virtual ~SFSBaseFileSpec();
  const String& TypeName() const;
  operator const sfs_fileSpec_t*() const;
  operator sfs_fileSpec_t*();
protected:
  sfs_fileSpec_t spec;
  String typeName;
};

inline SFSBaseRecord::operator sfs_record_t*()
     
{ return &rec; }

inline SFSBaseRecord::operator const sfs_record_t*() const
     
{ return &rec; }

inline SFSBaseKey::operator sfs_key_t*()

{ return &key; }

inline SFSBaseKey::operator const sfs_key_t*() const

{ return &key; }

inline const String& SFSBaseKey::Name() const

{ return indexName; }

inline SFSBaseSecondary::operator const sfs_secondaryIndexSpec_t*() const
     
{ return &spec; }

inline SFSBaseSecondary::operator sfs_secondaryIndexSpec_t*()
     
{ return &spec; }

inline const String& SFSBaseSecondary::Name() const

{ return indexName; }

inline const String& SFSBaseFileSpec::TypeName() const

{ return typeName; }

inline SFSBaseFileSpec::operator const sfs_fileSpec_t*() const
     
{ return &spec; }

inline SFSBaseFileSpec::operator sfs_fileSpec_t*()
     
{ return &spec; }

#endif
