/*
 * sfsbatching.H:
 *    Classes used to build lists of SFS operations to batch. 
 * The SFSBatchBase class
 * encapsulates the sfs_batchRequest_t structure, which is just a union.
 * The sfs_key_t* and sfs_record_t* members of the union have data buffers
 * that point to the original CDL-generated objects, so users must not
 * delete their objects before the batch completes.
 *    The SFSBatchQueue object contains a list of SFSBatchBases and a
 * tid, and is used as a per-transaction batch queue.
 *
 * sfsbatching.H,v 1.3 1994/04/07 19:31:24 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_SFS_BATCHING_H
#define CDL_SFS_BATCHING_H

extern "C" 
{
#include <pthread.h>
#include <sfs/sfs.h>
}
#include <iostream.h>
#include <List.h>
#include <objtran/transaction.H>

class SFSBaseRecord;
class SFSBaseKey;

class SFSBatchBase

{
public:
  SFSBatchBase();
  virtual ~SFSBatchBase();
  operator sfs_batchRequest_t*();
protected:
  sfs_batchRequest_t batchRep;
};

inline SFSBatchBase::operator sfs_batchRequest_t*()

{ return &batchRep; }

class SFSBatchQueue

{
public:
  SFSBatchQueue();
  SFSBatchQueue(const Transaction&);
  SFSBatchQueue(const SFSBatchQueue&);
  ~SFSBatchQueue();
  SFSBatchQueue& operator = (const SFSBatchQueue&);
  int operator == (const SFSBatchQueue&) const;

  tran_tid_t tid;
  int batching;
  List_of_p<SFSBatchBase> queue;
};

class SFSBatchInsert : public SFSBatchBase

{
public:
  SFSBatchInsert(sfs_ofd_t, SFSBaseRecord&);
  virtual ~SFSBatchInsert();
};

class SFSBatchDelete : public SFSBatchBase

{
public:
  SFSBatchDelete(sfs_ofd_t, const char* indexName, SFSBaseKey&);
  virtual ~SFSBatchDelete();
};

class SFSBatchRead : public SFSBatchBase

{
public:
  SFSBatchRead(sfs_ofd_t, const char* indexName, SFSBaseKey&,
	       sfs_lockMode_t, SFSBaseRecord&);
  virtual ~SFSBatchRead();
};

class SFSBatchUpdate : public SFSBatchBase

{
public:
  SFSBatchUpdate(sfs_ofd_t, const char* indexName, SFSBaseKey&, SFSBaseRecord&);
  virtual ~SFSBatchUpdate();
};

class SFSBatchModifyField : public SFSBatchBase

{
public:
  SFSBatchModifyField(sfs_ofd_t, const char*, SFSBaseKey&, const char* fieldName,
		      sfs_modifyOp_t, unsigned int incLock, 
		      sfs_pointer_t value, sfs_pointer_t newValue,
		      unsigned long fieldLen);
  virtual ~SFSBatchModifyField();
};
		
#endif
