/*
 * SFSFileBase
 *
 *    This class does the real work involved with SFS. The CDL-generated 
 * code puts the appropriate values in the classes defined by sfsbases.H,
 * so here we need only deal with nearly-direct calls to SFS.
 *    To avoid name space conflicts that would occur if we gave these
 * classes more reasonable names that have already been used, we start the
 * names of functions in this class with "I", for "Internal".
 *
 * sfsfilebase.H,v 1.9 1994/04/07 19:31:28 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef SFS_FILE_BASE_H
#define SFS_FILE_BASE_H

extern "C"
{
#include <pthread.h>
}
#include <cdl/cdlfilebase.H>
#include <cdl/cdlbasedefs.H>
#include <cdl/cdlerrors.H>
#include <objtran/transaction.H>
#include <objtran/tranmember.H>
#include <objdce/mutex.H>
#include <cdl/sfsbases.H>
#include <cdl/sfsbatching.H>
#include <objdce/transignals.H>
#include <String.h>
#include <Set.h>
#include <Path.h>
#include <List.h>
extern "C" {
#include <tran/tran.h>
#include <sfs/sfs.h>
}

/* 
 * SFSTranInfo
 *    A simple class that stores a tid, an ofd and info on batched requests.
 */

class SFSTranInfo

{
public:
  SFSTranInfo();
  SFSTranInfo(const Transaction&);
  SFSTranInfo(const SFSTranInfo&);
  SFSTranInfo& operator = (const SFSTranInfo&);
  int operator == (const SFSTranInfo&) const;

  tran_tid_t tid;
  sfs_ofd_t ofd;
};
  
class SFSFileBase : public CDLFileBase, 
		    public TransactionMember, public Terminated

{
protected:
  SFSFileBase();
public:
  virtual ~SFSFileBase();
  static CDLStatus IStartBatch();
  static CDLStatus IEndBatch();
protected:
  CDLStatus IOpenFile(const String& dbName, const String& fileName,
		      const String& typeName, unsigned short flags,
		      Boolean noNameCheck = false);
  CDLStatus ICloseFile();
  CDLStatus ICreate(SFSBaseFileSpec&, const String& dbName, 
		    const String& fileName, const String& volName, 
		    unsigned long numBytes);
  CDLStatus ISelect(const String& indexName, SFSBaseKey* start,
		    SFSBaseKey* end, CDLIndexDir direction);
  CDLStatus IInsert(SFSBaseRecord&);
  CDLStatus IReadCurrent(CDLReadLock, SFSBaseRecord&, CDLRecordDir = NEXT);
  CDLStatus IReadUnique(CDLReadLock, SFSBaseKey&, SFSBaseRecord& out);
  CDLStatus IDeleteCurrent(CDLRecordDir = NEXT);
  CDLStatus IDeleteUnique(SFSBaseKey&);
  CDLStatus IUpdateCurrent(SFSBaseRecord&, CDLRecordDir = NEXT);
  CDLStatus IUpdateUnique(SFSBaseKey&, SFSBaseRecord&);
  CDLStatus IModifyField(SFSBaseKey&, const String& fieldName,
			 sfs_pointer_t val, sfs_pointer_t out, 
			 unsigned long length);
  CDLStatus IIncrementField(SFSBaseKey&, const String& fieldName,
			    sfs_pointer_t val, sfs_pointer_t out, 
			    unsigned long length, 
			    Boolean incrementLock = false);
  CDLStatus IGetEsn(sfs_esn_t& esn);
  CDLStatus ISaveContext(void** context);
  CDLStatus IRestoreContext(void* context);
  CDLStatus IActivateIndex(SFSBaseSecondary&, const String& volume,
			   SFSBaseKey* nullKey = (SFSBaseKey*)0);
  CDLStatus IDeactivateIndex(SFSBaseSecondary&);
  CDLStatus IDestroyIndex(SFSBaseSecondary&);
  CDLStatus IDestroyFile();
  CDLStatus IEmptyFile();

  static Boolean NoAbort(CDLStatus);
  static Boolean RetryError(const sfs_status_t);
  static CDLStatus CheckError(const int msgNum, const sfs_status_t, 
			      Transaction&, ErrorLevels::ReportingLevels level);
  static CDLStatus ThrowError(const int msgNum, CDLStatus, 
			      Transaction&, ErrorLevels::ReportingLevels level);
  static CDLStatus ConvertStatus(const sfs_status_t);
  CDLStatus CheckOpen(Boolean, Transaction&);
  CDLStatus GetOfd(Transaction&, sfs_ofd_t&);
  static Boolean IsBatching(const Transaction&);
  static Boolean AddBatch(const Transaction&, const SFSBatchBase*);
  sfs_selector_t GetDirection(CDLRecordDir) const;
  sfs_lockMode_t GetMode(CDLReadLock) const;
  static void FirstTimeInit();
public:
  virtual void FinishedCallback(const Transaction&);
  virtual void HandleTermination();
protected:
  List<sfs_ofd_t> freeOfds;
  Set<SFSTranInfo> activeTrans;
  static Set<SFSBatchQueue>* batchQueues;
  static pthread_once_t initedBatches;
  static MutexLock* batchLatch;
  sfs_ofd_t masterOfd;
  MutexLock ofdLatch;
  Boolean opened;
  unsigned short openFlags;
  unsigned long timeout;
  sfs_isolationLevel_t isolation;
};  

#endif
