/*
 * dbtest
 *
 *    This program tests out the SFS database we've created. It's simple,
 * and I like it that way.
 *
 * dbtest.C,v 1.7 1994/04/07 19:28:27 gjb Exp
 */

#include <objdce/statics.H>
#include <objdce/error.H>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objtran/tranclient.H>
#include <dbtest_cdl.H>
#include <dbtest_cdl_sfs.H>

static const char* const NameDir = "";
static const char* const Volume = "lvol_main";
static const unsigned long InitialRecords = 32;

static void CreateDB(const char* serverName, const char* fName)

{
  TranClient myClient;
  SFSTestFile file;
  NewTransaction tran;

  file.CreateFile(serverName, fName, Volume, InitialRecords);
  tran.Commit();
}

static void RunDB(const char* serverName, const char* fName)

{
  char input[255];
  char command;
  String key;
  double doubleVal;
  long longVal;
  char charVal;
  String textVal;
  Transaction* trans[8];
  int nesting;
  Boolean openedOK;
  int inc;
  String errMsg;
  CDLStatus cdlErr;
  OtherType otherVal;

  SFSTestFile theFile;
  Test* protRec;
  Test* protKey;
  SmallTest* smallRec;
  TranClient myClient;
  List<Test> allRecs;
  Listiter<Test> allRecsIter(allRecs);

  NewTransaction* openTran = new NewTransaction();
  cdlErr = theFile.OpenFile(serverName, fName);
  openTran->Commit();
  if (cdlErr != CDL_SUCCESS)
    {
      CDLStatusToString(cdlErr, errMsg);
      printf("%s\n", (const char*)errMsg);
      openedOK = false;
    }
  else
    openedOK = true;
  delete openTran;
    
  command = '\0';
  if (openedOK)
    {
      nesting = 0;
      while (command != 'q')
	{
	  if (nesting == 0)
	    printf("[<B>egin <Q>uit] ");
	  else if (nesting < 8)
	    printf("[<A>dd <M>odify <D>elete <G>et <L>ist <S>ort <E>mpty <B>egin Abor<T> <C>ommit <Q>uit] ");
	  else
	    printf("[<A>dd <M>odify <D>elete <G>et <L>ist <S>ort <E>mpty Abor<T> <C>ommit <Q>uit] ");
	  gets(input);
	  command = input[0];
	  tolower(command);
	  if ((command == 'a') || (command == 'm') || 
	      (command == 'd') || (command == 'g'))
	    {
	      printf("Key: ");
	      gets(input);
	      key = input;
	    }
	  switch (command)
	    {
	    case 'b':				// Begin a (nested) transaction
	      trans[nesting++] = new NewTransaction();
	      break;
	    case 'c':				// Commit the transaction
	      trans[--nesting]->Commit();
	      delete trans[nesting];
	      break;
	    case 't':				// Abort the transaction
	      trans[--nesting]->Abort("User aborted transaction", 2, 1);
	      delete trans[nesting];
	      break;
	    case 'a':				// Add a record
	      protRec = new Test();
	      smallRec = new SmallTest();
	      protRec->key(key);
	      printf("Amount (real): ");
	      gets(input);
	      sscanf(input, "%lf", &doubleVal);
	      protRec->amount(doubleVal);
	      printf("Other (long): ");
	      gets(input);
	      sscanf(input, "%li", &longVal);
	      otherVal = (OtherType)longVal;
	      protRec->other(otherVal);
	      printf("Flag (char): ");
	      gets(input);
	      protRec->flag(input[0]);
	      printf("someText (string): ");
	      gets(input);
	      smallRec->someText(input);
	      printf("stuff (varBytes): ");
	      gets(input);
	      smallRec->variableStuff((unsigned char*)input, strlen(input) + 1);
	      protRec->little(*smallRec);
	      printf("otherText (string): ");
	      gets(input);
	      protRec->otherText(input);
	      for (inc = 0; inc < 10; inc++)
		protRec->oldAmounts(inc, 0.0);
	      allRecs.put(*protRec);
	      cdlErr = theFile.InsertList(allRecs);
	      if (cdlErr != 0)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      allRecsIter.reset();
	      while (allRecsIter.remove_next());
	      if (trans[nesting - 1]->Aborted())
		{
		  delete trans[--nesting];
		}
	      delete protRec;
	      delete smallRec;
	      break;
	    case 'l':			// List all the records
	      protRec = new Test();
	      cdlErr = theFile.ReadAll(BRACKET, TestFile::PRIMARY, (Test*)0,
				       (Test*)0, FORWARDS, allRecs);
	      if (cdlErr != CDL_SUCCESS)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      allRecsIter.reset();
	      while (allRecsIter.remove_next(*protRec))
		{
		  key = protRec->key();
		  printf("Key: %s    ", (const char*)key);
		  printf("Amount: %lf\n", protRec->amount());
		}
	      delete protRec;
	      break;
	    case 's':
	      protRec = new Test();
	      cdlErr = theFile.ReadAll(BRACKET, TestFile::AMOUNT, (Test*)0,
				       (Test*)0, FORWARDS, allRecs);
	      if (cdlErr != CDL_SUCCESS)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      allRecsIter.reset();
	      while (allRecsIter.remove_next(*protRec))
		{
		  printf("Key: %s    ", (const char*)(protRec->key()));
		  printf("Amount: %lf    ", protRec->amount());
		  printf("Flag: %c\n", protRec->flag());
		}
	      delete protRec;
	      break;
	    case 'm':			// Modify a record
	      protRec = new Test();
	      protKey = new Test();
	      printf("Amount (real): ");
	      gets(input);
	      sscanf(input, "%lf", &doubleVal);
	      protRec->key(key);
	      protKey->key(key);
	      cdlErr = theFile.ReadUnique(READ, TestFile::PRIMARY, 
					  *protKey, *protRec);
	      if (cdlErr != CDL_SUCCESS)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      if (trans[nesting - 1]->Aborted())
		{
		  delete trans[--nesting];
		}
	      else
		{
		  for (inc = 10; inc >= 1; inc--)
		    protRec->oldAmounts(inc, protRec->oldAmounts(inc - 1));
		  protRec->oldAmounts(0, protRec->amount());
		  protRec->amount(doubleVal);
		  cdlErr = theFile.UpdateUnique(TestFile::PRIMARY, 
						*protKey, *protRec);
		  if (cdlErr != CDL_SUCCESS)
		    {
		      CDLStatusToString(cdlErr, errMsg);
		      printf("%s\n", (const char*)errMsg);
		    }
		  if (trans[nesting - 1]->Aborted())
		    {
		      delete trans[--nesting];
		    }
		}
	      delete protKey;
	      delete protRec;
	      break;
	    case 'd':			// Delete a record
	      protKey = new Test();
	      protKey->key(key);
	      cdlErr = theFile.DeleteUnique(TestFile::PRIMARY, *protKey);
	      if (cdlErr != CDL_SUCCESS)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      if (trans[nesting - 1]->Aborted())
		{
		  delete trans[--nesting];
		}
	      delete protKey;
	      break;
	    case 'e':
	      cdlErr = theFile.DeleteAll(TestFile::PRIMARY, 
					 (Test*)0, (Test*)0);
	      if (cdlErr != CDL_SUCCESS)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      if (trans[nesting - 1]->Aborted())
		{
		  delete trans[--nesting];
		}
	      break;
	    case 'g':			// Retrieve a record
	      protKey = new Test();
	      protRec = new Test();
	      protKey->key(key);
	      cdlErr = theFile.ReadUnique(READ, TestFile::PRIMARY, 
					  *protKey, *protRec);
	      if (cdlErr != CDL_SUCCESS)
		{
		  CDLStatusToString(cdlErr, errMsg);
		  printf("%s\n", (const char*)errMsg);
		}
	      if (trans[nesting - 1]->Aborted())
		{
		  delete trans[--nesting];
		}
	      else
		{
		  key = protRec->key();
		  printf("Key: %s\n", (const char*)key);
		  doubleVal = protRec->amount();
		  printf("Amount: %lf\n", doubleVal);
		  for (inc = 0; inc < 10; inc++)
		    {
		      doubleVal = protRec->oldAmounts(inc);
		      printf(" Old amount %i: %lf\n", inc, doubleVal);
		    }
		  otherVal = protRec->other();
		  printf("Other: %i\n", (int)otherVal);
		  printf("otherText: %s\n", (const char*)(protRec->otherText()));
		  smallRec = new SmallTest(protRec->little());
		  printf("someText: %s\n", (const char*)(smallRec->someText()));
		  printf("Stuff: %s\n", (char*)(smallRec->variableStuff()));
		  printf("Stuff length: %li\n", smallRec->variableStuffLength());
		  charVal = protRec->flag();
		  printf("Flag: %c\n", charVal);
		  delete smallRec;
		}
	      delete protRec;
	      delete protKey;
	      break;
	    case 'q':			// Quit
	      for (inc = nesting - 1; inc >= 0; inc--)
		{
		  trans[inc]->Abort("Program ended", -1, 1);
		  delete trans[inc];
		}
	      break;
	    default:
	      printf("Incorrect command chosen\n");
	      break;
	    }
	}
      cdlErr = theFile.CloseFile();
      if (cdlErr != CDL_SUCCESS)
	{
	  CDLStatusToString(cdlErr, errMsg);
	  printf("%s\n", (const char*)errMsg);
	}
    }
}

int main (int argc, char** argv)

{
  if ((argc != 4) || (argv[1][0] != '-'))
    {
      printf("Usage: dbtest -c <DB name> <file name> -- create the test database\n");
      printf("       dbtest -r <DB name> <file name> -- do stuff with the test database\n");
      return 1;
    }
  else
    {
      switch (argv[1][1])
	{
	case 'c':
	  CreateDB(argv[2], argv[3]);
	  break;
	case 'r':
	  RunDB(argv[2], argv[3]);
	  break;
	default:
	  printf("Incorrect choice.\n");
	  break;
	}
      return 0;
    }
}


