/*
 * analyze:
 *    Read through the log files produced by the TPC driver, gather
 * statistics, and return 'em to the users.
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <string.h>
#include <iostream.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <logging.H>
#include <objdce/defs.H>

/*
 * times: Array of transaction counts. Index is number of tenths of seconds a
 * transaction took, and value is the number that took that much time.
 * Transactions that take more than 10 seconds go in slot 101.
 */
 
unsigned long count;
unsigned long badCount;
unsigned long fastCount;
unsigned long times[102];
double totalResTime;
long startTime;
long stopTime;
struct stat statBuf;
int logFile;

void PrintGraph()

{
    int inc;
    float timeItTook;

    cout.precision(4);
    for (inc = 0; inc < 101; inc++)
    {
	timeItTook = inc / 10.0;
	cout << timeItTook << " " << times[inc] << " ";
    }
}

void PrintResults()

{
    int inc;
    long elapsedTime = stopTime - startTime;
    float timeItTook;
    float timePerTran;
    double avgResTime;
    float concurrency;
    float lessTwo;

    cout.precision(6);
    cout << count << " transactions completed in " << endl;
    cout << elapsedTime / 60 << " minutes and " << elapsedTime % 60;
    cout  << " seconds." << endl;
    cout << badCount << " transactions failed.\n" << endl;
    timePerTran = (float)count / (float)elapsedTime;
    cout << "Average: " << timePerTran << " tpsB" << endl;
    avgResTime = totalResTime / (double)count;
    cout << "Average residency: " << avgResTime << " seconds." << endl;
    lessTwo = ((float)fastCount / (float)count) * 100.0;
    cout << lessTwo << " percent completed in less than two seconds." << endl;
    concurrency = timePerTran * avgResTime;
    cout << "Concurrency: " << concurrency << "\n" << endl;
    cout << "Transaction breakdown:\n" << endl;
    for (inc = 0; inc < 101; inc++)
    {
	if (times[inc] > 0)
	{
	    timeItTook = inc / 10.0;
	    cout << timeItTook << ":\t" << times[inc] << endl;
	}
    }
    cout << "> 10.0:\t" << times[101] << endl;
}

Boolean ScanFile(const char* fileName, Boolean graph)

{
    double elapsed;
    int tenths;
    LogItem lItem;
    Boolean done;
    Boolean didFile;
    int inc;

    if (stat(fileName, &statBuf) != 0)
    {
	perror("Couldn't stat file");
	return false;
    }
    else
    {
	for (inc = 0; inc < 102; inc++)
	    times[inc] = 0;
	logFile = open(fileName, O_RDONLY);
	count = 0;
	badCount = 0;
	done = false;
	while (!done)
	{
	    startTime = 0;
	    stopTime = 0;
	    totalResTime = 0.0;
	    fastCount = 0;
	    didFile = false;
	    while (!didFile && !done)
	    {
		if (read(logFile, (char*)&lItem, sizeof(LogItem)) 
		    != sizeof(LogItem))
		    done = true;
		else
		{
		    switch(lItem.status)
		    {
		    case LogFailed:
			badCount++;
			break;
		    case LogCommitted:
			count++;
			elapsed = (double)(lItem.time);
			elapsed += ((double)(lItem.millitm) / 1000);
			totalResTime += elapsed;
			tenths = (int)(elapsed * 10);
			if (tenths > 100)
			    times[101]++;
			else
			times[tenths]++;
			if (tenths < 20)
			    fastCount++;
			break;
		    case LogStart:
			startTime = lItem.time;
			break;
		    case LogStop:
			stopTime = lItem.time;
			didFile = true;
			break;
		    default:
			break;
		    }
		}
	    }
	    if (!done)
	    {
		if (!graph)
		{
		    PrintResults();
		}
		else
		{
		    PrintGraph();
		    done = true;
		}
	    }
	}
	close(logFile);
	return true;
    }
}

int main (int argc, char** argv)

{
    if (argc == 2)
    {
	ScanFile(argv[1], false);
	return 0;
    }
    else if ((argc == 3) && !strcmp(argv[1], "g"))
    {
	ScanFile(argv[2], true);
	return 0;
    }
    else
    {
	cout << "Usage: tpcanalyze [g] <log file>" << endl;
	return -1;
    }
}
