/*
 * driver.H:
 *    Classes used by the tpcB driver.
 *
 */

#ifndef TPC_DRIVER_H
#define TPC_DRIVER_H

#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <objdce/thread.H>
#include <params.H>
#include <curses.h>
#include <logging.H>

class Binding; 
class MutexLock;
class Condition;
class CurrentLoginContext;

static const int SleepTime = 1;

class Driver

{
public:
    Driver(const int serverNo);
    virtual ~Driver();
    void UserScreen(const int rThreads, 
		    const int wThreads, Boolean limited = false);
    void Update(LogItem*);
    void SpawnThread(Boolean readOnly = false,
		     int limit = 0);
    void KillThread();
    void ToggleLog();
    int Count() const;
    Boolean Logging() const;
    int NumThreads() const;
    char Command();
    static Boolean AllDone();
    static void SetDone(Boolean);
private:
    static Boolean allDone;
    void CleanUp();
    CurrentLoginContext* context;
    Binding** allSuts;
    Boolean useAllSuts;
    unsigned long count;
    Boolean logging;
    int logFile;
    AsyncThread* threadList[MAX_THREADS];
    ThreadObject* threadObjs[MAX_THREADS];
    ThreadObject* painter;
    AsyncThread* paintThread;
    int numThreads;
    MutexLock* countLatch;
    WINDOW* screen;
    char command;
    int serverNo;
  };

inline Boolean Driver::AllDone()

{ return allDone; }

class RunThread : public ThreadObject

{
public:
    RunThread(Driver*, Binding**, Boolean useAll, int number,
	      Boolean readOnly = false, int limit = 0);
    virtual ~RunThread();
    virtual void* Run();
private:
    Driver* driver;
    Binding** suts;
    Boolean useAllSuts;
    Boolean readOnly;
    int limit;
    int threadNo;
/*
    urand* genAccount;
    urand* genMyAccount;
    urand* genTeller;
    randint* gen;
    urand* genDelta;
 */
};

class PaintThread : public ThreadObject

{
public:
    PaintThread(Driver*);
    virtual ~PaintThread();
    virtual void* Run();
private:
    Driver* driver;
    MutexLock* paintLatch;
};

inline int Driver::Count() const

{ return count; }

inline Boolean Driver::Logging() const

{ return logging; }

inline int Driver::NumThreads() const

{ return numThreads; }

#endif
