/*
 * params.C:
 *
 *    A file that scans the "tpcParams" file and builds a data structure
 * from that info.
 *
 * params.C,v 1.1 1993/01/13 21:28:06 gjb Exp
 */

#include <params.H>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#ifdef __hpux
extern "C" double drand48();
extern "C" void srand48(long);
#endif

static ServerParams allParams[MAX_PARAMS];		// All parameters
static int NumParams;					// Number of params we have
ServerParams* Params;					// The ones for this one
ServerParams* TotalParams;				// The totals
static ServerParams totals;
static double** randTable;				// Tables of random #s	
static long* curRands;					// Position in each table

/*
 * The tpcParams file: cryptic, because I don't have time for anything
 * more complex. The format:
 *
 * name sfsServer firstAccount accounts firstTeller tellers firstBranch branches
 *
 * Example:
 *
 * /.:/servers/tpc/one /.:/encina/sfs/main 0 500000 0 50 0 5
 * /.:/servers/tpc/two /.:/encina/sfs/two 500000 500000 50 50 5 5
 * /.:/servers/tpc/three /.:/encina/sfs/main 1000000 500000 100 50 10 5
 *
 * Defines a database of three servers with 1,500,000 accounts and 15 branches.
 *
 */

void ScanParams(int serverNo)

{
  FILE* paramFile;
  int curParam;
  ServerParams* param;

  paramFile = fopen(ParamsFile, "r");
  if (paramFile == NULL)
    {
      perror("Couldn't open parameters file");
      exit(-1);
    }
  memset((void*)(&totals), sizeof(ServerParams), 0);
  curParam = 0;
  while ((curParam < MAX_PARAMS) && !feof(paramFile))
    {
      param = &(allParams[curParam]);
      fscanf(paramFile, "%s%s%li%li%li%li%li%li\n", param->tpcServerName,
	     param->serverName, &(param->startAccount), 
	     &(param->numAccounts), &(param->startTeller),
	     &(param->numTellers), &(param->startBranch), 
	     &(param->numBranches));
      param->endAccount = param->startAccount + param->numAccounts - 1;
      param->endTeller = param->startTeller + param->numTellers - 1;
      param->endBranch = param->startBranch + param->numBranches - 1;
      totals.numAccounts += param->numAccounts;
      totals.numTellers += param->numTellers;
      totals.numBranches += param->numBranches;
      curParam++;
    }
  if (curParam == MAX_PARAMS)
    {
      fprintf(stderr, "Too many entries in parameters file\n");
      exit(-1);
    }
  NumParams = curParam;
  totals.startAccount = allParams[0].startAccount;
  totals.startTeller = allParams[0].startTeller;
  totals.startBranch = allParams[0].startBranch;
  totals.endAccount = allParams[NumParams - 1].endAccount;
  totals.endTeller = allParams[NumParams - 1].endTeller;
  totals.endBranch = allParams[NumParams - 1].endBranch;
  Params = &(allParams[serverNo]);
  TotalParams = &totals;
  fclose(paramFile);
}

int GetNumServers()

{
  return NumParams;
}

char* GetServer(int serverNo)

{
  return allParams[serverNo].serverName;
}

char* GetTPCServer(int serverNo)

{
  return allParams[serverNo].tpcServerName;
}

int FindAccount(unsigned long accountNo)

{
  for (int curParam = 0; 
       (curParam < NumParams) && (accountNo > allParams[curParam].endAccount);
       curParam++);
  if (curParam < NumParams)
    return curParam;
  else
    return -1;
}
  
int FindTeller(unsigned long tellerNo)

{
  for (int curParam = 0;
       (curParam < NumParams) && (tellerNo > allParams[curParam].endTeller);
       curParam++);
  if (curParam < NumParams)
    return curParam;
  else
    return -1;
}
  
int FindBranch(unsigned long branchNo)

{
  for (int curParam = 0; 
       (curParam < NumParams) && (branchNo > allParams[curParam].endBranch);
       curParam++);
  if (curParam < NumParams)
    return curParam;
  else
    return -1;
}
  
int ScanRandoms(int numThreads)

{
  int curThread;
  long curNo;
  long randSeed;
  
  randSeed = time(NULL);
  srand48(randSeed);
  randTable = new double*[numThreads];
  curRands = new long[numThreads];
  fprintf(stderr, "Generating random numbers for %li ", randsPerThread);
  fprintf(stderr, " values in %i threads.\n", numThreads);
  for (curThread = 0; curThread < numThreads; curThread++)
    {
      fprintf(stderr, "%i...", curThread);
      randTable[curThread] = new double[randsPerThread];
      for (curNo = 0; curNo < randsPerThread; curNo++)
	randTable[curThread][curNo] = drand48();
      curRands[curThread] = 0;
    }
  fprintf(stderr, "Done.\n");
  return 0;
}

double GetRandom(int threadNo)

{
  if (curRands[threadNo] == randsPerThread)
    curRands[threadNo] = 0;
  return (randTable[threadNo][curRands[threadNo]++]);
}
