/*
 * params.H: Parameters for the whole TPC system
 */

#ifndef TPC_PARAMS_H
#define TPC_PARAMS_H

static const int BranchSize = 100000;	    // Number of accounts per branch
static const int PageSize = 4096;	    // Size of an SFS page
static const double InitialAccount = 1000;	    // Initial account balance
static const double InitialTeller = 10000000;	    // Initial teller "
static const double InitialBranch = 100000000;    // Initial branch "
static const char* const TPCPrincipal = "gjb";
static const char* const ParamsFile = "./tpcParams";
static const char* const ACLDir = "/.:/users/gjb";
static const char* const KeyTable = "./tpc.keys";
static const char* const Principal = "gjb";
static const long randsPerThread = 32768;

#define MAX_THREADS 32
#define MAX_PARAMS 8

class ServerParams
{
public:
  char serverName[255];
  char tpcServerName[255];
  unsigned long startAccount;
  unsigned long endAccount;
  unsigned long numAccounts;
  unsigned long startTeller;
  unsigned long endTeller;
  unsigned long numTellers;
  unsigned long startBranch;
  unsigned long endBranch;
  unsigned long numBranches;
};

extern ServerParams* Params;
extern ServerParams* TotalParams;

char* GetServer(int serverNo);
char* GetTPCServer(int serverNo);
void ScanParams(int serverNo);
int GetNumServers();
int FindAccount(unsigned long accountNo);
int FindTeller(unsigned long tellerNo);
int FindBranch(unsigned long branchNo);
int ScanRandoms(int numThreads);
double GetRandom(int threadNo);

#endif
