/*
 * tpcquery:
 *    This program queries the various TPC databases for various stuff.
 */

#include <objdce/statics.H>
#include <objtran/tranclient.H>
#include <params.H>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strstream.h>
#include <ctype.h>
#include <time.h>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <hyperint.H>
#include <tpcfiles_cdl.H>
#include <tpcfiles_cdl_sfs.H>

static const int DefaultTimeout = 240;

class Querier : public Signaled, public Signaler

{
public:
    Querier(const char*, const char*);
    virtual ~Querier();
    virtual Signaled::SigStatus HandleSignal(const int);
    void DoBranches();
    void DoTellers();
    void DoAccounts();
    void DoHistory();
    void DoMenu();
    void PrintHyper(const HyperInt&, char* out) const;
private:
    TranClient* client;
    SFSAccountFile* accounts;
    SFSTellerFile* tellers;
    SFSBranchFile* branches;
    SFSHistoryFile* history;
    int timeout;
    char input[255];
    char command[2];
    char server[255];
    char volume[255];
};

Querier::Querier(const char* s, const char* vol)

{
    char* timeoutStr = getenv("TPC_TIMEOUT");
    if (timeoutStr == nil)
	timeout = DefaultTimeout;
    else
	timeout = atoi(timeoutStr);
    client = new TranClient();
    CatchSignal(SIGINT, *this);
    CatchSignal(SIGTERM, *this);
    accounts = nil;
    tellers = nil;
    branches = nil;
    history = nil;
    strcpy(server, s);
    strcpy(volume, vol);
}

Querier::~Querier()

{
    HandleSignal(SIGINT);
}

Signaled::SigStatus Querier::HandleSignal(const int number)

{
  if ((number == SIGINT) || (number == SIGTERM))
    {
      cerr << "Closing SFS files..." << endl;
	if (accounts != nil)
	  accounts->CloseFile();
      if (tellers != nil)
	{
	  tellers->DeactivateIndex(TellerFile::BRANCH);
	  tellers->CloseFile();
	}
      if (branches != nil)
	branches->CloseFile();
      if (history != nil)
	history->CloseFile();
      if (accounts != nil)
	delete accounts;
      if (tellers != nil)
	delete tellers;
      if (branches != nil)
	delete branches;
      if (history != nil)
	delete history;
      return Signaled::TERMINATE;
    }
  else
    return Signaled::NOT_HANDLED;
}

void Querier::PrintHyper(const HyperInt& hyper, char* out) const

{
  ostrstream dollarBuf;
  ostrstream pennyBuf;
  HyperInt tmpHyper;
  
  tmpHyper = hyper / (HyperInt)100L;
  dollarBuf << tmpHyper;
  tmpHyper = hyper % (HyperInt)100L;
  pennyBuf << tmpHyper;
  sprintf(out, "%s.%s", dollarBuf.str(), pennyBuf.str());
}

void Querier::DoBranches()

{
    FunctionName("Querier::DoBranches");
    Transaction* tran;
    unsigned long key;
    int count;
    String errMsg;
    Branch branch;
    Branch branchKey;
    Teller teller;
    Teller tellerKey;
    HyperInt bigNum;
    HyperInt total;
    char numStr[255];

    if (branches == nil)
    {
	branches = new SFSBranchFile();
	tran = new NewTransaction();
	branches->OpenFile(server, "tpcBranches");
	tran->Commit();
	delete tran;
    }
    if (tellers == nil)
    {
	tellers = new SFSTellerFile();
	tran = new NewTransaction();
	tellers->OpenFile(server, "tpcTellers", 
			  CDL_EXCLUSIVE_OPEN | CDL_NONTRANSACTIONAL_OPEN);
	tellers->ActivateIndex(TellerFile::BRANCH, volume);
	tran->Commit();
	delete tran;
    }
    command[0] = '\0';
    while (command[0] != 'r')
    {
	printf("[<E>xamine branch; sum <B>ranches; sum <T>ellers <R>eturn] ");
	scanf("%s", input);
	strncpy(command, input, (size_t)1);
	tolower(command[0]);
	switch (command[0])
	{
	case 'e':
	    printf("Which Branch? ");
	    scanf("%s", input);
	    key = atol(input);
	    branchKey.branchID(key);
	    tran = new NewTransaction(timeout);
	    branches->ReadUnique(READ, BranchFile::PRIMARY, branchKey, branch);
	    tran->Commit();
	    if (tran->Aborted())
	    {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	      {
		PrintHyper(branch.balance(), numStr);
		printf("Balance: %s\n", numStr);
		printf("Fill: %s\n", branch.fill());
	    }
	    delete tran;
	    break;
	case 'b':
	    total = 0L;
	    count = 0;
	    tran = new NewTransaction();
	    branches->Select(BranchFile::PRIMARY, nil, nil, FORWARDS);
	    while (!tran->Aborted() && 
		   (branches->ReadCurrent(READ, NEXT, branch) == CDL_SUCCESS))
	    {
		total += branch.balance();
		count++;
	    }
	    tran->Commit();
	    if (tran->Aborted())
	    {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	      {
		PrintHyper(total, numStr);
		printf("%i branches. Total: %s\n", count, numStr);
	    }
	    delete tran;
	    break;
	case 't':
	    printf("Which Branch? ");
	    scanf("%s", input);
	    key = atol(input);
	    tellerKey.branchID(key);
	    total = 0L;
	    count = 0;
	    tran = new NewTransaction();
	    tellers->Select(TellerFile::BRANCH, &tellerKey, 
			    &tellerKey, FORWARDS);
	    while (!tran->Aborted() &&
		   (tellers->ReadCurrent(READ, NEXT, teller) == CDL_SUCCESS))
	    {
		total += teller.balance();
		count++;
	    }
	    tran->Commit();
	    if (tran->Aborted())
	    {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	      {
		PrintHyper(total, numStr);
		printf("%i tellers. Total: %s\n", count, numStr);
	      }
	    delete tran;
	    break;
	case 'r':
	    break;
    	default:
	    printf("Invalid command.\n");
	    break;
	}
    }
}

void Querier::DoTellers()

{
    FunctionName("Querier::DoTellers");
    Transaction* tran;
    unsigned long key;
    int count;
    String errMsg;
    Teller teller;
    Teller tellerKey;
    HyperInt bigNum;
    char numStr[255];

    if (tellers == nil)
    {
	tellers = new SFSTellerFile();
	tran = new NewTransaction();
	tellers->OpenFile(server, "tpcTellers", 
			  CDL_EXCLUSIVE_OPEN | CDL_NONTRANSACTIONAL_OPEN);
	tellers->ActivateIndex(TellerFile::BRANCH, volume);
	tran->Commit();
	delete tran;
    }
    command[0] = '\0';
    while (command[0] != 'r')
    {
	printf("[<E>xamine teller; sum <T>ellers; <R>eturn] ");
	scanf("%s", input);
	strncpy(command, input, (size_t)1);
	tolower(command[0]);
	switch(command[0])
	{
	case 'e':
	    printf("Which teller? ");
	    scanf("%s", input);
	    key = atol(input);
	    tellerKey.tellerID(key);
	    tran = new NewTransaction(timeout);
	    tellers->ReadUnique(READ, TellerFile::PRIMARY, tellerKey, teller);
	    tran->Commit();
	    if (tran->Aborted())
	    {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	    {
	      PrintHyper(teller.balance(), numStr);
	      printf("Branch: %li\n", teller.branchID());
	      printf("Balance: %s\n", numStr);
	      printf("Fill: %s\n", (const char*)(teller.fill()));
	    }
	    delete tran;
	    break;
	case 't':
	    bigNum = 0L;
	    count = 0;
	    tran = new NewTransaction();
	    tellers->Select(TellerFile::PRIMARY, nil, nil, FORWARDS);
	    while (!tran->Aborted() &&
		   (tellers->ReadCurrent(READ, NEXT, teller) == CDL_SUCCESS))
	    {
		bigNum += teller.balance();
		count++;
	    }
	    tran->Commit();
	    if (tran->Aborted())
	    {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	      {
		PrintHyper(bigNum, numStr);
		printf("%i tellers. Total: %s\n", count, numStr);
	      }
	    delete tran;
	    break;
	case 'r':
	    break;
	default:
	    printf("Invalid command.\n");
	    break;
	}
    }
}

void Querier::DoAccounts()

{
    FunctionName("Querier::DoAccounts");
    Transaction* tran;
    unsigned long key;
    unsigned long count;
    String errMsg;
    Account account;
    Account accountKey;
    HyperInt bigNum;
    char numStr[255];

    if (accounts == nil)
    {
	accounts = new SFSAccountFile();
	tran = new NewTransaction();
	accounts->OpenFile(server, "tpcAccounts");
	tran->Commit();
	delete tran;
    }
    command[0] = '\0';
    while (command[0] != 'r')
    {
	printf("[<E>xamine account; sum all <A>ccounts; <R>eturn] ");
	scanf("%s", input);
	strncpy(command, input, (size_t)1);
	tolower(command[0]);
	switch(command[0])
	{
	case 'e':
	    printf("Which account? ");
	    scanf("%s", input);
	    key = atol(input);
	    accountKey.accountID(key);
	    tran = new NewTransaction(timeout);
	    accounts->ReadUnique(READ, AccountFile::PRIMARY, 
				 accountKey, account);
	    tran->Commit();
	    if (tran->Aborted())
	    {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	    {
	      PrintHyper(account.balance(), numStr);
	      printf("Branch: %li\n", account.branchID());
	      printf("Balance: %s\n", numStr);
	    }
	    delete tran;
	    break;
	case 'a':
	    bigNum = 0L;
	    count = 0;
	    tran = new NewTransaction();
	    accounts->Select(AccountFile::PRIMARY, nil, nil, FORWARDS);
	    while (!tran->Aborted() &&
		   (accounts->ReadCurrent(BRACKET, NEXT, account) == CDL_SUCCESS))
	      {
		bigNum += account.balance();
		count++;
	      }
	    tran->Commit();
	    if (tran->Aborted())
	      {
		tran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	      } 
	    else 
	      {
		PrintHyper(bigNum, numStr);
		printf("%li accounts. Total: %s\n", count, numStr );
	      }
	    delete tran;
	    break;
	case 'r':
	    break;
	default:
	    printf("Invalid command.\n");
	    break;
	}
    }
}

void Querier::DoHistory()

{
    FunctionName("Querier::DoHistory");
    Transaction* mainTran;
    unsigned long count;
    long secs;
    String errMsg;
    History hist;
    CDLStatus status;
    HyperInt bigNum;
    char numStr[255];

    if (history == nil)
    {
	history = new SFSHistoryFile();
	mainTran = new NewTransaction(timeout);
	history->OpenFile(server, "tpcHistory");
	mainTran->Commit();
	delete mainTran;
    }
    mainTran = new NewTransaction();
    history->Select(HistoryFile::PRIMARY, nil, nil, FORWARDS);
    command[0] = '\0';
    while (command[0] != 'r')
    {
	printf("[<N>ext; <P>revious; <F>irst; <L>ast; sum <D>eltas; <R>eturn] ");
	scanf("%s", input);
	strncpy(command, input, (size_t)1);
	tolower(command[0]);
	switch(command[0])
	{
	case 'n':
	    status = history->ReadCurrent(BRACKET, NEXT, hist);
	    if (mainTran->Aborted())
	    {
		mainTran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else if (status == CDL_END_OF_RANGE)
	      {
		printf("End of range reached.\n");
	      }
	    else
	    {
	      PrintHyper(hist.delta(), numStr);
	      printf("Account: %li\n", hist.accountID());
	      printf("Teller: %li\n", hist.tellerID());
	      printf("Branch: %li\n", hist.branchID());
	      printf("Delta: %s\n", numStr);
	      secs = hist.seconds();
	      printf("Timestamp: %s", ctime(&secs));
	    }
	    break;
	case 'p':
	    status = history->ReadCurrent(BRACKET, PREVIOUS, hist);
	    if (mainTran->Aborted())
	    {
		mainTran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else if (status == CDL_END_OF_RANGE)
	      {
		printf("End of range reached.\n");
	      }
	    else
	    {
	      PrintHyper(hist.delta(), numStr);
	      printf("Account: %li\n", hist.accountID());
	      printf("Teller: %li\n", hist.tellerID());
	      printf("Branch: %li\n", hist.branchID());
	      printf("Delta: %s\n", numStr);
	      secs = hist.seconds();
	      printf("Timestamp: %s", ctime(&secs));
	    }
	    break;
	case 'd':
	    bigNum = 0L;
	    count = 0;
	    mainTran->Commit();
	    delete mainTran;
	    mainTran = new NewTransaction();
	    history->Select(HistoryFile::PRIMARY, nil, nil, FORWARDS);
	    while (!mainTran->Aborted() &&
		   (history->ReadCurrent(BRACKET, NEXT, hist) == CDL_SUCCESS))
	    {
		bigNum += hist.delta();
		count++;
	    }
	    if (mainTran->Aborted())
	    {
		mainTran->GetAbortReason()->Print(errMsg);
		printf("Error: %s\n", (const char*)errMsg);
	    }
	    else
	    {
	      PrintHyper(bigNum, numStr);
	      printf("%li records. Total: %s\n", count, numStr);
	    }
	    break;
	case 'f':
	    mainTran->Commit();
	    delete mainTran;
	    mainTran = new NewTransaction();
	    history->Select(HistoryFile::PRIMARY, nil, nil, FORWARDS);
	    break;
	case 'l':
	    mainTran->Commit();
	    delete mainTran;
	    mainTran = new NewTransaction();
	    history->Select(HistoryFile::PRIMARY, nil, nil, BACKWARDS);
	    break;
	case 'r':
	    mainTran->Commit();
	    delete mainTran;
	    break;
    	default:
	    printf("Invalid command.\n");
	    break;
	}
    }
}

void Querier::DoMenu()

{
    char theBuf[255];
    command[0] = '\0';
    while (command[0] != 'q')
    {
	printf("[<B>ranches <T>ellers <A>ccounts <H>istory <Q>uit] ");
	scanf("%s", theBuf);
	strncpy(command, theBuf, (size_t)1);
	tolower(command[0]);
	switch (command[0])
	{
	case 'b':
	    DoBranches();
	    break;
	case 't':
	    DoTellers();
	    break;
	case 'a':
	    DoAccounts();
	    break;
	case 'h':
	    DoHistory();
	    break;
	case 'q':
	    break;
	default:
	    printf("Invalid command.\n");
	    break;
	}
    }
}

static void PrintUsage()

{
    printf("Usage: tpcquery <server name> <volume name>\n");
}

int main (int argc, char** argv)

{
    Querier* obj;

    if (argc == 3)
    {
	obj = new Querier(argv[1], argv[2]);
	obj->DoMenu();
	delete obj;
	return 0;
    }
    else
    {
	PrintUsage();
	return -1;
    }
}
