/*
 * params.C:
 *
 *   Common parameters for the benchmarks.
 *
 * params.C,v 1.1 1993/08/09 21:42:07 gjb Exp
 */

#include <params.H>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

/* Constants */

const long NumBranches = 1;
const long AccountsPerBranch = 100000;
const long TellersPerBranch = 10;
const long NumAccounts = AccountsPerBranch * NumBranches;
const long NumTellers = TellersPerBranch * NumBranches;

const char* const AccountFile = "simple_tpc_accounts";
const char* const TellerFile = "simple_tpc_tellers";
const char* const BranchFile = "simple_tpc_branches";

const long InitialAccount = 1000;
const long InitialTeller = 10000000;
const long InitialBranch = 100000000;
const long SleepTime = 5;

static const long RandsPerThread = 32768;

static double** randTable;		// Table of random numbers
static long* curRands;			// Our position in this table

#ifdef __hpux
extern "C" double drand48();
extern "C" void srand48(long);
#endif

int ScanRandoms(int numThreads)

{
  int curThread;
  long curNo;
  long randSeed;
  
  randSeed = time(NULL);
  srand48(randSeed);
  randTable = new double*[numThreads];
  curRands = new long[numThreads];
  fprintf(stderr, "Generating random numbers for %li ", RandsPerThread);
  fprintf(stderr, " values in %i threads.\n", numThreads);
  for (curThread = 0; curThread < numThreads; curThread++)
    {
      fprintf(stderr, "%i...", curThread);
      randTable[curThread] = new double[RandsPerThread];
      for (curNo = 0; curNo < RandsPerThread; curNo++)
	randTable[curThread][curNo] = drand48();
      curRands[curThread] = 0;
    }
  fprintf(stderr, "Done.\n");
  return 0;
}

double GetRandom(int threadNo)

{
  if (curRands[threadNo] == RandsPerThread)
    curRands[threadNo] = 0;
  return (randTable[threadNo][curRands[threadNo]++]);
}
