/*
 * simple_sut.C
 *
 *    This is a real simple program that just tries to do pseudo-TPC
 * transactions quickly.
 *
 * simple_sut.C,v 1.1 1993/08/09 21:42:12 gjb Exp
 */

#include <objdce/statics.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objtran/tranclient.H>
#include <objdce/thread.H>
#include <objdce/transignals.H>
#include <objdce/dcetime.H>
#include <dce/idlbase.h>
#include <params.H>
#include <tpc_simple_cdl_sfs.H>

SFSAccountFile* Accounts;
SFSTellerFile* Tellers;
SFSBranchFile* Branches;
static volatile Boolean Done;
static MutexLock CountLock;
static unsigned long TranCount;

class ProcessThread : public ThreadObject

{
public:
  ProcessThread(int threadNum);
  virtual ~ProcessThread();
  virtual void* Run();
private:
  int threadNum;
};

ProcessThread::ProcessThread(int n)

{
  threadNum = n;
}

ProcessThread::~ProcessThread()

{
}

void* ProcessThread::Run()

{
  Transaction* tran;
  unsigned long acctID;
  unsigned long tellerID;
  unsigned long branchID;
  ndr_hyper_int_rep delta;
  Account acctKey;
  Teller tellerKey;
  Branch branchKey;
  Account acct;
  Teller teller;
  Branch branch;

  while (!Done)
    {
      tellerID = (unsigned long)(GetRandom(threadNum) * (double)NumTellers);
      if (NumBranches == 1)
	branchID = 0;
      else
	branchID = tellerID / TellersPerBranch;
      // Make this TPC-compliant later
      acctID = (unsigned long)(GetRandom(threadNum) * (double)NumAccounts);
      delta.low = (GetRandom(threadNum) * 100) - 50;
      delta.high = 0;

      tran = new NewTransaction();

      SFSFileBase::IStartBatch();
      acctKey.accountID(acctID);
      Accounts->Increment_balance(AccountFile::PRIMARY, acctKey,
				  delta, acct);
      tellerKey.tellerID(tellerID);
      Tellers->Increment_balance(TellerFile::PRIMARY, tellerKey,
				 delta, teller);
      branchKey.branchID(branchID);
      Branches->Increment_balance(BranchFile::PRIMARY, branchKey,
				  delta, branch);
      SFSFileBase::IEndBatch();

      tran->Commit();
      if (tran->Committed())
	{
	  CountLock.Lock();
	  TranCount++;
	  CountLock.Unlock();
	}
      else
	{
	  DefaultReport.VReport(NormalMsg, "Failed: teller %li\n", tellerID);
	  DefaultReport.VReport(NormalMsg, "Failed: account %li\n", acctID);
	  DefaultReport.VReport(NormalMsg, "Failed: branch %li\n", branchID);
	}
      delete tran;
    }
  return (void*)0;
}

class DisplayThread : public ThreadObject

{
public:
  DisplayThread();
  virtual ~DisplayThread();
  virtual void* Run();
private:
  AbsoluteTime startTime;
  AbsoluteTime lastTime;
  unsigned long lastCount;
};

DisplayThread::DisplayThread()

{
  lastCount = 0;
}

DisplayThread::~DisplayThread()

{
}

void* DisplayThread::Run()

{
  double elapsedRate;
  double totalRate;
  Boolean started = false;
  AbsoluteTime now;
  RelativeTime elapsedRel;
  RelativeTime totalRel;
  
  while (!Done)
    {
      SleepMe(SleepTime, 0);
      CountLock.Lock();
      if ((TranCount > 0) && !started)
	{
	  startTime.SetCurrentTime();
	  started = true;
	}
      if (started)
	{
	  now.SetCurrentTime();
	  elapsedRel = now - lastTime;
	  lastTime = now;
	  totalRel = now - startTime;
	  elapsedRate = elapsedRel.CalculateRate(TranCount - lastCount);
	  lastCount = TranCount;
	  totalRate = totalRel.CalculateRate(TranCount);
	  printf("%li: %.2lf/second now %.2lf/second total\n",
		 TranCount, elapsedRate, totalRate);
	}
      CountLock.Unlock();
    }
  return (void*)0;
}

class StopHandler : public Signaler, public Signaled

{
public:
  StopHandler();
  virtual ~StopHandler();
  virtual Signaled::SigStatus HandleSignal(const int);
};

StopHandler::StopHandler()

{
}

StopHandler::~StopHandler()

{
}

Signaled::SigStatus StopHandler::HandleSignal(const int sig)

{
  if ((sig == SIGINT) || (sig == SIGTERM))
    {
      printf("Going down on signal %i\n", sig);
      Done = true;
      return Signaled::HANDLED;
    }
  else
    return Signaled::NOT_HANDLED;
}

static void PrintUsage(char* execName)

{
  printf("Usage: %s <sfs server name> <# threads>\n", execName);
}

int main (int argc, char** argv)

{
  int inc;
  TranClient client;
  StopHandler handler;
  DisplayThread display;
  ProcessThread** processors;
  AsyncThread** subThreads;
  unsigned int numThreads;
  unsigned int numProcessors;
  String serverName;
  Transaction* tran;
  String tmpStr;

  if (argc != 3)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  else
    {
      numProcessors = atoi(argv[2]);
      if (numProcessors == 0)
	{
	  PrintUsage(argv[0]);
	  return 1;
	}
    }

  client.DontHandleSignals();
  handler.CatchSignal(SIGINT, handler);
  handler.CatchSignal(SIGTERM, handler);

  Accounts = (SFSAccountFile*)0;
  Tellers = (SFSTellerFile*)0;
  Branches = (SFSBranchFile*)0;
  Done = false;
  TranCount = 0;
  tran = new NewTransaction();

  serverName = argv[1];
  Accounts = new SFSAccountFile();
  Accounts->OpenFile(serverName, AccountFile);
  if (!tran->Aborted())
    {
      Tellers = new SFSTellerFile();
      Tellers->OpenFile(serverName, TellerFile);
    }
  if (!tran->Aborted())
    {
      Branches = new SFSBranchFile();
      Branches->OpenFile(serverName, BranchFile);
    }
  tran->Commit();
  if (tran->Aborted())
    {
      printf("Couldn't open files:\n");
      tran->GetAbortReason()->Print(tmpStr);
      printf("%s\n", (const char*)tmpStr);
      Done = true;
    }
  delete tran;

  if (!Done)
    {
      ScanRandoms(numProcessors);
      numThreads = numProcessors + 1;
      subThreads = new AsyncThread*[numThreads];
      processors = new ProcessThread*[numProcessors];
      for (inc = 0; inc < numProcessors; inc++)
	{
	  processors[inc] = new ProcessThread(inc);
	  subThreads[inc] = new AsyncThread(*(processors[inc]));
	}
      subThreads[inc] = new AsyncThread(display);

      for (inc = 0; inc < numProcessors; inc++)
	{
	  subThreads[inc]->Wait();
	  delete processors[inc];
	  delete subThreads[inc];
	}
      subThreads[inc]->Wait();
      delete subThreads[inc];
      delete [] processors;
      delete [] subThreads;
    }
  if (Accounts != (SFSAccountFile*)0)
    {
      Accounts->CloseFile();
      delete Accounts;
    }
  if (Tellers != (SFSTellerFile*)0)
    {
      Tellers->CloseFile();
      delete Tellers;
    }
  if (Branches != (SFSBranchFile*)0)
    {
      Branches->CloseFile();
      delete Branches;
    }
  return 0;
}

