/*
 * RFSDataFile
 *
 *    This is a file that abstracts dealing with the database of RFS
 * records. It also has code to print them out and input them, which is
 * does by executing the user's EDITOR.
 *
 * rfsdatafile.H,v 1.1 1993/11/27 01:53:48 gjb Exp
 */

#ifndef RFS_DATA_FILE_H
#define RFS_DATA_FILE_H

#include <objdce/defs.H>
#include <stdio.h>
#include <String.h>
#include <rfsitem_cdl.H>
#include <rfsitem_cdl_sfs.H>

static const int LineBufSize = 256;
static const int ClosedRFSPriority = -1;

class RFSDataFile

{
public:
  enum DisplayType { DISPLAY, SUMMARIZE };

  RFSDataFile(const String& dbName, const String& fileName);
  ~RFSDataFile();

  Boolean Insert();			// Insert a new item
  Boolean Edit(const unsigned long id);
  Boolean Delete(const unsigned long id);
  Boolean Close(const unsigned long id);
  Boolean ReOpen(const unsigned long id, const long newPriority);

  Boolean DisplayAllItems(FILE* outFile, const DisplayType,
			  Boolean openOnly = false);
  Boolean DisplayByID(FILE* outFile, const unsigned long id,
		      const DisplayType);
  Boolean DisplayByApplication(FILE* outFile, const String& appName,
			       const DisplayType);
  Boolean DisplayByPriority(FILE* outFile, const long priority,
			    const DisplayType);
  Boolean DisplayByEstimatedDate(FILE* outFile, const long requestDate,
				 const DisplayType);
  
  Boolean Restore(FILE* inFile);
  
  static long StringToDate(const String& date);
  static String DateToString(const long date);

private:
  Boolean SaveItem(const String& fileName, const RFSItem& item);
  Boolean DisplayItem(FILE* outFile, const RFSItem& item,
		      const DisplayType, Boolean fromUser = true);
  Boolean ScanLine(const String& title, String& out);
  Boolean ScanMultiLine(FILE* inFile, const String& title,
			const String& nextTitle, String& out);
  Boolean ScanLine(const String& title, long& out);
  Boolean ScanLine(const String& title, unsigned long& out);
  Boolean ScanDateLine(const String& title, long& out);
  Boolean ReadLine(FILE* inFile);
  Boolean ReadLineAndSkipBlanks(FILE* inFile);
  Boolean ReadItem(FILE* inFile, RFSItem& newItem, Boolean fromUser = true);
  Boolean WriteTemplate(FILE* outFile);
  Boolean EditBlankItem(RFSItem& newItem);
  Boolean EditExistingItem(RFSItem& newItem);
  Boolean LaunchEditor(const String& fileName);

  char lineBuf[LineBufSize];
  SFSRFSDatabase rfsFile;
  String editorName;
  Boolean firstOne;
};
    
#endif
