/*
 * rfsdump
 *
 *    This program dumps all the items in the RFS list.
 *
 * rfsdump.C,v 1.1 1993/11/27 01:53:53 gjb Exp
 */

#include <objdce/statics.H>
#include <objtran/tranclient.H>
#include <objdce/login.H>
#include <rfsdatafile.H>
#include <stdio.h>
#include <stdlib.h>

static void PrintUsage(char* execName)

{
  printf("Usage:\n");
  printf("%s <SFS file> <output file>\n", execName);
  printf("    To dump the file on $ENCINA_SFS_SERVER\n");
  printf("    Use an output file of \"-\" to dump to standard output\n");
  printf("%s <SFS server> <SFS file> <output file>\n", execName);
  printf("    To dump the file on the specified server\n");
  printf("%s <SFS server> <SFS file> <output file> <principal name> <keytable file>\n", execName);
  printf("    To use a keytable file for authentication\n");
}

int main(int argc, char** argv)

{
  String sfsFile;
  String sfsServer;
  String outFileName;
  String keytableName;
  String principalName;
  FILE* outFile;
  Boolean parsed = true;
  LoginContext* context;
  Boolean useKeytable = false;
  Boolean loginOK = true;

  if ((argc < 3) || (argc > 6))
    {
      PrintUsage(argv[0]);
      parsed = false;
    }
  else if (argc == 3)
    {
      char* env = getenv("ENCINA_SFS_SERVER");
      if (env == (char*)0)
	{
	  printf("ENCINA_SFS_SERVER must be set.\n");	
	  PrintUsage(argv[0]);
	  parsed = false;
	}
      else
	{
	  sfsFile = argv[1];
	  sfsServer = env;
	  outFileName = argv[2];
	}
    }
  else if (argc == 4)
    {
      sfsServer = argv[1];
      sfsFile = argv[2];
      outFileName = argv[3];
    }
  else if (argc == 5)
    {
      PrintUsage(argv[0]);
      parsed = false;
    }
  else if (argc == 6)
    {
      sfsServer = argv[1];
      sfsFile = argv[2];
      outFileName = argv[3];
      principalName = argv[4];
      keytableName = argv[5];
      useKeytable = true;
    }

  if (parsed)
    {
      if (outFileName == "-")
	{
	  outFile = stdout;
	}
      else
	{
	  outFile = fopen(outFileName, "w");
	}
      if (outFile == (FILE*)0)
	{
	  perror("Couldn't open output file");
	}
      else
	{
	  if (useKeytable)
	    {
	      KeytableLoginContext* kContext = new KeytableLoginContext();
	      context = kContext;
	      if (kContext->LogIn(principalName, keytableName, 
				 LoginContext::CALL) == LoginContext::FAILURE)
		{
		  loginOK = false;
		}
	    }
	  else
	    {
	      CurrentLoginContext* cContext = new CurrentLoginContext();
	      context = cContext;
	      if (cContext->LogIn(LoginContext::CALL) == LoginContext::FAILURE)
		{
		  loginOK = false;
		}
	    }
	  if (!loginOK)
	    {
	      printf("Couldn't establish login context.\n");
	    }
	  else
	    {
	      TranClient client;
	      RFSDataFile file(sfsServer, sfsFile);
	      if (file.DisplayAllItems(outFile, RFSDataFile::DISPLAY))
		{
		  printf("Dump successful.\n");
		}
	      delete context;
	    }
	  if (outFileName != "-")
	    {
	      fclose(outFile);
	    }
	}
    }
  return (!parsed);
}

