/*
 * RFSMenu
 *
 *    This application lets the user manipulate RFS data via SFS. It
 * uses the RFSDataFile class for all data manipulation, and fires off
 * the user's default editor via RFSDataFile for all editing of records.
 * This makes life a lot easier.
 *
 * rfsmenu.C,v 1.1 1993/11/27 01:54:06 gjb Exp
 */

#include <objdce/statics.H>
#include <objtran/tranclient.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <rfsdatafile.H>
#include <stdio.h>
#include <stdlib.h>	
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>

void DoHelp()

{
  printf("a -- Add a new record\n");
  printf("s -- Summarize records...\n");
  printf("l -- List records...\n");
  printf("f -- Dump records to a file\n");
  printf("g -- Get a record by RFS ID\n");
  printf("e -- Edit a record by RFS ID\n");
  printf("c -- Close a record by RFS ID\n");
  printf("r -- Re-open a record by RFS ID\n");
  printf("d -- Delete a record by RFS ID\n");
  printf("q -- Quit\n");
}

void DoDisplay(RFSDataFile& theFile,
	       RFSDataFile::DisplayType type,
	       FILE* outFile)
{
  char input[256];
  char command;
  long tmpLong;

  printf("<A>ll; <O>pen; <C>losed; by (N)ame; by (P)riority; by (E)stimated date: ");
  gets(input);
  command = input[0];
  tolower(command);
  switch(command)
    {
    case 'a':
      theFile.DisplayAllItems(outFile, type, false);
      break;
    case 'o':
      theFile.DisplayAllItems(outFile, type, true);
      break;
    case 'c':
      theFile.DisplayByPriority(outFile, ClosedRFSPriority, type);
      break;
    case 'n':
      printf("Enter application name: ");
      gets(input);
      theFile.DisplayByApplication(outFile, input, type);
      break;
    case 'p':
      printf("Enter priority: ");
      gets(input);
      sscanf(input, "%li", &tmpLong);
      theFile.DisplayByPriority(outFile, tmpLong, type);
      break;
    case 'e':
      printf("Enter latest estimated date required: ");
      gets(input);
      tmpLong = theFile.StringToDate(input);
      theFile.DisplayByEstimatedDate(outFile, tmpLong, type);
      break;
    default:
      printf("Invalid choice.\n");
      break;
    }
}

void DoDump(RFSDataFile& theFile)

{
  char input[256];
  char command;
  RFSDataFile::DisplayType dType;
  Boolean OK = true;

  printf("Dump <S>ummary or <L>ist: ");
  gets(input);
  command = input[0];
  tolower(command);
  switch(command)
    {
    case 's':
      dType = RFSDataFile::SUMMARIZE;
      break;
    case 'l':
      dType = RFSDataFile::DISPLAY;
      break;
    default:
      printf("Invalid choice.\n");
      OK = false;
      break;
    }
  if (OK)
    {
      printf("Enter filename, or <Return> to dump to standard output: ");
      gets(input);	
      if (input[0] == '\0')
	{
	  DoDisplay(theFile, dType, stdout);
	}
      else
	{
	  FILE* dumpFile;
	  dumpFile = fopen(input, "a");
	  if (dumpFile == (FILE*)0)
	    {
	      perror("Couldn't open dump file");
	    }
	  else
	    {
	      DoDisplay(theFile, dType, dumpFile);
	      fclose(dumpFile);
	    }
	}
    }
}

unsigned long GetID()

{
  char input[256];
  unsigned long retVal;

  printf("Enter ID: ");
  gets(input);
  sscanf(input, "%lu", &retVal);
  return retVal;
}

void DoMainMenu(RFSDataFile& theFile)

{
  char inputBuf[256];
  char command = '\0';
  unsigned long id;
  unsigned long tmpLong;
  
  while (command != 'q')
    {
      printf("Enter command (? for help): ");
      gets(inputBuf);
      command = inputBuf[0];
      tolower(command);
      switch(command)
	{
	case '?':
	case 'h':
	  DoHelp();
	  break;
	case 'a':
	  theFile.Insert();
	  break;
	case 's':
	  DoDisplay(theFile, RFSDataFile::SUMMARIZE, stdout);
	  break;
	case 'l':
	  DoDisplay(theFile, RFSDataFile::DISPLAY, stdout);
	  break;
	case 'f':
	  DoDump(theFile);
	  break;
	case 'g':
	  id = GetID();
	  theFile.DisplayByID(stdout, id, RFSDataFile::DISPLAY);
	  break;
	case 'd':
	  id = GetID();
	  theFile.Delete(id);
	  break;
	case 'e':
	  id = GetID();
	  theFile.Edit(id);
	  break;	
	case 'c':
	  id = GetID();
	  theFile.Close(id);
	  break;
	case 'r':
	  id = GetID();
	  printf("New priority: ");
	  gets(inputBuf);
	  sscanf(inputBuf, "%lu", &tmpLong);
	  theFile.ReOpen(id, tmpLong);
	  break;
	case 'q':
	  break;
	default:
	  printf("Invalid command\n");
	  break;
	}
    }
}

static void PrintUsage(char* execName)

{
  printf("Usage:\n");
  printf("%s <file name>\n", execName);
  printf("	To open file on $ENCINA_SFS_SERVER\n");
  printf("%s <SFS server name> <file name>\n", execName);
  printf("      To open file on the named server\n");
  printf("%s -c <SFS server name> <file name> <volume name>\n", execName);
  printf("      To create the file\n");
}

int main(int argc, char** argv)

{
  String fileName;
  String serverName;
  String volName;
  Boolean create = false;
  Boolean parsed = true;

  if ((argc < 2) || (argc > 5))
    {
      PrintUsage(argv[0]);
      parsed = false;
    }
  else if (argc == 2)
    {
      char* env = getenv("ENCINA_SFS_SERVER");
      if (env == (char*)0)
	{
	  printf("ENCINA_SFS_SERVER must be set\n");
	  parsed = false;
	}
      else
	{
	  fileName = argv[1];
	  serverName = env;
	}
    }
  else if (argc == 3)
    {
      serverName = argv[1];
      fileName = argv[2];
    }
  else if (argc == 4)
    {
      PrintUsage(argv[0]);
      parsed = false;
    }
  else if (argc == 5)
    {
      if (strcmp(argv[1], "-c"))
	{
	  PrintUsage(argv[0]);
	  parsed = false;
	}
      else
	{
	  serverName = argv[2];
	  fileName = argv[3];
	  volName = argv[4];
	  create = true;
	}
    }

  if (parsed)
    {
      TranClient client;
      if (create)
	{
	  SFSRFSDatabase createFile;
	  NewTransaction createTran;
	  createFile.CreateFile(serverName, fileName, volName, 100);
	  createTran.Commit();
	  if (createTran.Aborted())
	    {
	      String abtReas;
	      createTran.GetAbortReason()->Print(abtReas);
	      printf("Error creating file:\n%s\n", (const char*)abtReas);
	    }
	  else
	    {
	      printf("Successfully created the file.\n");
	    }
	}
      else
	{
	  RFSDataFile dFile(serverName, fileName);
	  DoMainMenu(dFile);
	}
    }
  return (!parsed);
}
