/*
 * rfsrestore
 *
 *    This restores an RFS file from a file dumped by rfsdump,
 * and only from such a file.
 *
 * rfsrestore.C,v 1.1 1993/11/27 01:54:08 gjb Exp
 */

#include <objdce/statics.H>
#include <objtran/tranclient.H>
#include <objdce/abortreason.H>
#include <objtran/transaction.H>
#include <rfsdatafile.H>
#include <stdio.h>
#include <stdlib.h>

static void PrintUsage(char* execName)

{
  printf("Usage:\n");
  printf("%s <SFS file> <SFS volume> <input file>\n", execName);
  printf("    To restore the file on $ENCINA_SFS_SERVER\n");
  printf("%s <SFS server> <SFS file> <SFS volume> <input file>\n", execName);
  printf("    To restore on the specified server\n");
}

int main(int argc, char** argv)

{
  String sfsFile;
  String sfsServer;
  String inFileName;
  String sfsVolume;
  FILE* inFile;
  Boolean parsed = true;

  if ((argc < 4) || (argc > 5))
    {
      PrintUsage(argv[0]);
      parsed = false;
    }
  else if (argc == 4)
    {
      char* env = getenv("ENCINA_SFS_SERVER");
      if (env == (char*)0)
	{
	  printf("ENCINA_SFS_SERVER must be set.\n");	
	  PrintUsage(argv[0]);
	  parsed = false;
	}
      else
	{
	  sfsFile = argv[1];
	  sfsServer = env;
	  sfsVolume = argv[2];
	  inFileName = argv[3];
	}
    }
  else if (argc == 5)
    {
      sfsServer = argv[1];
      sfsFile = argv[2];
      sfsVolume = argv[3];
      inFileName = argv[4];
    }

  if (parsed)
    {
      inFile = fopen(inFileName, "r");
      if (inFile == (FILE*)0)
	{
	  perror("Couldn't open output file");
	}
      else
	{
	  TranClient client;
	  SFSRFSDatabase dataFile;

	  NewTransaction* createTran = new NewTransaction();
	  dataFile.CreateFile(sfsServer, sfsFile, sfsVolume, 100);
	  createTran->Commit();
	  if (createTran->Aborted())
	    {
	      String abtReason;
	      createTran->GetAbortReason()->Print(abtReason);
	      printf("Couldn't create file:\n%s\n",
		     (const char*)abtReason);
	      delete createTran;
	    }
	  else
	    {
	      delete createTran;
	      printf("Created file %s on %s\n", (const char*)sfsFile,
		     (const char*)sfsServer);
	      RFSDataFile file(sfsServer, sfsFile);
	      if (file.Restore(inFile))
		{
		  printf("Restore was successful.\n");
		}
	    }
	  fclose(inFile);
	}
    }
  return (!parsed);
}
