/*
 * requestsvr.C
 *
 *    This is the server that implements the functions defined in
 * requestee.idl.
 *
 * requestsvr.C,v 1.1 1993/11/27 01:54:44 gjb Exp
 */

#include <objdce/statics.H>
#include <objdce/nontranserver.H>
#include <request_cdl.H>
#include <List.h>
#include <requestee.h>
#include <iostream.h>

/*
 * Globals
 */

NonTranServer* server;

/*
 * SendOneRequest
 *    Receive a request from the client. The RequestRec class has a copy
 * constructor (and an assignment operator) for a RequestRecBase, so we
 * just dereference the pointer and use it to create the RequestRec.
 */

error_status_t SendOneRequest(handle_t handle,
			      RequestRecBase* request)
{
  if (!server->IsClntAuthorized(handle))
    {
      return rpc_s_binding_has_no_auth;
    }
  
  RequestRec req(*request);
  cout << "Got a request:" << endl;
  cout << req;
  return 0;
}

/*
 * GetOneRequest
 *    Send a request back to the client. The client passes a RequestRecBase*
 * as an [out] parameter, so we cannot change the value of this pointer
 * (just like in C). So rather than assigning a new value to the pointer,
 * we use the "Copy" function on the RequestRec class to copy a new value
 * into the memory that the client "allocated".
 */

error_status_t GetOneRequest(handle_t handle,
			     RequestRecBase* request)

{
  if (!server->IsClntAuthorized(handle))
    {
      return rpc_s_binding_has_no_auth;
    }
  
  RequestRec req;
  req.requestType(RequestDebit);
  req.numAccounts(3);
  req.accounts(0, "011982");
  req.balances(0, 112.21);
  req.accounts(1, "923982");
  req.balances(1, 1.00);
  req.accounts(2, "000001");
  req.balances(2, 99.99);
  cout << "Returning one request:" << endl;	
  cout << req;
  req.Copy(*request);
  return 0;
}

/*
 * ExchangeOneRequest
 *    Receive a record from the client, than replace the data in that
 * record with new data. We read the client record using the copy constructor
 * like we did in SendOneRequest, and get it back using the "Copy" function
 * like we did in GetOneRequest.
 */

error_status_t ExchangeOneRequest(handle_t handle,
				  RequestRecBase* request)

{
  if (!server->IsClntAuthorized(handle))
    {
      return rpc_s_binding_has_no_auth;
    }
  
  RequestRec req(*request);
  cout << "Got a request:" << endl;
  cout << req;
  req.requestType(RequestDebit);
  req.numAccounts(4);
  req.accounts(0, "011982");
  req.balances(0, 112.21);
  req.accounts(1, "923982");
  req.balances(1, 1.00);
  req.accounts(2, "000001");
  req.balances(2, 99.99);
  req.accounts(3, "000002");
  req.balances(3, 109.12);
  cout << "Returning one request:" << endl;
  cout << req;
  req.Copy(*request);
  return 0;
}

/*
 * SendSomeRequests
 *    Get a list of records from the client. We use the ArrayToList
 * function to copy the array into a List object, then iterate through
 * the list. DCE takes care of the storage for the array of
 * records on this side, so we don't have to allocate storage or free it.
 */

error_status_t SendSomeRequests(handle_t handle,
				RequestRecBaseArray* requests)

{
  if (!server->IsClntAuthorized(handle))
    {
      return rpc_s_binding_has_no_auth;
    }

  RequestRec req;  
  List<RequestRec> reqList;
  Listiter<RequestRec> reqListIter(reqList);

  RequestRec::ArrayToList(*requests, reqList);
  cout << "Got " << reqList.length() << " requests:" << endl;
  while (reqListIter.next(req))
    {
      cout << req << endl;
    }
  
  // Empty the list. We don't have to free "requests", however
  while (reqListIter.remove_next());
  return 0;
}

/*
 * GetSomeRequests
 *    Return an array of records to the client. We build a List of
 * RequestRec objects, then use ServerListToArray to copy the list
 * into a RequestRecBaseArray, which the client uses. The memory
 * allocated in ServerListToArray is automatically freed by DCE, so
 * we should not free it.
 */

error_status_t GetSomeRequests(handle_t handle,
			       RequestRecBaseArray* requests)

{
  if (!server->IsClntAuthorized(handle))
    {
      return rpc_s_binding_has_no_auth;
    }

  RequestRec req;  
  List<RequestRec> reqList;
  Listiter<RequestRec> reqListIter(reqList);
  char tmpAccount[10];

  req.requestType(RequestCredit);
  req.numAccounts(1);
  for (int inc = 0; inc < 2; inc++)
    {	
      sprintf(tmpAccount, "%6i", inc);
      req.accounts(0, tmpAccount);
      req.balances(0, 1.19);
      reqList.put(req);
    }

  cout << "Returning " << reqList.length() << " requests." << endl;
  RequestRec::ServerListToArray(reqList, *requests);
  while (reqListIter.remove_next());
  return 0;
}

/*
 * ExchangeSomeRequests
 *    Get one array of records from the client, and return another.
 * We can't free the memory the client allocated for us in the "input"
 * array and expect the client to "know", so we use a separate array
 * to return data.
 */

error_status_t ExchangeSomeRequests(handle_t handle,
				    RequestRecBaseArray* inRequests,
				    RequestRecBaseArray* outRequests)
{
  if (!server->IsClntAuthorized(handle))
    {
      return rpc_s_binding_has_no_auth;
    }

  RequestRec req;  
  List<RequestRec> reqList;
  Listiter<RequestRec> reqListIter(reqList);
  char tmpAccount[10];

  // Process the data sent us. Empty our List while we print out
  // the elements.
  RequestRec::ArrayToList(*inRequests, reqList);
  cout << "Got " << reqList.length() << " requests:" << endl;
  while (reqListIter.remove_next(req))
    {
      cout << req << endl;
    }

  // Put items in the list to send back to the client
  req.requestType(RequestCredit);
  req.numAccounts(1);
  for (int inc = 0; inc < 5; inc++)
    {	
      sprintf(tmpAccount, "%6i", inc);
      req.accounts(0, tmpAccount);
      req.balances(0, 1.19);
      reqList.put(req);
    }

  RequestRec::ServerListToArray(reqList, *outRequests);
  while (reqListIter.remove_next());
  return 0;
}

static void PrintUsage(const char* execName)

{
  cerr << "Usage: " << execName << " <server name> <principal> <keytable>";
  cerr << endl;
}

int main(int argc, char** argv)

{
  if (argc != 4)
    {
      PrintUsage(argv[0]);
      return 1;
    }

  String svrName = argv[1];
  String prinName = argv[2];
  String keytable = argv[3];

  ServerLoginContext context;
  if (context.LogIn(prinName, keytable, 
		    "", LoginContext::CALL) != LoginContext::SUCCESS)
    {
      cerr << "DCE log in failed." << endl;
      return 1;
    }

  server = new NonTranServer(RPCServer(Requestee,1,0), svrName,
			     "Sample YADDL array server", UDP,
			     &context);
  cout << "Listening..." << endl;
  server->StartServer();
  cout << "Shutting down..." << endl;
  delete server;
  return 0;
}

  
  
