/*
 * nontrantest.C
 *
 *    This talks to the server using only DCE RPC.
 *
 * nontrantest.C,v 1.1 1994/04/19 19:18:49 gjb Exp
 */

#include <svrtest_cdl.H>
#include <nontransvrtest.h>
#include <objdce/binding.H>
#include <cdl/cdlerrors.H>
#include <String.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define ENDPT 9000

static Boolean CheckResult(unsigned long dceErr, unsigned long dbErr)

{
  if (dceErr != rpc_s_ok)
    {
      String dceMsg;
      
      Binding::CheckErrorType(dceErr, dceMsg);
      cout << "DCE error: " << dceMsg << endl;
      return false;
    }
  else if (dbErr != CDL_SUCCESS)
    {
      String cdlMsg;

      CDLStatusToString((CDLStatus)dbErr, cdlMsg);
      cout << "YADDL error: " << cdlMsg << endl;
      return false;
    }
  else
    {
      return true;
    }
}

static void PrintUsage(const char* execName)

{
  printf("Usage: %s <server name>\n", execName);
}

int main(int argc, char** argv)

{
  char input[1024];
  char command;
  char hostName[256];
  Binding* curBinding;

  if (argc != 2)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  
  String svrName = argv[1];
#ifdef WK_ENDPOINTS
  gethostname(hostName, 256);
  curBinding = new Binding(hostName, ENDPT);
#else
  curBinding = new Binding(svrName, RPCClient(NonTranSvrTest,1,0),
			   LoginContext::CONNECT);
  if (curBinding->IsValid())
    {
      printf("Bound to \"%s\"\n", (const char*)svrName);
    }
  else
    {
      printf("Couldn't bind to \"%s\"\n", (const char*)svrName);
      delete curBinding;
      curBinding = (Binding*)0;
    }
#endif
  if (curBinding == (Binding*)0)
    {
      printf("Couldn't find any servers in \"%s\"\n", (const char*)svrName);
    }
  else
    {
      long key;
      SimpleRecord rec;
      unsigned long dceErr;
      unsigned long dbErr;
      SimpleRecordContext context;
      
      command = '\0';
      while (command != 'q')
	{
	  rec.Initialize();
	  printf("[I]nsert [S]tart [N]ext [D]one [Q]uit: ");
	  gets(input);
	  command = input[0];
	  tolower(command);
	  switch(command)
	    {
	    case 'i':
	      printf("Key: ");
	      gets(input);
	      key = atol(input);
	      rec.key(key);
	      printf("Data: ");	
	      gets(input);
	      rec.data(input);
	      dceErr = NonTranInsert(*curBinding, rec, &dbErr);
	      CheckResult(dceErr, dbErr);
	      break;
	    case 's':
	      printf("Key: ");
	      gets(input);
	      key = atol(input);
	      dceErr = NonTranFindBegin(*curBinding, key, &context, &dbErr);
	      CheckResult(dceErr, dbErr);
	      break;
	    case 'n':
	      dceErr = NonTranFindNext(&context, rec, &dbErr);
	      if (CheckResult(dceErr, dbErr))
		{
		  cout << rec << endl;
		}
	      break;
	    case 'd':
	      dceErr = NonTranFindDone(&context, &dbErr);
	      CheckResult(dceErr, dbErr);
	      break;
	    case 'q':
	      break;
	    default:
	      printf("Invalid command\n");
	      break;
	    }
	}
    }
  delete curBinding;
  return 0;
}

