/*
 * svrtest.C
 *
 *    This fires up our simple server and lets it do its thing.
 *
 * svrtest.C,v 1.2 1994/04/07 19:29:12 gjb Exp
 */

#include <svrtest_cdl_sfs.H>
#include <nontransvrtest.h>
extern "C"
{
#include <transvrtest.h>
}
#include <objtran/transerver.H>
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objdce/uuid.H>
#include <objdce/bindinggroup.H>

TranServer* MainServer;
SFSSimpleFile* MainFile;

#define ENDPT 9000

static void PrintUsage(const char* execName)

{
  printf("Usage: %s <RPC name> <RPC group> <principal> <keytable> <SFS svr> <SFS file>\n",
	 execName);
}

int main(int argc, char** argv)

{
  CDLStatus cdlErr;
  char buffer[1024];
  Boolean created;
  Boolean opened = false;
  String errMsg;
  DCEuuid objUUID;
  
  if (argc != 7)
    {
      PrintUsage(argv[0]);
      return 1;
    }

  String svrName = argv[1];
  String groupName = argv[2];
  String prinName = argv[3];
  String keytable = argv[4];
  String sfsName = argv[5];
  String fileName = argv[6];

  ServerLoginContext context;
  if (context.LogIn(prinName, keytable, "", 
		    LoginContext::CONNECT) != LoginContext::SUCCESS)
    {
      printf("Could not establish login context.\n");
      return 1;
    }
  
#ifdef WK_ENDPOINTS
  MainServer = new TranServer(RPCServer(TranSvrTest,1,0),
			      RPCVector(TranSvrTest,1,0),
			      &context, ENDPT);
#else
  MainServer = new TranServer(RPCServer(TranSvrTest,1,0),
			      RPCVector(TranSvrTest,1,0),
			      svrName, "Simple transactional server",
			      UDP, &context, (List_of_p<TranComponent>*)0, 
			      (DCEuuid*)0, &objUUID);
  MainServer->AddInterface(RPCServer(NonTranSvrTest,1,0),
			   svrName, "Simple non-transactional server",
			   (rpc_mgr_epv_t*)0, (DCEuuid*)0, &objUUID);
#endif
  
  BindingGroup bGroup(groupName);	
  bGroup.Add(svrName);

  MainFile = new SFSSimpleFile();
  cdlErr = MainFile->OpenFile(sfsName, fileName);	
  opened = (cdlErr == CDL_SUCCESS);
  if (!opened)
    {
      printf("Could not open the file \"%s\"\n", (const char*)fileName);
      printf("Enter a volume name to create it on, or <Return> to exit: ");
      gets(buffer);
      created = false;
      if (buffer[0] != '\0')
	{
	  String volName = buffer;
	  cdlErr = MainFile->CreateFile(sfsName, fileName, volName, 0);
	  if (cdlErr == 0)
	    {
	      cdlErr = MainFile->OpenFile(sfsName, fileName,
			  CDL_EXCLUSIVE_OPEN | CDL_NONTRANSACTIONAL_OPEN);

	    }
	  if (cdlErr == 0)
	    {
		/****
	      SimpleRecord excludedKey;
	      excludedKey.data("");
	      cdlErr = MainFile->ActivateIndex(SimpleFile::DATA, volName, 
					       &excludedKey);
		***/
	      cdlErr = MainFile->ActivateIndex(SimpleFile::DATA, volName);
	      MainFile->CloseFile();
	    }
	  created = (cdlErr == CDL_SUCCESS);
	}
      if (created)
	{
	  cdlErr = MainFile->OpenFile(sfsName, fileName);
	  opened = (cdlErr == CDL_SUCCESS);
	}
      else
	{
	  CDLStatusToString(cdlErr, errMsg);
	  printf("Couldn't create file: %s\n", (const char*)errMsg);
	}
    }
  
  if (opened)
    {
      printf("Listening...\n");
      MainServer->StartServer();
      printf("Closing files and exiting...\n");
    }
  else
    {
      printf("Exiting...\n");
    }
  bGroup.Remove(svrName);
  delete MainFile;
  delete MainServer;
  return 0;
}
