/*
 * trantest.C
 *
 *    This calls the TRPC interface to the server.
 *
 * trantest.C,v 1.2 1994/04/07 19:29:26 gjb Exp
 */

#include <svrtest_cdl.H>
extern "C"
{
#include <transvrtest.h>
}
#include <objtran/transaction.H>
#include <objdce/abortreason.H>
#include <objtran/tranbinding.H>
#include <objtran/tranclient.H>
#include <objdce/bindinggroup.H>
#include <String.h>
#include <List.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
extern "C"
{
#include <utils/trace.h>
}

// extern int do_initialize;

#define ENDPT 9000

void PrintUsage(const char* execName)

{
  printf("Usage: %s <server name>\n", execName);
}

int main(int argc, char** argv)

{
  char input[1024];
  char command;
//  do_initialize = 1;
  TranClient client;
  FILE* traceFile;
  char hostName[256];
  List<String> groupList;
  Const_listiter<String> groupListIter(groupList);
  List<TranBinding*> bindingList;
  Listiter<TranBinding*> bindingListIter(bindingList);
  String curName;
  TranBinding* curBinding;
  TranBinding* groupBinding = (TranBinding*)0;

/***
  traceFile = fopen("EncinaTrace", "w");
  trace_Register(0xffffffff, trace_FileUpcall, (void*)traceFile);
  trpc_traceMask = 0xffffffff;
  tran_traceMask = 0xffffffff;
  ***/
  if (argc != 2)
    {
      PrintUsage(argv[0]);
      return 1;
    }
  
  String svrName = argv[1];
  gethostname(hostName, 256);
#ifdef WK_ENDPOINTS
  curBinding = new TranBinding(hostName, ENDPT);
  bindingList.put(curBinding);
#else
  BindingGroup bGroup(svrName);
  bGroup.GetEntries(groupList);
  groupListIter.reset();
  while (groupListIter.next(curName))
    {
      curBinding = new TranBinding(curName, RPCClient(TranSvrTest,1,0),
				   LoginContext::CONNECT);
      if (curBinding->IsValid())
	{
	  printf("Bound to \"%s\" from the group\n", (const char*)curName);
	  bindingList.put(curBinding);
	}
      else
	{
	  printf("Couldn't bind to \"%s\" from the group\n", 
		 (const char*)curName);
	  delete curBinding;
	}
    }
  groupBinding = new TranBinding(svrName, RPCClient(TranSvrTest,1,0),
				 LoginContext::CONNECT);
#endif
  if (bindingList.length() == 0)
    {
      printf("Couldn't find any servers in \"%s\"\n", (const char*)svrName);
    }
  else
    {
      long key;
      SimpleRecord rec;
      List<SimpleRecord> records;
      Listiter<SimpleRecord> iter(records);
      SimpleRecordBaseArray recArray;
      Transaction* tran;
      String abortReason;
      unsigned char dataKey[64];
      
      command = '\0';
      while (command != 'q')
	{
	  printf("[I]nsert [F]ind [S]earch [L]ist [Q]uit: ");
	  gets(input);
	  command = input[0];
	  tolower(command);
	  switch(command)
	    {
	    case 'i':
	      printf("Key: ");
	      gets(input);
	      key = atol(input);
	      rec.key(key);
	      printf("Data: ");	
	      gets(input);
	      rec.data(input);
	      tran = new NewTransaction();
	      bindingListIter.reset();
	      while (!tran->Aborted() && bindingListIter.next(curBinding))
		{
		  TranInsert(*curBinding, rec);
		}
	      tran->Commit();
	      if (tran->Aborted())
		{
		  tran->GetAbortReason()->Print(abortReason);
		  printf("Transaction aborted: %s\n", (const char*)abortReason);
		}
	      delete tran;
	      break;
	    case 'f':
	      printf("Key: ");
	      gets(input);
	      key = atol(input);
	      tran = new NewTransaction();	
/****
	      TranFind(*svrBinding, key, rec);
	      tran->Commit();
	      if (tran->Aborted())
		{
		  tran->GetAbortReason()->Print(abortReason);
		  printf("Transaction aborted: %s\n", (const char*)abortReason);
		}
	      else
		{
		  cout << rec;
		}
 ****/
	      TranFind(*groupBinding, key, &recArray);
	      tran->Commit();
	      if (tran->Aborted())
		{
		  tran->GetAbortReason()->Print(abortReason);
		  printf("Transaction aborted: %s\n", (const char*)abortReason);
		}
	      SimpleRecord::ArrayToList(recArray, records);
	      SimpleRecord::ClientArrayFree(recArray);
	      iter.reset();
	      while (iter.remove_next(rec))
		{
		  cout << rec;
		}
	      delete tran;
	      break;
	    case 's':
	      printf("Key: ");
	      gets(input);
	      strncpy((char*)dataKey, input, 64);
	      dataKey[63] = '\0';
	      tran = new NewTransaction();
	      TranFindData(*groupBinding, dataKey, rec);
	      tran->Commit();
	      if (tran->Aborted())
		{
		  tran->GetAbortReason()->Print(abortReason);
		  printf("Transaction aborted: %s\n", (const char*)abortReason);
		}
	      else
		{
		  cout << rec;
		}
	    case 'l':
	      tran = new NewTransaction();
	      TranListData(*groupBinding, &recArray);
	      tran->Commit();
	      if (tran->Aborted())
		{
		  tran->GetAbortReason()->Print(abortReason);
		  printf("Transaction aborted: %s\n", (const char*)abortReason);
		}
	      SimpleRecord::ArrayToList(recArray, records);
	      SimpleRecord::ClientArrayFree(recArray);
	      iter.reset();
	      while (iter.remove_next(rec))
		{
		  cout << rec;
		}
	      delete tran;
	      break;
	    case 'q':
	      break;
	    default:
	      printf("Invalid command\n");
	      break;
	    }
	}
    }
  bindingListIter.reset();
  while (bindingListIter.remove_next(curBinding))
    {
      delete curBinding;
    }
  delete groupBinding;
/***
  trace_Unregister(0xffffffff, trace_FileUpcall, (void*)traceFile);
  fclose(traceFile);
  ***/
  return 0;
}

