/*
 * cdlerrorfuncs.C
 *
 *    Here, we have functions that deal with CDL errors. Basically, this
 * means CDLStatusToString.
 *
 * cdlerrorfuncs.C,v 1.1 1994/04/19 19:19:35 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#include <cdl/cdldefs.H>
#include <cdl/cdlerrors.H>
#include <objdce/error.H>

/*
 * This is the message ID of the start of the "CDLStatus" message text
 * in the YADDL message catalog.
 */

static const int CDLErrorMsgBase = 6000;
static const int CDLErrorDiff = CDLErrorMsgBase - CDLErrorBase;

void CDLStatusToString(const CDLStatus status,
		       String& out)

{
  int realCatNum = status + CDLErrorDiff;
  CDLReport.GetMsg(realCatNum, out);
}

/*
 * This stuff takes care of initializing the "CDLReport" object for use
 * by the library code.
 */

static pthread_once_t didCDLFirstInit = pthread_once_init;
int CDLReportRefCount = 0;
CatReport CDLReport("cdl.cat", &CDLReportRefCount);

static void RealCDLFirstInit()

{
  new (&CDLReport)CatReport("cdl.cat", &CDLReportRefCount);
}

void CDLFirstInit()

{
  if (pthread_once(&didCDLFirstInit, RealCDLFirstInit) != 0)
    {
      perror("FATAL ERROR: Couldn't initalize YADDL message catalog. Exiting.");
      exit(1);
    }
}


