/*
 * sfsbases
 * 
 *    Implement the SFS base classes describes in sfsbases.H. Most of these
 * classes do very little but store a bunch of data.
 *
 * sfsbases.C,v 1.9 1993/12/03 14:42:11 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/binding.H>
#include <cdl/cdldefs.H>
#include <cdl/sfsbases.H>
#include <objtran/transaction.H>
extern "C"
{
#include <threadTid/threadTid.h>
}

SFSBaseRecord::SFSBaseRecord()

{
  rec.buffer = (sfs_pointer_t)0;
  rec.bufferLength = 0;
  rec.dataLength = 0;
  rec.numExternalFields = 0;
  rec.externalFieldsArrayP = (sfs_externalField_t*)0;
}

SFSBaseRecord::~SFSBaseRecord()

{
  if (rec.buffer != (sfs_pointer_t)0)
    free((char*)rec.buffer);
  if (rec.externalFieldsArrayP != (sfs_externalField_t*)0)
    free((char*)rec.externalFieldsArrayP);
}

sfs_record_t* SFSBaseRecord::Copy() const

{
  int inc;
  sfs_externalField_t* retField;
  sfs_externalField_t* recField;
  sfs_record_t* ret = (sfs_record_t*)malloc(sizeof(sfs_record_t));

  ret->format = rec.format;
  if (rec.buffer != (sfs_pointer_t)0)
    {
      ret->buffer = (sfs_pointer_t)malloc(sizeof(unsigned char) * 
					  (unsigned int)rec.bufferLength);
      memcpy(ret->buffer, rec.buffer, (unsigned int)rec.dataLength);
    }
  else
    ret->buffer = (sfs_pointer_t)0;
  ret->bufferLength = rec.bufferLength;
  ret->dataLength = rec.dataLength;
  ret->numExternalFields = rec.numExternalFields;
  if (rec.numExternalFields > 0)
    {
      ret->externalFieldsArrayP = (sfs_externalField_t*)malloc
	               (sizeof(sfs_externalField_t) * 
			(unsigned int)rec.numExternalFields);
      for (inc = 0; inc < rec.numExternalFields; inc++)
	{
	  retField = &(ret->externalFieldsArrayP[inc]);
	  recField = &(rec.externalFieldsArrayP[inc]);
	  retField->name = (char*)malloc(sizeof(char) * 
					 (strlen(recField->name) + 1));
	  strcpy(retField->name, recField->name);
	  retField->buffer = recField->buffer;		// This is important
	  retField->bufferLength = recField->bufferLength;
	  retField->dataLength = recField->dataLength;
	}
    }
  else
    ret->externalFieldsArrayP = (sfs_externalField_t*)0;
  return ret;
}

void SFSBaseRecord::Destroy(sfs_record_t* kill)

{
  int inc;
  sfs_externalField_t* killField;

  if (kill->buffer != (sfs_pointer_t)0)
    free((char*)kill->buffer);
  for (inc = 0; inc < kill->numExternalFields; inc++)
    {
      killField = &(kill->externalFieldsArrayP[inc]);
      free((char*)killField->name);
    }
  if (kill->externalFieldsArrayP != (sfs_externalField_t*)0)
    free((char*)kill->externalFieldsArrayP);
  free((char*)kill);
}

CDLStatus SFSBaseRecord::CheckPacking(unsigned long len)

{
  if (len > rec.bufferLength)
    {
      String msg;
      CDLReport.GetMsg(10021, msg);
      if (threadTid_Lookup() != TRAN_TID_NULL)
	{
	  ExistingTransaction tran;
	  tran.Abort(msg, (int)CDL_PACKING_ERROR, CDLId);
	  CDLReport.Report(ErrorLevels::InfoMsg, msg);
	}
      else
	CDLReport.Report(ErrorLevels::ErrorMsg, msg);
      return CDL_PACKING_ERROR;
    }
  else
    return CDL_SUCCESS;  
}

CDLStatus SFSBaseRecord::CheckUnpacking(unsigned long len)

{
  if (len != rec.dataLength)
    {
      String msg;
      CDLReport.GetMsg(10022, msg);
      if (threadTid_Lookup() != TRAN_TID_NULL)
	{
	  ExistingTransaction tran;
	  tran.Abort(msg, (int)CDL_UNPACKING_ERROR, CDLId);
	  CDLReport.Report(ErrorLevels::InfoMsg, msg);
	}
      else
	CDLReport.Report(ErrorLevels::ErrorMsg, msg);
      return CDL_UNPACKING_ERROR;
    }
  else
    return CDL_SUCCESS;
}

SFSBaseKey::SFSBaseKey()

{ 
  key.keyRecord.buffer = (sfs_pointer_t)0;
  key.keyRecord.bufferLength = 0;
  key.keyRecord.dataLength = 0;
  key.keyRecord.numExternalFields = 0;
  key.keyRecord.externalFieldsArrayP = (sfs_externalField_t*)0;
}

SFSBaseKey::~SFSBaseKey()

{	
  if (key.keyRecord.buffer != (sfs_pointer_t)0)
    free((char*)key.keyRecord.buffer);
  if (key.keyRecord.externalFieldsArrayP != (sfs_externalField_t*)0)
    free((char*)key.keyRecord.externalFieldsArrayP);
}

sfs_key_t* SFSBaseKey::Copy() const

{
  int inc;
  sfs_externalField_t* retField;
  sfs_externalField_t* recField;
  sfs_key_t* ret = (sfs_key_t*)malloc(sizeof(sfs_key_t));

  ret->keyFormat = key.keyFormat;
  ret->keyRecord.format = key.keyRecord.format;
  if (key.keyRecord.buffer != (sfs_pointer_t)0)
    {
      ret->keyRecord.buffer = (sfs_pointer_t)malloc(sizeof(unsigned char) *
				    (unsigned int)key.keyRecord.bufferLength);
      memcpy(ret->keyRecord.buffer, key.keyRecord.buffer,
	     (unsigned int)key.keyRecord.dataLength);
    }
  else
    ret->keyRecord.buffer = (sfs_pointer_t)0;
  ret->keyRecord.bufferLength = key.keyRecord.bufferLength;
  ret->keyRecord.dataLength = key.keyRecord.dataLength;
  ret->keyRecord.numExternalFields = key.keyRecord.numExternalFields;
  if (key.keyRecord.numExternalFields > 0)
    {
      ret->keyRecord.externalFieldsArrayP = (sfs_externalField_t*)malloc
	(sizeof(sfs_externalField_t) * 
	 (unsigned int)key.keyRecord.numExternalFields);
      for (inc = 0; inc < key.keyRecord.numExternalFields; inc++)
	{
	  retField = &(ret->keyRecord.externalFieldsArrayP[inc]);
	  recField = &(key.keyRecord.externalFieldsArrayP[inc]);
	  retField->name = (char*)malloc(sizeof(char) * 
					 (strlen(recField->name) + 1));
	  strcpy(retField->name, recField->name);
	  retField->buffer = recField->buffer;		// This is important
	  retField->bufferLength = recField->bufferLength;
	  retField->dataLength = recField->dataLength;
	}
    }
  else
    ret->keyRecord.externalFieldsArrayP = (sfs_externalField_t*)0;
  return ret;
}

void SFSBaseKey::Destroy(sfs_key_t* kill)

{
  int inc;
  sfs_externalField_t* killField;

  if (kill->keyRecord.buffer != (sfs_pointer_t)0)
    free((char*)kill->keyRecord.buffer);
  for (inc = 0; inc < kill->keyRecord.numExternalFields; inc++)
    {
      killField = &(kill->keyRecord.externalFieldsArrayP[inc]);
      free((char*)killField->name);
    }
  if (kill->keyRecord.externalFieldsArrayP != (sfs_externalField_t*)0)
    free((char*)kill->keyRecord.externalFieldsArrayP);
  free((char*)kill);
}

CDLStatus SFSBaseKey::CheckPacking(unsigned long len)

{
  if (len > key.keyRecord.bufferLength)
    {
      String msg;
      CDLReport.GetMsg(10021, msg);
      if (threadTid_Lookup() != TRAN_TID_NULL)
	{
	  ExistingTransaction tran;
	  tran.Abort(msg, (int)CDL_PACKING_ERROR, CDLId);
	  CDLReport.Report(ErrorLevels::InfoMsg, msg);
	}
      else
	CDLReport.Report(ErrorLevels::ErrorMsg, msg);
      return CDL_PACKING_ERROR;
    }
  else
    return CDL_SUCCESS;
}

SFSBaseSecondary::SFSBaseSecondary()

{
  spec.active = 1;
  spec.unique = 1;
  spec.alternateRecordSpecP = (sfs_recordSpec_t*)0;
  spec.excludedKeyP = (sfs_key_t*)0;
  spec.numFields = 0;
  spec.fieldSpecArrayP = (sfs_indexFieldSpec_t*)0;
}

SFSBaseSecondary::~SFSBaseSecondary()

{
  if (spec.alternateRecordSpecP != (sfs_recordSpec_t*)0)
    free((char*)spec.alternateRecordSpecP);
  if (spec.excludedKeyP != (sfs_key_t*)0)
    free((char*)spec.excludedKeyP);
  if (spec.fieldSpecArrayP != (sfs_indexFieldSpec_t*)0)
    free((char*)spec.fieldSpecArrayP);
}

SFSBaseFileSpec::SFSBaseFileSpec() : typeName("")

{
  spec.fileOrganization = sfs_btreeClustered;
  spec.primaryIndexName = "Primary";
  spec.maxNumberRecords = SFS_NATURAL_RECORD_LIMIT;
  spec.recordSpec.numFields = 0;
  spec.recordSpec.fieldSpecArrayP = (sfs_recordFieldSpec_t*)0;
  spec.primaryIndexSpec.unique = 1;
  spec.primaryIndexSpec.numFields = 0;
  spec.primaryIndexSpec.fieldSpecArrayP = (sfs_indexFieldSpec_t*)0;
}

SFSBaseFileSpec::SFSBaseFileSpec(const String& tName) : typeName(tName)

{
  spec.fileOrganization = sfs_btreeClustered;
  spec.primaryIndexName = "Primary";
  spec.maxNumberRecords = SFS_NATURAL_RECORD_LIMIT;
  spec.recordSpec.numFields = 0;
  spec.recordSpec.fieldSpecArrayP = (sfs_recordFieldSpec_t*)0;
  spec.primaryIndexSpec.unique = 1;
  spec.primaryIndexSpec.numFields = 0;
  spec.primaryIndexSpec.fieldSpecArrayP = (sfs_indexFieldSpec_t*)0;
}

SFSBaseFileSpec::~SFSBaseFileSpec()

{
  if (spec.recordSpec.fieldSpecArrayP != (sfs_recordFieldSpec_t*)0)
    free((char*)spec.recordSpec.fieldSpecArrayP);
  if (spec.primaryIndexSpec.fieldSpecArrayP != (sfs_indexFieldSpec_t*)0)
    free((char*)spec.primaryIndexSpec.fieldSpecArrayP);
}
