/*
 * sfsbatching.C,v 1.7 1994/02/01 22:29:50 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/binding.H>
#include <cdl/sfsbatching.H>
#include <cdl/sfsbases.H>
#include <String.h>
#include <Set.h>
#include <cdl/cdldefs.H>

#ifdef _AIX
#include <List.c>
#include <set.c>

#pragma define (Const_list_of_piter<SFSBatchBase>)
#pragma define (List_of_p<SFSBatchBase>)
#pragma define (List_of_piter<SFSBatchBase>)
#pragma define (Const_listiter<SFSBatchQueue>)
#pragma define (List<SFSBatchQueue>)
#pragma define (Listiter<SFSBatchQueue>)
#pragma define (Set<SFSBatchQueue>)
#pragma define (lnnk_ATTLC<SFSBatchQueue>)
#endif

SFSBatchQueue::SFSBatchQueue()

{
  batching = 0;
}

SFSBatchQueue::SFSBatchQueue(const Transaction& tran)

{
  tid = tran;
  batching = 0;
}

SFSBatchQueue::SFSBatchQueue(const SFSBatchQueue& obj)

{
  tid = obj.tid;
  batching = obj.batching;
  queue = obj.queue;
}

SFSBatchQueue::~SFSBatchQueue()

{
  if (queue)			// If items are on the queue
    {
      SFSBatchBase* dead;

      CDLReport.ReportMsg(ErrorLevels::ErrorMsg, 11002);
      List_of_piter<SFSBatchBase> iter(queue);
      while (iter.remove_next(dead))
	delete dead;
    }
}

SFSBatchQueue& SFSBatchQueue::operator = (const SFSBatchQueue& obj)

{
  tid = obj.tid;
  batching = obj.batching;
  queue = obj.queue;
  return *this;
}

int SFSBatchQueue::operator == (const SFSBatchQueue& obj) const

{
  return tran_TidEqual(tid, obj.tid);
}

Set_or_Bag_hashval Set<SFSBatchQueue>::hash(const SFSBatchQueue& obj)

{
  return (Set_or_Bag_hashval)tran_TidHash(obj.tid);
}

/* 
 * SFSBatchBase and its subclasses
 */

SFSBatchBase::SFSBatchBase()

{
}

SFSBatchBase::~SFSBatchBase()

{
}

SFSBatchInsert::SFSBatchInsert(sfs_ofd_t ofd, SFSBaseRecord& rec) 

{
  batchRep.batchOp = sfs_batchInsert;
  batchRep.params.insertP = (sfs_batchInsertParams_t*)malloc
                                  (sizeof(sfs_batchInsertParams_t));
  batchRep.params.insertP->ofd = ofd;
  batchRep.params.insertP->recordP = rec.Copy();
}

SFSBatchInsert::~SFSBatchInsert()

{
  SFSBaseRecord::Destroy(batchRep.params.insertP->recordP);
  free((char*)batchRep.params.insertP);
}

SFSBatchDelete::SFSBatchDelete(sfs_ofd_t ofd, const char* indexName, 
			       SFSBaseKey& k)

{
    batchRep.batchOp = sfs_batchDeleteByKey;
    batchRep.params.deleteByKeyP = (sfs_batchDeleteByKeyParams_t*)malloc
                                       (sizeof(sfs_batchDeleteByKeyParams_t));
    batchRep.params.deleteByKeyP->ofd = ofd;
    batchRep.params.deleteByKeyP->indexName = (char*)malloc(sizeof(char) *
 					    (strlen(indexName) + 1));
    strcpy(batchRep.params.deleteByKeyP->indexName, indexName);
    batchRep.params.deleteByKeyP->keyP = k.Copy();
}

SFSBatchDelete::~SFSBatchDelete()

{
  free((char*)batchRep.params.deleteByKeyP->indexName);
  SFSBaseKey::Destroy(batchRep.params.deleteByKeyP->keyP);
  free((char*)batchRep.params.deleteByKeyP);
}

SFSBatchRead::SFSBatchRead(sfs_ofd_t ofd, const char* indexName, SFSBaseKey& k,
		           sfs_lockMode_t lockMode, SFSBaseRecord& rec)
{
  batchRep.batchOp = sfs_batchReadByKey;
  batchRep.params.readByKeyP = (sfs_batchReadByKeyParams_t*)malloc
                                    (sizeof(sfs_batchReadByKeyParams_t));
  batchRep.params.readByKeyP->ofd = ofd;
  batchRep.params.readByKeyP->indexName = (char*)malloc(sizeof(char) *
							(strlen(indexName) + 1));
  strcpy(batchRep.params.readByKeyP->indexName, indexName);
  batchRep.params.readByKeyP->keyP = k.Copy();
  batchRep.params.readByKeyP->lockMode = lockMode;
  batchRep.params.readByKeyP->recordP = rec.Copy();
}

SFSBatchRead::~SFSBatchRead()

{
  free((char*)batchRep.params.readByKeyP->indexName);
  SFSBaseKey::Destroy(batchRep.params.readByKeyP->keyP);
  SFSBaseRecord::Destroy(batchRep.params.readByKeyP->recordP);
  free((char*)batchRep.params.readByKeyP);
}

SFSBatchUpdate::SFSBatchUpdate(sfs_ofd_t ofd, const char* indexName,
			       SFSBaseKey& k, SFSBaseRecord& rec)
{
  batchRep.batchOp = sfs_batchUpdateByKey;
  batchRep.params.updateByKeyP = (sfs_batchUpdateByKeyParams_t*)malloc
                                  (sizeof(sfs_batchUpdateByKeyParams_t));
  batchRep.params.updateByKeyP->ofd = ofd;
  batchRep.params.updateByKeyP->indexName = (char*)malloc(sizeof(char) *
					  (strlen(indexName) + 1));
  strcpy(batchRep.params.updateByKeyP->indexName, indexName);
  batchRep.params.updateByKeyP->keyP = k.Copy();
  batchRep.params.updateByKeyP->recordP = rec.Copy();
}

SFSBatchUpdate::~SFSBatchUpdate()

{
  free((char*)batchRep.params.updateByKeyP->indexName);
  SFSBaseKey::Destroy(batchRep.params.updateByKeyP->keyP);
  SFSBaseRecord::Destroy(batchRep.params.updateByKeyP->recordP);
  free((char*)batchRep.params.updateByKeyP);
}

SFSBatchModifyField::SFSBatchModifyField(sfs_ofd_t ofd, const char* indexName,
					 SFSBaseKey& k, const char* fieldName,
					 sfs_modifyOp_t op, unsigned int incLock,
					 sfs_pointer_t value, 
					 sfs_pointer_t newValue,
				         unsigned long fieldLen)
{
  batchRep.batchOp = sfs_batchModifyFieldByKey;
  batchRep.params.modifyFieldByKeyP = (sfs_batchModifyFieldByKeyParams_t*)
                            malloc(sizeof(sfs_batchModifyFieldByKeyParams_t));
  batchRep.params.modifyFieldByKeyP->ofd = ofd;
  batchRep.params.modifyFieldByKeyP->indexName = (char*)malloc(sizeof(char) *
					       (strlen(indexName) + 1));
  strcpy(batchRep.params.modifyFieldByKeyP->indexName, indexName);
  batchRep.params.modifyFieldByKeyP->keyP = k.Copy();
  batchRep.params.modifyFieldByKeyP->fieldName = 
            (char*)malloc(sizeof(char) * (strlen(fieldName) + 1));
  strcpy(batchRep.params.modifyFieldByKeyP->fieldName, fieldName);	
  batchRep.params.modifyFieldByKeyP->modifyOp = op;
  batchRep.params.modifyFieldByKeyP->incrementLock = incLock;
  batchRep.params.modifyFieldByKeyP->valueP = (char*)malloc(sizeof(char) * 
						    (unsigned int)fieldLen);
  memcpy(batchRep.params.modifyFieldByKeyP->valueP, value, (int)fieldLen);
  batchRep.params.modifyFieldByKeyP->newValueP = newValue;  // Don\'t copy this!
}

SFSBatchModifyField::~SFSBatchModifyField()

{
  free((char*)batchRep.params.modifyFieldByKeyP->indexName);
  SFSBaseKey::Destroy(batchRep.params.modifyFieldByKeyP->keyP);
  free((char*)batchRep.params.modifyFieldByKeyP->fieldName);
  free((char*)batchRep.params.modifyFieldByKeyP->valueP);
  free((char*)batchRep.params.modifyFieldByKeyP);
}

