/*
 * ArgList:
 *
 *    Perform various manipulations on a list of arguments.
 *
 * arglist.C,v 1.3 1993/12/03 14:42:31 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <arglist.H>
#include <stdlib.h>

static const int ARGV_ALLOC_SIZE = 16;

ArgList::ArgList()

{
  myArgvAlloc = ARGV_ALLOC_SIZE;
  myArgv = (char**)malloc(sizeof(char*) * myArgvAlloc);
  for (int inc = 0; inc < myArgvAlloc; inc++)
    myArgv[inc] = (char*)0;
}

ArgList::ArgList(const int argc, const char** argv)

{
  int inc;
  String tmpStr;

  for (inc = 0; inc < argc; inc++)
    {
      tmpStr = argv[inc];
      args.put(tmpStr);
    }
  myArgvAlloc = ARGV_ALLOC_SIZE;
  myArgv = (char**)malloc(sizeof(char*) * myArgvAlloc);
  for (inc = 0; inc < myArgvAlloc; inc++)
    myArgv[inc] = (char*)0;
}

ArgList::ArgList(const ArgList& list)

{
  args = list.args;
  myArgvAlloc = ARGV_ALLOC_SIZE;
  myArgv = (char**)malloc(sizeof(char*) * myArgvAlloc);
  for (int inc = 0; inc < myArgvAlloc; inc++)
    myArgv[inc] = (char*)0;
}

ArgList::ArgList(const ArgList& list, const String& options)

{
  int curChar;
  char tmpStr[3];
  long strLen = options.length();
  int listLen = list.GetArgc();

  tmpStr[0] = '-';
  tmpStr[2] = '\0';
  for (int inc = 0; inc < list.args.length(); inc++)
    {
      for (curChar = 0; curChar < strLen; curChar++)
	{
	  tmpStr[1] = options[curChar];
	  if (list.args[inc].match(tmpStr) >= 2)
	    args.put(list.args[inc]);
	}
    }
  myArgvAlloc = list.args.length() + 2;
  myArgv = (char**)malloc(sizeof(char*) * myArgvAlloc);
  for (inc = 0; inc < myArgvAlloc; inc++)
    myArgv[inc] = (char*)0;
}

ArgList::~ArgList()

{
  for (int inc = 0; inc < myArgvAlloc; inc++)
    if (myArgv[inc] != (char*)0)
      free(myArgv[inc]);
  free((char*)myArgv);
}

ArgList& ArgList::operator = (const ArgList& list)

{
  args = list.args;
  return *this;
};

void ArgList::Remove(const String&)

{
  Listiter<String> iter(args);
  while (args)
    iter.remove_next();
}

char* const* ArgList::GetArgv()

{
  int inc;

  for (inc = 0; inc < myArgvAlloc; inc++)
    if (myArgv[inc] == (char*)0)
      free(myArgv[inc]);
  if (args.length() >= myArgvAlloc)
    {
      myArgvAlloc = args.length() + 2;
      myArgv = (char**)realloc((char*)myArgv, sizeof(char*) * myArgvAlloc);
    }
  for (inc = 0; inc < args.length(); inc++)
    myArgv[inc] = strdup(args[inc]);
  for (; inc < myArgvAlloc; inc++)
    myArgv[inc] = (char*)0;
  return myArgv;
}

void ArgList::PutFront(const String& str)
     
{ 
  Listiter<String> iter(args);
  iter.insert_next(str);
}

void ArgList::PutEnd(const String& str)

{ 
  Listiter<String> iter(args);
  iter.end_reset();
  iter.insert_prev(str);
}

ostream& operator << (ostream& out, const ArgList& argL)

{
  Const_listiter<String> iter(argL.args);
  String tmpStr;
  out << "ArgList contents:" << endl;
  while (iter.next(tmpStr))
    out << tmpStr << endl;
  return out;
}
