/*
 * ArgList:
 *
 *    This class represents a C argument list (argc and argv). It can be
 * used to add strings to the beginning and end of the list, and to get only
 * options beginning with certain letters. 
 *
 * arglist.H,v 1.4 1994/04/07 19:25:06 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_ARG_LIST_H
#define CDL_ARG_LIST_H

extern "C"
{
#include <pthread.h>
}
#include <iostream.h>
#include <String.h>
#include <Path.h>
#include <List.h>

class ArgList

{
  friend ostream& operator << (ostream&, const ArgList&);
public:
  ArgList();
  ArgList(const int argc, const char** argv);
  ArgList(const ArgList&);
  ArgList(const ArgList&, const String& options);
  ~ArgList();
  ArgList& operator = (const ArgList&);
  void PutFront(const String&);
  void PutEnd(const String&);
  void Remove(const String&);
  const String& Get(const int i) const;
  const int GetArgc() const;
  char* const* GetArgv();
private:
  List<String> args;
  char** myArgv;
  int myArgvAlloc;
};

inline const String& ArgList::Get(const int i) const

{ return args[i]; }

inline const int ArgList::GetArgc() const

{ return args.length(); }

#endif
