/*
 * CDLAuxParser:
 *
 *    This parser parses the auxiliary file.
 *
 * cdlauxparser.C,v 1.7 1994/02/01 22:32:29 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <cdlauxparser.H>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <arglist.H>
#include <sfsparsedefs.H>
#include <mainparsedefs.H>
#include <cdlfiledef.H>
#include <cdlmainparser.H>
#include <parselist.H>
#include <objdce/error.H>

CDLAuxParser::CDLAuxParser(const ArgList& args)
    : CDLParser(args)

{
}

CDLAuxParser::~CDLAuxParser()

{
}

Boolean CDLAuxParser::Parse(const String& fileName,
			    List_of_p<CDLDataGenerator>& generators)

{
  String tmpName;
  CDLRecord* curRec;
  List_of_piter<CDLDataGenerator> iter(generators);
  CDLDataGenerator* curGen;
  List_of_piter<CDLFileDef> fileIter(fileList);
  CDLFileDef* curFile;
  Boolean stillOK;
  String errMsg;
  
  CDLReport.VReport(ErrorLevels::TraceMsg, "Parsing File: %s\n", 
		    (const char *)fileName);
  if (PreProcess(fileName, tmpName))
    {
      CDLSfsin = fopen(tmpName, "r");
      if (CDLSfsin == (FILE*)0)
        {
	  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1000, (const char*)tmpName);
          return false;
        }
      else
        {
	  CDLSfsSetName(fileName);
	  CDLSfsFailed = 0;
          int status = CDLSfsparse();
          fclose(CDLSfsin);
          unlink(tmpName);
          if ((status != 0) || CDLSfsFailed)
            {
	      CDLReport.ReportMsg(ErrorLevels::ErrorMsg, 500);
              return false;   
	    }
	  else
	    {
	      stillOK = true;
	      while (stillOK && iter.next(curGen))
		{
		  if (!(curGen->Start(fileName)))
		    stillOK = false;
		  else
		    {
		      fileIter.reset();
		      while (fileIter.next(curFile))
			{
			  curRec = CDLMainGetParser()->Find(curFile->RecName());
			  if (curRec == (CDLRecord*)0)
			    {
			      CDLReport.VGetMsg(1100, errMsg,
						(const char*)curFile->RecName());
			      CDLPError(errMsg);
			      stillOK = false;
			    }
			  else if (!(curGen->Generate(*curRec, *curFile)))
			    stillOK = false;
			}
		    }
		  curGen->End();
		}
	      return stillOK;
	    }
        }
    }
  else
    return false;
}

Boolean CDLAuxParser::AddFile(CDLFileDef* newFile)

{
  Boolean found = false;
  Boolean retVal;
  List_of_piter<CDLFileDef> iter(fileList);
  CDLFileDef* curFile;
  String errMsg;

  while (!found && iter.next(curFile))
    if (*curFile == *newFile)
      found = true;
  if (found)
    {
      CDLReport.VGetMsg(1003, errMsg, (const char*)newFile->Name());
      CDLPError(errMsg);
      retVal = false;
    }
  else if (CDLMainGetParser()->Find(newFile->Name()) != (CDLRecord*)0)
    {
      found = true;
      CDLReport.VGetMsg(1101, errMsg, (const char*)newFile->Name());
      CDLPError(errMsg);
      retVal = false;
    }
  else
    {
      fileList.put(newFile);
      retVal = true;
    }
  return retVal;
}
