/*
 * CDLAuxParser:
 *
 *    This class parses the auxiliary CDL file, the file which contains
 * definitions of files and their indices. After preprocessing, it builds a
 * list of CDLFileDef objects, which consist of lists of CDLIndexDef objects.
 * Here, however, instead of activating several CDLGenerator objects, we
 * simply return the list to our caller, who presumably is already a generator
 * that has been activitated by CDLMainParser.
 *
 * cdlauxparser.H,v 1.5 1994/04/07 19:25:13 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_AUX_PARSER_H
#define CDL_AUX_PARSER_H

extern "C"
{
#include <pthread.h>
}
#include <cdlparser.H>
#include <objdce/objtranparams.H>
#include <cdl/cdldefs.H>
#include <cdlgenerator.H>
#include <Path.h>
#include <List.h>

class CDLFileDef;

class CDLAuxParser : public CDLParser

{
public:
  CDLAuxParser(const ArgList&);
  virtual ~CDLAuxParser();
  Boolean Parse(const String& fileName, 
		List_of_p<CDLDataGenerator>& generators);
  Boolean AddFile(CDLFileDef*);
private:
  List_of_p<CDLFileDef> fileList;
};

#endif
 
