/* 
 * CDLClassGenerator:
 *
 *    This file generates a C++ class based on the field definitions in
 * the CDL file.
 *
 * cdlclassgenerator.C,v 1.18 1994/04/07 19:25:16 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <cdlclassgenerator.H>
#include <cdlfield.H>
#include <cdlrecord.H>
#include <parselist.H>
#include <cdlconstinfo.H>
#include <cdlsuffixes.H>
#include <objdce/objtranparams.H>

/*
 * CDLClassGenerator
 *
 *    Set a few defaults, then pass the arguments up to CDLGenerator.
 */

CDLClassGenerator::CDLClassGenerator(const ArgList& args, Boolean usedIdl)
    : CDLGenerator(args)
{
  createIdl = usedIdl;
  mainName = "";
  inclName = "";
}

CDLClassGenerator::~CDLClassGenerator()

{
}

/*
 * Start
 *
 *    Open the output files, and write the headers of both the .C
 * and .H files. This means a few comments and some #include directives.
 *    The .H file will also contain definitions of our constants, with the
 * excptions of string constants, which are "const extern" and have their
 * values defined in the .C file. This is all accomplished by the
 * GenConstants function.
 */

Boolean CDLClassGenerator::Start(const String& fileName)

{
  int inc;
  String importName;
  String symbolName;

  MakeFileName(fileName, inclName, CDLCppHeaderSuffix);
  MakeFileName(fileName, mainName, CDLCppSourceSuffix);
  // outFile is the .C file
  outFile.open(mainName);
  if (outFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1001, (const char*)mainName);
      return false;
    }
  // inclFile is the .H file
  inclFile.open(inclName);
  if (inclFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1001, (const char*)inclName);
      return false;
    }
  else
    {
      /**** Header for the header file ***/
      inclFile << "/*" << endl;
      inclFile << " * Class generated by Citi-DDL:" << endl;
      inclFile << " * cdlclassgenerator.C,v 1.18 1994/04/07 19:25:16 gjb Exp" << endl;
      inclFile << " */\n" << endl;
      
      MakeFileName(fileName, symbolName, CDLHeaderIfdefSuffix);
      inclFile << "#ifndef " << symbolName << endl;
      inclFile << "#define " << symbolName << "\n" << endl;

      inclFile << "extern \"C\" {" << endl;
      inclFile << "#include <pthread.h>" << endl;
      inclFile << "}" << endl;
      inclFile << "#include <String.h>" << endl;
      inclFile << "#include <Path.h>" << endl;
      inclFile << "#include <List.h>" << endl;
      inclFile << "#include <objdce/defs.H>" << endl;
      for (inc = 0; inc < NumMainIncludeNames(); inc++)
	{
	  MakeFileName(GetMainIncludeName(inc), importName, 
		       CDLCppHeaderSuffix);
	  inclFile << indent << "#include <" << importName << ">" << endl;
	}
      if (createIdl)
	{
	  String idlInclName;
	  MakeFileName(fileName, idlInclName, CDLIdlHeaderSuffix);
	  inclFile << "#include <" << idlInclName << ">" << endl;
	  inclFile << "#include <dce/idlbase.h>\n" << endl;
	}

      /**** Header for the source file ****/
      outFile << "/*" << endl;
      outFile << " * Class generated by Citi-DDL:" << endl;
      outFile << " * cdlclassgenerator.C,v 1.18 1994/04/07 19:25:16 gjb Exp" << endl;
      outFile << " */\n" << endl;
      outFile << "#include <stdlib.h>" << endl;
      outFile << "#include <string.h>" << endl;
      outFile << "#include <iostream.h>" << endl;
      outFile << "#include <" << inclName << ">\n" << endl;

      return GenConstants();
    }
}

/*
 * End
 *    Finish up the output files and close them. Also, write any necessary
 * stuff at the end of the file.
 */

Boolean CDLClassGenerator::End()

{
  inclFile << "#endif" << endl;
  outFile << flush;
  outFile.close();
  inclFile << flush;
  inclFile.close();
  return true;
}

/*
 * DeleteFiles
 *    Something went wrong, so delete the output files.
 */

void CDLClassGenerator::DeleteFiles()

{
  outFile.close();
  inclFile.close();
  unlink(mainName);
  unlink(inclName);
}

/*
 * GenConstants
 *    All the constants defined in this .cdl file -- but not the "imported"
 * constants that were defined in imported .cdl files -- are written out
 * by this function. Integer constants are "static const", but string and real
 * constants are "extern const" and are defined in the .C file. We make
 * integer constants static so people can use them for array sizes in their
 * code.
 */

Boolean CDLClassGenerator::GenConstants()

{
  CDLConstInfo curConst;
  Mapiter<Symbol, CDLConstInfo> constIter(CDLConstants);
  Map_pair<Symbol, CDLConstInfo>* curPair;
  String modName;
  String typeName;

  while ((curPair = constIter.next()) != 0)
    {
      curConst = curPair->value;
      if (!curConst.Imported())
	{
	  switch(curConst.Type())
	    {
	    case CDLField::REAL:
	      inclFile << "extern const double " << curConst.Name();
	      inclFile << ";" << endl;
	      outFile << "const double " << curConst.Name();
	      outFile << " = " << curConst.Value() << ";" << endl;
	      break;
	    case CDLField::STRING:
	      inclFile << "extern const char* const " << curConst.Name();
	      inclFile << ";" << endl;
	      outFile << "const char* const " << curConst.Name();
	      outFile << " = \"" << curConst.Value() << "\";" << endl;
	      break;
	    case CDLField::CHAR:
	      inclFile << "extern const char " << curConst.Name();
	      inclFile << ";" << endl;
	      outFile << "const char " << curConst.Name();
	      outFile << " = \'" << curConst.Value() << "\';" << endl;
	      break;
	    case CDLField::LONG:
	    case CDLField::SHORT:
	      if (curConst.Modifier() == CDLField::UNSIGNED)
		modName = "unsigned ";
	      else
		modName = "";
	      if (curConst.Type() == CDLField::LONG)
		typeName = modName + "long";
	      else
		typeName = modName + "short";
	      inclFile << "static const " << typeName << " " << curConst.Name();
	      inclFile << " = " << curConst.Value() << ";" << endl;
	      break;
	    default:
              /* There are other checks for this -- it won't happen */
	      break;
	    }
	}
    }
  inclFile << endl;
  outFile << endl;
  return true;
}

/*
 * Generate
 *    The main code generation routine. This writes out all the code we
 * need for the record specified.
 */

Boolean CDLClassGenerator::Generate(CDLRecord& rec)

{
  Boolean retVal;
  retVal = GenHeader(rec);
  if (retVal)
    retVal = GenMain(rec);
  if (retVal)
    retVal = GenInlines(rec);
  if (retVal)
    retVal = GenOstream(rec);
  if (retVal && createIdl && rec.MakeIdlArray())
    retVal = GenArrayPacking(rec);
  return retVal;
}
  
/*
 * GetTypeName
 *    Given a type identifier, generate the text that describes how
 * the type will be declared in C++. If the "ptr" argument is true,
 * generate the type as it will appear in the header, or external class
 * interface. Otherwise, generate it as it will appear in the private part
 * of the class where the data is stored.
 */

void CDLClassGenerator::GetTypeName(CDLField* field, String& name,
				    Boolean ptr)

{
  String modName;
  
  switch(field->Modifier())
    {
    case CDLField::UNSIGNED:
      modName = "unsigned ";
      break;
    case CDLField::SIGNED:
    case CDLField::NONE:
      modName = "";
      break;
    }
  switch(field->Type())
    {
    case CDLField::REAL:
      name = "double";
      break;
    case CDLField::BYTES:
    case CDLField::VARBYTES:
      if (ptr)
	name = "unsigned char*";
      else
	name = "unsigned char";
      break;
    case CDLField::STRING:
      if (ptr)
	name = "String";
      else
	name = "char";
      break;
    case CDLField::RECORD:
      if (field->Record() == (CDLRecord*)0)
	name = "Invalid";
      else
	{
	  name = field->Record()->Name();
	  if (ptr)
	    name += "&";
	}
      break;
    case CDLField::HYPER:
      if (field->Modifier() == CDLField::UNSIGNED)
	name = "idl_uhyper_int";
      else
	name = "idl_hyper_int";
      break;
    case CDLField::LONG:
      name = modName + "long";
      break;
    case CDLField::SHORT:
      name = modName + "short";
      break;
    case CDLField::CHAR:
      name = modName + "char";
      break;
    case CDLField::CHARRAY:
      name = modName + "char*";
      break;
    case CDLField::ENUM:
      name = field->RealTypeName();
      break;
    }
}
      
/* 
 * MakeFieldName
 *    Make the name of the storage for this field. If the field is stored
 * in an idl struct, use that field. Otherwise, generate the name as if the
 * data is stored in the class itself. 
 */

void CDLClassGenerator::MakeFieldName(CDLField* field,
				      String& out)
{
  if (createIdl)
    {
      out = CDLCppIdlRecordName;
      out += ".";
      out += field->Name();
    }
  else
    out = field->Name();
  if (!createIdl)
    {
      switch(field->Type())
	{
	case CDLField::REAL:
	  out += "real";
	  break;
	case CDLField::RECORD:
	  out += "record";
	  break;
	case CDLField::BYTES:
	  out += "bytes";
	  break;
	case CDLField::VARBYTES:
	  out += "varBytes";
	  break;
	case CDLField::STRING:
	  out += "string";
	  break;
	case CDLField::HYPER:
	  out += "hyper";
	  break;
	case CDLField::LONG:
	  out += "long";
	  break;
	case CDLField::SHORT:
	  out += "short";
	  break;
	case CDLField::CHAR:
	case CDLField::CHARRAY:
	  out += "char";
	  break;
	case CDLField::ENUM:
	  out += field->RealTypeName();
	  break;
	}
    }
}
      
/*
 * GenHeader
 *    Generate the class defnition for this record and write it to the
 * header file.
 */

Boolean CDLClassGenerator::GenHeader(CDLRecord& rec)

{
  CDLField* curField;
  int inc;
  String typeName;
  CDLField::FieldType fType;
  String baseRecName;
  baseRecName = rec.Name();
  baseRecName += CDLIdlRecordSuffix;
  Boolean isArray;

  inclFile << "/*" << endl;
  inclFile << " * " << rec.Name() << ":" << endl;
  inclFile << " * " << endl;
  inclFile << " * " << rec.Documentation() << endl;
  inclFile << " */\n" << endl;

  inclFile << indent << "class " << rec.Name() << endl;
  inclFile << indent << "{" << endl;
  inclFile << indent << "public:" << endl;
  Indent();
  
  inclFile << indent << rec.Name() << "();" << endl;
  inclFile << indent << rec.Name() << "(const " << rec.Name() << "&);" << endl;
  if (createIdl)
    {
      inclFile << indent << rec.Name() << "(const " << baseRecName;
      inclFile << "&);" << endl;
    }
  inclFile << indent << "virtual ~" << rec.Name() << "();" << endl;
  inclFile << indent << "void Initialize();" << endl;
  inclFile << indent << rec.Name() << "& operator = (const ";
  inclFile << rec.Name() << "&);" << endl;
  if (createIdl)
    {
      inclFile << indent << rec.Name() << "& operator = (const " << baseRecName;
      inclFile << "&);" << endl;
      inclFile << indent << "operator " << baseRecName << "*();" << endl;
      inclFile << indent;
      inclFile << "operator const " << baseRecName << "*() const;" << endl;
      inclFile << indent << "void Copy(" << baseRecName << "&);" << endl;
    }
  inclFile << endl;	// To make it look nice

   // lcp 11/2/92 add a const == operator 
  inclFile << indent << "int operator == (const " << rec.Name();
  inclFile << "&) const;" << endl;
  inclFile << indent << "int operator != (const " << rec.Name();
  inclFile << "&) const;" << endl;

   // alina 09/15/93 add operator <<
  inclFile << indent << "void Print(ostream&, String, const ";
  inclFile << rec.Name() << "&) const;" << endl;
  inclFile << indent << "friend ostream & operator<<(ostream&, const ";
  inclFile << rec.Name() << "&);" << endl;

  /* Method declarations */
  for (inc = 0; inc < rec.NumFields(); inc++)
    {
      curField = rec.Field(inc);
      GetTypeName(curField, typeName, true);
      fType = curField->Type();
      isArray = (curField->ArraySize() > 1);

      /* Write accessor */
      if (IsRecord(fType))
	{
	  if (curField->Record() == (CDLRecord*)0)
	    return false;
	  inclFile << indent << curField->Record()->Name();
	  inclFile << " " << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element";
	  inclFile << ") const;" << endl;
	}
      else
	{
	  inclFile << indent;
	  if (IsAtomic(fType) || IsPointer(fType))
	    inclFile << "const ";
	  inclFile << typeName << " " << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element";
	  inclFile << ")";
	  if (IsAtomic(fType) || IsPointer(fType))
	    inclFile << " const";
	  inclFile << ";" << endl;
	  if (IsByteArray(fType))
	    {
	      inclFile << indent << "const unsigned int " << curField->Name();
	      inclFile << "Length(";
	      if (isArray)
		inclFile << "int element";
	      inclFile << ") const;" << endl;
	    }
	}

      /* Write mutator */
      switch(fType)
	{
	case CDLField::RECORD:
	  inclFile << indent << "void " << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element, ";
	  inclFile << typeName << ");" << endl;
	  break;
	case CDLField::STRING:
	  inclFile << indent << "void " << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element, ";
	  inclFile << "const String&);" << endl;
	  break;
	case CDLField::BYTES:
	case CDLField::VARBYTES:
	case CDLField::CHARRAY:
	  inclFile << indent << "void " << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element, ";
	  inclFile << "const " << typeName;
	  inclFile << ", const unsigned int length = ";
	  inclFile << curField->Length() << ");" << endl;
	  break;
	default:
	  inclFile << indent << "void " << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element, ";
	  inclFile << "const " << typeName << ");" << endl;
	  break;
	}
    }
  if (createIdl && rec.MakeIdlArray())
    {
      String arrayName = baseRecName;
      arrayName += CDLIdlArraySuffix;
      /* Add functions to convert List to IDL array */
      inclFile << endl;		// Make it look nice
      inclFile << indent << "static Boolean ServerListToArray(const List<";
      inclFile << rec.Name() << ">& in, " << arrayName << "& out);" << endl;
      inclFile << indent << "static Boolean ClientListToArray(const List<";
      inclFile << rec.Name() << ">& in, " << arrayName << "& out);" << endl;
      inclFile << indent << "static Boolean ArrayToList(const ";
      inclFile << arrayName << "& in, List<" << rec.Name();
      inclFile << ">& out);" << endl;
      inclFile << indent << "static Boolean ServerArrayFree(" << arrayName;
      inclFile << "& dead);" << endl;
      inclFile << indent << "static Boolean ClientArrayFree(" << arrayName;
      inclFile << "& dead);" << endl;
    }
  Outdent();
  inclFile << indent << "private:" << endl;
  Indent();
  if (createIdl)
    {
      inclFile << indent << baseRecName << " " << CDLCppIdlRecordName;
      inclFile << ";" << endl;
    }
  else
    {
      /* Write private data members */
      for (inc = 0; inc < rec.NumFields(); inc++)
	{
	  curField = rec.Field(inc);
	  GetTypeName(curField, typeName, false);
	  fType = curField->Type();
	  isArray = (curField->ArraySize() > 1);

	  if (!IsPointer(fType))
	    {
	      inclFile << indent << typeName << " ";
	      inclFile << curField->Name();
	      if (isArray)
		inclFile << "[" << curField->ArraySize() << "];" << endl;
	      else
		inclFile << ";" << endl;
	    }
	  else if (IsString(fType))
	    {
	      inclFile << indent << typeName << " " << curField->Name();
	      if (isArray)
		inclFile << "[" << curField->ArraySize() << "]" << endl;
	      inclFile << "[" << curField->Length() + 1 << "];";
	    }
	  else
	    {	// It\'s a byte array
	      inclFile << indent << typeName << " " << curField->Name();
	      inclFile << "[" << curField->Length() << "];" << endl;
	      inclFile << indent << "unsigned int " << curField->Name();
	      inclFile << CDLByteArrayLengthSuffix << ";" << endl;
	    }
	}
    }
  Outdent();
  inclFile << indent << "};\n" << endl;
  return true;
}

/*
 * GenMain
 *    Write the main body of code for this class, which entails most of
 * the mutator functions. These go in the .C file.
 */

Boolean CDLClassGenerator::GenMain(CDLRecord& rec)

{
  String typeName;
  CDLField* curField;
  int inc;
  String fieldName;
  Boolean isArray;

  String idlRecName = rec.Name();
  idlRecName += CDLIdlRecordSuffix;
  
  outFile << "/*" << endl;
  outFile << " * " << rec.Name() << ":" << endl;
  outFile << " * " << endl;
  outFile << " * " << rec.Documentation() << endl;
  outFile << " */\n" << endl;
  
  /* Record::Record() */
  outFile << indent << rec.Name() << "::" << rec.Name() << "()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "Initialize();" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

  /* Record::Record(const Record&) */
  outFile << indent << rec.Name() << "::" << rec.Name() << "(const ";
  outFile << rec.Name() << "& rec)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  if (createIdl)
    {
      outFile << indent << CDLCppIdlRecordName << " = rec.";
      outFile << CDLCppIdlRecordName << ";" << endl;	// No pointers, 
    }
  else							// so this works
    {
      outFile << indent<< "*this = rec;" << endl;
    }
  Outdent();
  outFile << indent << "}\n" << endl;
  
  /* Record::Record(const RecordBase&) */
  if (createIdl)
    {
      outFile << indent << rec.Name() << "::" << rec.Name() << "(const ";
      outFile << idlRecName << "& rec)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << CDLCppIdlRecordName << " = rec;" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;
    }

  /* Record::~Record() */
  outFile << indent << rec.Name() << "::~" << rec.Name() << "()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

  /* Record::Initialize() */
  outFile << indent << "void " << rec.Name() << "::Initialize()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  if (createIdl)
    {
      outFile << indent << "memset((char*)&" << CDLCppIdlRecordName;
      outFile << ", 0, sizeof(" << CDLCppIdlRecordName << "));" << endl;
    }
  else
    {
      outFile << indent << "memset((char*)this, 0, sizeof(*this));" << endl;
    }
  Outdent();
  outFile << indent << "}\n" << endl;

  /* Record& Record::operator = (const Record&) */
  outFile << indent << rec.Name() << "& " << rec.Name() << "::operator = (const ";
  outFile << rec.Name() << "& rec)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  if (createIdl)
    {
      outFile << indent << CDLCppIdlRecordName << " = rec.";
      outFile << CDLCppIdlRecordName << ";" << endl;	// No pointers, 
    }
  else							// so this works
    {
      outFile << indent<< "*this = rec;" << endl;
    }
  outFile << indent << "return *this;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

  /* Record& Record::operator = (const RecordBase&) */
  outFile << indent << rec.Name() << "& " << rec.Name() << "::operator = (const ";
  outFile << idlRecName << "& rec)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << CDLCppIdlRecordName << " = rec;" << endl;
  outFile << indent << "return *this;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

  /* void Record::Copy(RecordBase&) */
  outFile << indent << "void " << rec.Name() << "::Copy(";	
  outFile << idlRecName << "& rec)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "rec = " << CDLCppIdlRecordName << ";" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

  /* int Record::operator == (const Record&) const */
  outFile << indent << "int " << rec.Name() << "::operator == (const ";
  outFile << rec.Name() << "& rec) const" << endl;
  outFile << indent << "{" << endl;
  Indent();
  curField = rec.Field(0);		// Just compare the first field
  GetTypeName(curField, typeName, true);
  MakeFieldName(curField, fieldName);
  isArray = (curField->ArraySize() > 1);
  outFile << indent << "int match = 1;" << endl;
  if (isArray)
    {
      outFile << indent << "for (int inc = 0; match && (inc < ";
      outFile << curField->ArraySize() << "); inc++)" << endl;
      outFile << indent << "{" << endl;
      Indent();
    }
  switch (curField->Type())
    {
    case CDLField::STRING:
      if (isArray)
	fieldName += "[inc]";
      outFile << indent << "if (strcmp((const char*)" << fieldName;
      outFile << ", (const char*)(rec." << fieldName << ")))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << indent << "if (memcmp((const void*)" << fieldName;
      outFile << ", (const void*)&rec." << fieldName;
      outFile << ", " << curField->Length() << "))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::VARBYTES:
      outFile << indent << "if (memcmp((const void*)" << fieldName;
      outFile << ", (const void*)&rec." << fieldName << ", (int)";
      outFile << fieldName << CDLByteArrayLengthSuffix << "))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::HYPER:
      if (isArray)
	fieldName += "[inc]";
      outFile << indent << "if ((" << fieldName << ".high != rec.";
      outFile << fieldName << ".high) || (" << fieldName << ".low != rec.";
      outFile << fieldName << ".low))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::RECORD:
      fieldName = curField->Name();
      if (isArray)
	{
	  outFile << indent << "if (" << fieldName << "(inc) != rec.";
	  outFile << fieldName << "(inc))" << endl;
	}
      else
	{
	  outFile << indent << "if (" << fieldName << "() != rec.";
	  outFile << fieldName << "())" << endl;
	}
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    default:
      if (isArray)
	fieldName += "[inc]";
      outFile << indent << "if (" << fieldName << " != rec.";
      outFile << fieldName << ")" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    }
  if (isArray)
    {
      Outdent();
      outFile << indent << "}" << endl;
    }
  outFile << indent << "return match;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
  /* int Record::operator != (const Record&) const */
  outFile << indent << "int " << rec.Name() << "::operator != (const ";
  outFile << rec.Name() << "& rec) const" << endl;
  outFile << indent << "{" << endl;
  Indent();
  curField = rec.Field(0);		// Just compare the first field
  GetTypeName(curField, typeName, true);
  MakeFieldName(curField, fieldName);
  isArray = (curField->ArraySize() > 1);
  outFile << indent << "int match = 1;" << endl;
  if (isArray)
    {
      outFile << indent << "for (int inc = 0; match && (inc < ";
      outFile << curField->ArraySize() << "); inc++)" << endl;
      outFile << indent << "{" << endl;
      Indent();
    }
  switch (curField->Type())
    {
    case CDLField::STRING:
      if (isArray)
	fieldName += "[inc]";
      outFile << indent << "if (!strcmp((const char*)" << fieldName;
      outFile << ", (const char*)(rec." << fieldName << ")))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << indent << "if (!memcmp((const void*)" << fieldName;
      outFile << ", (const void*)&rec." << fieldName;
      outFile << ", " << curField->Length() << "))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::VARBYTES:
      outFile << indent << "if (!memcmp((const void*)" << fieldName;
      outFile << ", (const void*)&rec." << fieldName << ", (int)";
      outFile << fieldName << CDLByteArrayLengthSuffix << "))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::HYPER:
      if (isArray)
	fieldName += "[inc]";
      outFile << indent << "if ((" << fieldName << ".high == rec.";
      outFile << fieldName << ".high) || (" << fieldName << ".low == rec.";
      outFile << fieldName << ".low))" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    case CDLField::RECORD:
      fieldName = curField->Name();
      if (isArray)
	{
	  outFile << indent << "if (" << fieldName << "(inc) == rec.";
	  outFile << fieldName << "(inc))" << endl;
	}
      else
	{
	  outFile << indent << "if (" << fieldName << "() == rec.";
	  outFile << fieldName << "())" << endl;
	}
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    default:
      if (isArray)
	fieldName += "[inc]";
      outFile << indent << "if (" << fieldName << " == rec.";
      outFile << fieldName << ")" << endl;
      Indent();
      outFile << indent << "match = 0;" << endl;
      Outdent();
      break;
    }
  if (isArray)
    {
      Outdent();
      outFile << indent << "}" << endl;
    }
  outFile << indent << "return match;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
  /* Mutator functions */
  for (inc = 0; inc < rec.NumFields(); inc++)
    {
      curField = rec.Field(inc);
      isArray = (curField->ArraySize() > 1);
      GetTypeName(curField, typeName, true);		// We use pointer types
      MakeFieldName(curField, fieldName);
      outFile << indent << "void " << rec.Name() << "::" << curField->Name();
      switch(curField->Type())
	{
	case CDLField::BYTES:
	case CDLField::VARBYTES:
	  if (isArray)
	    {
	      outFile << "(int element, const " << typeName;
	      outFile << " val, const unsigned int length)" << endl;
	      outFile << indent << "{" << endl;
	      Indent();
	      outFile << indent << "memcpy((char*)(" << fieldName;
	      outFile << "[element]), (const char*)val, (int)length);" << endl;
	      outFile << indent << fieldName << CDLByteArrayLengthSuffix;
	      outFile << "[element] = length;" << endl;
	      Outdent();
	      outFile << indent << "}\n" << endl;
	    }
	  else
	    {
	      outFile << "(const " << typeName;
	      outFile << " val, const unsigned int length)" << endl;
	      outFile << indent << "{" << endl;
	      Indent();
	      outFile << indent << "memcpy((char*)" << fieldName;
	      outFile << ", (const char*)val, (int)length);" << endl;
	      outFile << indent << fieldName << CDLByteArrayLengthSuffix;
	      outFile << " = length;" << endl;
	      Outdent();
	      outFile << indent << "}\n" << endl;
	    }
	  break;
	case CDLField::CHARRAY:
	  if (isArray)
	    {
	      outFile << "(int element, const " << typeName;
	      outFile << " val, const unsigned int length)" << endl;
	      outFile << indent << "{" << endl;
	      Indent();
	      outFile << indent << "memcpy((char*)(" << fieldName;
	      outFile << "[element]), (const char*)val, (int)length);" << endl;
	      Outdent();
	      outFile << indent << "}\n" << endl;
	    }
	  else
	    {
	      outFile << "(const " << typeName;
	      outFile << " val, const unsigned int length)" << endl;
	      outFile << indent << "{" << endl;
	      Indent();
	      outFile << indent << "memcpy((char*)" << fieldName;
	      outFile << ", (const char*)val, (int)length);" << endl;
	      Outdent();
	      outFile << indent << "}\n" << endl;
	    }
	  break;
	case CDLField::STRING:
	  outFile << "(";
	  if (isArray)
	    {
	      outFile << "int element, ";
	      fieldName += "[element]";
	    }
	  outFile << "const String& str)" << endl;
	  outFile << indent << "{" << endl;
	  Indent();
	  outFile << indent << "strncpy((char*)" << fieldName << ", str, ";
	  outFile << curField->Length() << ");" << endl;
	  outFile << indent << fieldName << "[" << curField->Length();
	  outFile << "] = \'\\0\';" << endl;
	  Outdent();
	  outFile << indent << "}\n" << endl;
	  break;
	case CDLField::RECORD:
	  outFile << "(";
	  if (isArray)
	    {
	      outFile << "int element, ";
	      fieldName += "[element]";
	    }
	  outFile << typeName << " obj)" << endl;
	  outFile << indent << "{" << endl;
	  Indent();
	  outFile << indent << curField->Record()->Name();
	  outFile << CDLIdlRecordSuffix << "* objBase = obj;" << endl;
	  outFile << indent << fieldName << " = *objBase;" << endl;
	  Outdent();
	  outFile << indent << "}\n" << endl;
	  break;
	default:
	  outFile << "(";
	  if (isArray)
	    {
	      outFile << "int element, ";
	      fieldName += "[element]";
	    }
	  outFile << "const " << typeName << " val)" << endl;
	  outFile << indent << "{" << endl;
	  Indent();
	  outFile << indent << fieldName << " = val;" << endl;
	  Outdent();
	  outFile << indent << "}\n" << endl;
	  break;
	}
    }

  /* Template definition sybols for various OSes */
  outFile << "#ifdef _AIX" << endl;
  outFile << "#ifndef USL_LIST_C_INCLUDED" << endl;
  outFile << "#define USL_LIST_C_INCLUDED" << endl;
  outFile << "#include <List.c>" << endl;
  outFile << "#endif" << endl;
  outFile << "#pragma define (Const_listiter<" << rec.Name() << ">)" << endl;
  outFile << "#pragma define (List<" << rec.Name() << ">)" << endl;
  outFile << "#pragma define (Listiter<" << rec.Name() << ">)" << endl;
  outFile << "#pragma define (lnnk_ATTLC<" << rec.Name() << ">)" << endl;
  outFile << "#endif" << endl;

  return true;
}

/*
 * GenOstream
 *    This generates code for the << operator, which "writes" to an ostream
 * object. This operator just calls the "Print" function, which actually
 * generates the code for a particular record. Print is also used
 * recursively to print out nested records.
 */

Boolean CDLClassGenerator::GenOstream(CDLRecord& rec)

{
  CDLField* curField;
  int inc;
  int isArray;

  /* Print function for the ostream& operator<< */
  int fl = 0;
  int flr = 0;
  int hyper_fl = 0; 
  int hyperarr_fl = 0;
  int uhyper_fl = 0; 
  int uhyperarr_fl = 0;
  outFile << indent << "void " <<rec.Name()<< "::Print(ostream& os, String s, ";
  outFile << "const " << rec.Name() << "& rec) const" << endl;
  outFile << indent << "{" << endl;
  for (inc = 0; inc < rec.NumFields(); inc++)
    {
      curField = rec.Field(inc);
      int md = curField->Modifier();
      int tp = curField->Type();
      if (tp != CDLField::RECORD)  
	{
	  isArray = (curField->ArraySize() > 1);
	  if (!isArray)  
	    {
	      if((tp == CDLField::BYTES)||(tp == CDLField::VARBYTES))
		{
		  outFile<<indent<<" os << s << \""<<curField->Name()<<":   \""; 
		  outFile<<" << (char*)rec."<<curField->Name()<<"() << endl;"<<endl;
		}
	      else if(tp == CDLField::HYPER)
		{
		  if (md == CDLField::UNSIGNED)
		    {
		      if (uhyper_fl == 0)
			{
			  outFile<<indent<<" idl_uhyper_int uvall;"<<endl;
			  uhyper_fl = 1;
			}
		      outFile<<indent<<" uvall = rec."<<curField->Name()<<"();"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name()<<".high:   \"";  
		      outFile<<indent<<" << uvall.high << endl;"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name()<<".low:   \"";  
		      outFile<<indent<<" << uvall.low << endl;"<<endl;
		    }
		  else 
		    {
		      if(hyper_fl == 0)  
			{
			  outFile<<indent<<" idl_hyper_int vall;"<<endl;
			  hyper_fl = 1;
			} 
		      outFile<<indent<<" vall = rec."<<curField->Name()<<"();"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name()<<".high:   \"";  
		      outFile<<indent<<" << vall.high << endl;"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name()<<".low:   \"";  
		      outFile<<indent<<" << vall.low << endl;"<<endl;
		    }
		}
	      else if (tp == CDLField::CHARRAY)
		{
		  outFile<< indent << " os << s << \"" << curField->Name();
		  outFile << ":   \" << (const char*)(rec.";
		  outFile << curField->Name() << "()) << endl;" <<endl;
		}
	      else
		{
		  outFile<<indent << " os << s << \""<<curField->Name()<<":   \""; 
		  outFile << " << rec."<<curField->Name()<<"() << endl;" <<endl;
		}
	   } /* field is an array */
	  else 
	    {
	      int num = curField->ArraySize();
	      if((tp == CDLField::BYTES)||(tp == CDLField::VARBYTES))
		{
		  if (fl == 0) 
		    {
		      outFile<<indent<<"for(int i=0;i<"<<num<<";i++)"<<endl;
		      fl = 1;
		    }
		  else
		    outFile<<indent<<"for(i=0;i<"<<num<<";i++)"<<endl;
		  outFile << indent << "{" << endl;
		  outFile<<indent<<" os << s << \""<<curField->Name();
		  outFile<<"[\" << "<<"i"<<" << \"]:   \"";   
		  outFile<<" << (char*)rec."<<curField->Name()<<"(i) << endl;"<<endl;
		  outFile << indent << "}" << endl;
		}
	      else if (tp == CDLField::CHARRAY)
		{
		  if (fl == 0)  
		    {
		      outFile<<indent<<"for(int i=0;i<"<<num<<";i++)"<<endl;
		      fl=1;
		    }
                  else outFile<<indent<<"for(i=0;i<"<<num<<";i++)"<<endl;     
		  outFile<<indent<<" os << s << \""<<curField->Name();
		  outFile<<"[\" << "<<"i"<<" << \"]:   \"";   
		  outFile<<" << (const char*)(rec."<<curField->Name();
		  outFile<<"(i)) << endl;"<<endl;
		}
	      else if(tp!=CDLField::HYPER)  
		{
		  if (fl == 0)  
		    {
		      outFile<<indent<<"for(int i=0;i<"<<num<<";i++)"<<endl;
		      fl=1;
		    }
                  else outFile<<indent<<"for(i=0;i<"<<num<<";i++)"<<endl;     
		  outFile<<indent<<" os << s << \""<<curField->Name();
		  outFile<<"[\" << "<<"i"<<" << \"]:   \"";   
		  outFile<<" << rec."<<curField->Name()<<"(i) << endl;"<<endl;
		}
	      else  // if it is a HYPER 
		{
		  if ((md == CDLField::UNSIGNED) && (uhyperarr_fl == 0))
		    {
		      outFile<<indent<<" idl_uhyper_int uarrvall;"<<endl;
		      uhyperarr_fl = 1;
		    }		
		  else if (hyperarr_fl == 0) 
		    {
		      outFile<<indent<<" idl_hyper_int arrvall;"<<endl;
		      hyperarr_fl = 1;
		    } 
		  int num = curField->ArraySize();
		  if (fl == 0) 
		    {
		      outFile<<indent<<"for(int i=0;i<"<<num<<";i++)"<<endl;
		      fl=1;
		    }
		  else   outFile<<indent<<"for(i=0;i<"<<num<<";i++)"<<endl;
		  
		  outFile<<indent<<" {"<<endl;
		  if (md == CDLField::UNSIGNED)
		    {
		      outFile<<indent<<" uarrvall = rec."<<curField->Name()<<"(i);"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name();
		      outFile<<"[\""<<" << i << "<<"\"]high:   \"";  
		      outFile<<" << uarrvall.high << endl;"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name();
		      outFile<<"[\""<<" << i << "<<"\"]low:   \"";  
		      outFile<<" << uarrvall.low << endl;"<<endl;
		    }
		  else
		    {
		      outFile<<indent<<" arrvall = rec."<<curField->Name()<<"(i);"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name();
		      outFile<<"[\""<<" << i << "<<"\"]high:   \"";  
		      outFile<<" << arrvall.high << endl;"<<endl;
		      outFile<<indent<<" os << s << \""<<curField->Name();
		      outFile<<"[\""<<" << i << "<<"\"]low:   \"";  
		      outFile<<" << arrvall.low << endl;"<<endl;
		    }
		  outFile<<indent<<" }"<<endl;          
		}
	    }
	}
      else 		// Item is a record
	{
	  CDLRecord* ptRec;
	  ptRec = curField->Record();
	  String name;
	  name = ptRec->Name();
	  
	  isArray = (curField->ArraySize() > 1);
	  if (isArray)       
	    {
	      int num = curField->ArraySize();
	      if (flr == 0)  
		{
		  outFile<<indent<<"for(int j=0;j<"<<num<<";j++)"<<endl;
		  flr=1;
		}
	      else   outFile<<indent<<" for(j=0;j<"<<num<<";j++)"<<endl;

	      outFile<<indent<<" {"<<endl;
	      outFile<<indent<<name<<" obj_"<<curField->Name()<<";"<<endl;
	      outFile<<" obj_"<<curField->Name();
	      outFile<<" = rec."<<curField->Name()<<"(j);"<<endl;
	      outFile<<indent<< " s =  s + \""<< curField->Name();
	      outFile << "\" + \"[\" + int_to_str(j) + \"]\" + \".\";"<<endl;
	      outFile << indent <<" obj_"<<curField->Name();
	      outFile << ".Print(os, s,";
	      outFile <<" obj_"<< curField->Name() << ");"<<endl;
	      outFile <<" s = \"\";"<<endl;	
	      outFile << " }" << endl;
	    }  /*if RECORD is not an array of records*/
	  else 
	    {
	      outFile<<indent<<name<<" obj_"<<curField->Name()<<";"<<endl;
	      outFile<<" obj_"<<curField->Name();
	      outFile<<" = rec."<<curField->Name()<<"();"<<endl;
	      outFile<<indent<< " s =  s + \""<< curField->Name();
	      outFile << "\" + \".\";"<<endl;
	      outFile << indent <<" obj_"<<curField->Name();
	      outFile<< ".Print(os, s,";
	      outFile<<" obj_"<< curField->Name() << ");"<<endl;
	    }
	}
    }
  outFile << indent << "}\n" <<endl;
  
  /* ostream& operator<<(ostream& , const Record& ) */
  outFile << indent << "ostream & operator << (ostream & os ,";
  outFile << "const " << rec.Name() << "& rec)" << endl;
  outFile << indent << "{" << endl;
  outFile << indent <<" rec.Print(os,\"\",rec);"<<endl;
  outFile << indent << " return os;" << endl;
  outFile << indent << "}" << endl;

  return true;
}
  
/*
 * GenInlines
 *    Write the inline functions for this record. These go in the header
 * file, and include most of the accessors. Accessors for record and string
 * fields are placed out-of-line.
 */

Boolean CDLClassGenerator::GenInlines(CDLRecord& rec)

{
  String typeName;
  CDLField* curField;
  int inc;
  String fieldName;
  Boolean isArray;

  /* Inline conversion operator */
  if (createIdl)
    {
      inclFile << indent << "inline " << rec.Name() << "::operator ";
      inclFile << rec.Name() << CDLIdlRecordSuffix << "*()" << endl;
      inclFile << indent << "{ return &" << CDLCppIdlRecordName;
      inclFile << "; }\n" << endl; 

      inclFile << indent << "inline " << rec.Name() << "::operator const ";
      inclFile << rec.Name() << CDLIdlRecordSuffix << "*() const" << endl;
      inclFile << indent << "{ return &" << CDLCppIdlRecordName;
      inclFile << "; }\n" << endl;
   }

  /* Inline accessors */
  for (inc = 0; inc < rec.NumFields(); inc++)
    {
      curField = rec.Field(inc);
      isArray = (curField->ArraySize() > 1);
      GetTypeName(curField, typeName, true);
      MakeFieldName(curField, fieldName);
      if (IsRecord(curField->Type()))
	{
	  if (curField->Record() == (CDLRecord*)0)
	    return false;
	  inclFile << indent << "inline " << curField->Record()->Name() << " ";
	  inclFile << rec.Name() << "::" << curField->Name() << "(";
	  if (isArray)
	    {
	      inclFile << "int element";
	      fieldName += "[element]";
	    }
	  inclFile << ") const" << endl;
	  if (createIdl)
	    {
	      inclFile << indent << "{ return " << curField->Record()->Name();
	      inclFile << "(" << fieldName << "); }\n" << endl;
	    }
	  else
	    {
	      inclFile << indent << "{ return " << fieldName << "; }\n" << endl;
	    }
	}
      else if (IsString(curField->Type()))
	{
	  inclFile << indent;
	  inclFile << "inline const String " << rec.Name() << "::";
	  inclFile << curField->Name() << "(";
	  if (isArray)
	    {
	      inclFile << "int element";
	      fieldName += "[element]";
	    }
	  inclFile << ") const" << endl;
	  inclFile << indent << "{ return String((const char*)(";
	  inclFile << fieldName << ")); }\n" << endl;
	}
      else if (curField->Type() == CDLField::CHARRAY)
	{
	  inclFile << indent;
	  inclFile << "inline const " << typeName << " " << rec.Name();
	  inclFile << "::" << curField->Name() << "(";
	  if (isArray)
	    {
	      inclFile << "int element";
	      fieldName += "[element]";
	    }
	  inclFile << ") const" << endl;
	  inclFile << indent << "{ return (const " << typeName << ")";
	  inclFile << fieldName << "; }\n" << endl;
	}
      else
	{
	  inclFile << indent;
	  inclFile << "inline ";
	  if (IsAtomic(curField->Type()) || IsPointer(curField->Type()))
	    inclFile << "const ";
	  inclFile << typeName << " " << rec.Name() << "::";
	  inclFile << curField->Name() << "(";
	  if (isArray)
	    inclFile << "int element";
	  inclFile << ")";
	  if (IsAtomic(curField->Type()) || IsPointer(curField->Type()))
	    inclFile << " const";
          inclFile << endl;
          inclFile << indent << "{ return " << fieldName;
          if (isArray)
	    inclFile << "[element]";
          inclFile << "; }\n" << endl;
	  if (IsByteArray(curField->Type()))
	    {
	      inclFile << indent << "inline const unsigned int " << rec.Name();
	      inclFile << "::" << curField->Name() << "Length(";
	      if (isArray)
		inclFile << "int element";
	      inclFile << ") const" << endl;
	      inclFile << "{ return " << fieldName << CDLByteArrayLengthSuffix;
	      if (isArray)
		inclFile << "[element]";
	      inclFile << "; }\n" << endl;
	    }
	}
    }
  return true;
}

/*
 * GenArrayPacking
 *    If IDL arrays are being generated, this functions spits out the
 * bodies of the four functions that transfer List objects to IDL arrays,
 * do the reverse, and free IDL arrays.
 */

Boolean CDLClassGenerator::GenArrayPacking(CDLRecord& rec)

{
  String baseName = rec.Name();
  baseName += CDLIdlRecordSuffix;
  String arrayName = baseName;
  arrayName += CDLIdlArraySuffix;

  if (createIdl && rec.MakeIdlArray())
    {	
      /* ServerListToArray */
      outFile << indent << "Boolean " << rec.Name();
      outFile << "::ServerListToArray(const List<" << rec.Name();
      outFile << ">& cdlCppIn, " << endl;
      outFile << indent << "    " << arrayName << "& cdlCppOut)" << endl;
      OpenBrace(outFile);
      outFile << indent << "unsigned int cdlCppInc;\n" << endl;
      outFile << indent << "cdlCppOut.size = cdlCppIn.length();" << endl;
      outFile << indent << "if (cdlCppOut.size == 0)" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.array = (" << baseName << "*)0;" << endl;
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      outFile << indent << "else" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.array = (" << baseName;
      outFile << "*)rpc_ss_allocate(sizeof(" << baseName;
      outFile << ") * cdlCppIn.length());" << endl;
      outFile << indent << "if (cdlCppOut.array == (" << baseName;
      outFile << "*)0)" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.size = 0;" << endl;
      outFile << indent << "return 0;" << endl;
      CloseBrace(outFile);
      outFile << indent << "else" << endl;
      OpenBrace(outFile);
      outFile << indent << "for (cdlCppInc = 0; ";
      outFile << "cdlCppInc < cdlCppIn.length(); cdlCppInc++)" << endl;
      OpenBrace(outFile);
      outFile << indent << "const " << baseName;
      outFile << "* cdlCppTmpRec = cdlCppIn[cdlCppInc];" << endl;
      outFile << indent << "cdlCppOut.array[cdlCppInc] = *cdlCppTmpRec;";
      outFile << endl;
      CloseBrace(outFile);
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      CloseBrace(outFile);
      CloseBrace(outFile);
      outFile << endl;

      /* ClientListToArray */
      outFile << indent << "Boolean " << rec.Name();
      outFile << "::ClientListToArray(const List<" << rec.Name();
      outFile << ">& cdlCppIn, " << endl;
      outFile << indent << "    " << arrayName << "& cdlCppOut)" << endl;
      OpenBrace(outFile);
      outFile << indent << "unsigned int cdlCppInc;\n" << endl;
      outFile << indent << "cdlCppOut.size = cdlCppIn.length();" << endl;
      outFile << indent << "if (cdlCppOut.size == 0)" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.array = (" << baseName << "*)0;" << endl;
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      outFile << indent << "else" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.array = (" << baseName;
      outFile << "*)malloc(sizeof(" << baseName;
      outFile << ") * cdlCppIn.length());" << endl;
      outFile << indent << "if (cdlCppOut.array == (" << baseName;
      outFile << "*)0)" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.size = 0;" << endl;
      outFile << indent << "return 0;" << endl;
      CloseBrace(outFile);
      outFile << indent << "else" << endl;
      OpenBrace(outFile);
      outFile << indent << "for (cdlCppInc = 0; ";
      outFile << "cdlCppInc < cdlCppIn.length(); cdlCppInc++)" << endl;
      OpenBrace(outFile);
      outFile << indent << "const " << baseName;
      outFile << "* cdlCppTmpRec = cdlCppIn[cdlCppInc];" << endl;
      outFile << indent << "cdlCppOut.array[cdlCppInc] = *cdlCppTmpRec;";
      outFile << endl;
      CloseBrace(outFile);
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      CloseBrace(outFile);
      CloseBrace(outFile);
      outFile << endl;

      /* ArrayToList */
      outFile << indent << "Boolean " << rec.Name();
      outFile << "::ArrayToList(const " << arrayName << "& cdlCppIn, List<";
      outFile << rec.Name() << ">& cdlCppOut)" << endl;
      OpenBrace(outFile);
      outFile << indent << "unsigned int cdlCppInc;" << endl;
      outFile << indent << "Listiter<" << rec.Name();
      outFile << "> cdlCppIterator(cdlCppOut);\n" << endl;
      outFile << indent << "while (cdlCppIterator.remove_next());" << endl;
      outFile << indent << "for (cdlCppInc = 0; ";
      outFile << "cdlCppInc < cdlCppIn.size; cdlCppInc++)" << endl;
      OpenBrace(outFile);
      outFile << indent << "cdlCppOut.put(cdlCppIn.array[cdlCppInc]);" << endl;
      CloseBrace(outFile);
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      outFile << endl;

      /* ServerArrayFree */
      outFile << indent << "Boolean " << rec.Name();
      outFile << "::ServerArrayFree(" << arrayName << "& cdlCppIn)" << endl;
      OpenBrace(outFile);
      outFile << indent << "if ((cdlCppIn.size > 0) && ";
      outFile << "(cdlCppIn.array != (" << baseName << "*)0))" << endl;
      OpenBrace(outFile);
      outFile << indent << "rpc_ss_free((void*)cdlCppIn.array);" << endl;
      outFile << indent << "cdlCppIn.size = 0;" << endl;
      CloseBrace(outFile);
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      outFile << endl;

      /* ClientArrayFree */
      outFile << indent << "Boolean " << rec.Name();
      outFile << "::ClientArrayFree(" << arrayName << "& cdlCppIn)" << endl;
      OpenBrace(outFile);
      outFile << indent << "if ((cdlCppIn.size > 0) && ";
      outFile << "(cdlCppIn.array != (" << baseName << "*)0))" << endl;
      OpenBrace(outFile);
      outFile << indent << "free((char*)cdlCppIn.array);" << endl;
      outFile << indent << "cdlCppIn.size = 0;" << endl;
      CloseBrace(outFile);
      outFile << indent << "return 1;" << endl;
      CloseBrace(outFile);
      outFile << endl;
    }
  return true;
}
