/*
 * CDLClassGenerator:
 *
 *    This generator creates a C++ class for the CDL file in question. 
 * The class contains a data member for each field in the record, with a routine
 * to read and write each field. The class may also contain a copy constructor,
 * conversion operator and a "Copy" function for the C struct generated from
 * the IDL code also generated by CDL.
 *
 * cdlclassgenerator.H,v 1.7 1994/04/07 19:25:20 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_CLASS_GEN_H
#define CDL_CLASS_GEN_H

extern "C"
{
#include <pthread.h>
}
#include <stdio.h>
#include <cdlgenerator.H>

class CDLClassGenerator : public CDLGenerator

{
public:
  CDLClassGenerator(const ArgList&, Boolean usedIdl);
  virtual ~CDLClassGenerator();
  virtual Boolean Start(const String& fileName);
  virtual Boolean End();
  virtual Boolean Generate(CDLRecord&);
  virtual void DeleteFiles();
  static void GetTypeName(CDLField*, String& out,
			  Boolean ptr = false);
private:
  Boolean GenConstants();		// Generate all the constants
  Boolean GenHeader(CDLRecord&);	// Generate .H file
  Boolean GenMain(CDLRecord&);		// And the .C file
  Boolean GenInlines(CDLRecord&);	// And more for the .H file
  Boolean GenArrayPacking(CDLRecord&);	// Code for IDL arrays
  Boolean GenOstream(CDLRecord&);	// Code for the ostream operator
  void MakeFieldName(CDLField*, String& out);

  Boolean createIdl;
  ofstream inclFile;
  String mainName;
  String inclName;
};

#endif  
