/*
 * CDLConstInfo
 *
 *    This class contains all the information on a constant. Basically,
 * that means its type and value. If we were cute, we'd try to make a 
 * bunch of subclasses, but that doesn't really work out in the YADDL
 * model where the generators decide what code to generate when they see
 * something. So, we'll just keep track of the type and the name.
 *    Also, constant information has to be global. So, we'll declare a 
 * static variable that contains a Map of CDLConstInfo objects that we
 * can search later.
 *
 * cdlconstinfo.H,v 1.2 1994/04/07 19:25:26 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#ifndef CDL_CONST_INFO_H
#define CDL_CONST_INFO_H

extern "C"
{
#include <pthread.h>
}
#include <cdl/cdldefs.H>
#include <String.h>
#include <iostream.h>
#include <Symbol.h>
#include <Map.h>
#include <namedobject.H>
#include <cdlfield.H>

class CDLConstInfo : public NamedObject

{
public:
  CDLConstInfo();
  CDLConstInfo(CDLField::FieldType fType, CDLField::ModifierType mType,
	       const String& name, const String& value,
	       Boolean imported = false);
  CDLConstInfo(const CDLConstInfo&);
  CDLConstInfo& operator = (const CDLConstInfo&);
  virtual ~CDLConstInfo();
  CDLField::FieldType Type() const;
  CDLField::ModifierType Modifier() const;
  const String& Value() const;
  Boolean Imported() const;
private:
  CDLField::FieldType fType;
  CDLField::ModifierType mType;
  String value;
  Boolean imported;
};

extern Map<Symbol, CDLConstInfo> CDLConstants;

inline CDLField::FieldType CDLConstInfo::Type() const

{ return fType; }

inline CDLField::ModifierType CDLConstInfo::Modifier() const

{ return mType; }

inline const String& CDLConstInfo::Value() const

{ return value; }

inline Boolean CDLConstInfo::Imported() const

{ return imported; }

#endif
