/*
 * CDLEnumInfo
 *
 *    This class represents an "enum" type. An enum is simply translated
 * into an IDL enum, which is just a name and a list of possible values. 
 * IDL does not allow the user to assign values to particular cases, so
 * we don't either. So, a CDLEnumInfo object stores a name and a list of
 * strings, each of which represents a single case.
 *
 * cdlenuminfo.H,v 1.2 1994/04/07 19:25:31 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/

#ifndef CDL_ENUM_INFO_H
#define CDL_ENUM_INFO_H

extern "C"
{
#include <pthread.h>
}
#include <cdl/cdldefs.H>
#include <String.h>
#include <Symbol.h>
#include <Path.h>
#include <List.h>
#include <Map.h>
#include <namedobject.H>

class CDLEnumInfo : public NamedObject

{
public:
  CDLEnumInfo();
  CDLEnumInfo(const String& name, const List<String>& values,
	      Boolean imported = false);
  CDLEnumInfo(const CDLEnumInfo&);
  CDLEnumInfo& operator = (const CDLEnumInfo&);
  virtual ~CDLEnumInfo();
  const unsigned int NumCases() const;
  const String& Case(const unsigned int) const;
  Boolean Imported() const;
private:
  List<String> cases;
  Boolean imported;
};

extern Map<Symbol, CDLEnumInfo> CDLEnums;

inline const unsigned int CDLEnumInfo::NumCases() const

{ return cases.length(); }

inline const String& CDLEnumInfo::Case(const unsigned int i) const

{ return cases[i]; }

inline Boolean CDLEnumInfo::Imported() const

{ return imported; }

#endif
