    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


/*
 * CDLFileDef:
 *
 *    Here, we implement the CDLFileDef class. It's pretty straightforward.
 *
 * cdlfiledef.C,v 1.6 1994/02/01 22:32:50 gjb Exp
 */

#include <cdlfiledef.H>
#include <cdlindexdef.H>
#include <String.h>
#include <parselist.H>

#ifdef _AIX
#include <List.c>
#pragma define (Const_list_of_piter<CDLFileDef>)
#pragma define (List_of_p<CDLFileDef>)
#pragma define (List_of_piter<CDLFileDef>)
#endif

CDLFileDef::CDLFileDef() : NamedObject()

{
  allowDuplicates = false;
  myType = CLUSTERED;
  primary = (CDLIndexDef*)0;
  aclName = "";
  recName = "";
}

CDLFileDef::CDLFileDef(const String& name, const String& rec,
		       CDLFileType type, Boolean duplicates, 
		       const String& acl) 
    : NamedObject(name)
{
  myType = type;
  allowDuplicates = duplicates;
  primary = (CDLIndexDef*)0;
  aclName = acl;
  recName = rec;
  hasIndices = hasUniqueIndices = false;
}

CDLFileDef::CDLFileDef(const CDLFileDef& def)

{
  this->operator = (def);
}

CDLFileDef::~CDLFileDef()

{
}

void CDLFileDef::DeleteIndices()

{
  List_of_piter<CDLIndexDef> iter(indices);
  CDLIndexDef* dead;

  while (iter.remove_next(dead))
    {
      dead->DeleteFields();
      delete dead;
    }
}

CDLFileDef& CDLFileDef::operator = (const CDLFileDef& def)

{
  NamedObject::operator = (def);
  allowDuplicates = def.allowDuplicates;
  myType = def.myType;
  indices = def.indices;
  aclName = def.aclName;
  recName = def.recName;
  hasIndices = def.hasIndices;
  hasUniqueIndices = def.hasUniqueIndices;
  return *this;
}

Boolean CDLFileDef::AddIndex(CDLIndexDef* newIndex)

{
  Boolean found = false;
  Boolean retVal;
  List_of_piter<CDLIndexDef> iter(indices);
  CDLIndexDef* curIndex;
  String errMsg;

  while (!found && iter.next(curIndex))
    if (*curIndex == *newIndex)
      found = true;
  if (found)
    {
      CDLReport.VGetMsg(1106, errMsg, (const char*)(newIndex->Name()), 
			(const char*)name);
      CDLPError(errMsg);
      retVal = false;
    }
  else
    {
      indices.put(newIndex);
      if ((newIndex->Name() == "Primary") || 
	  (newIndex->Name() == "primary"))
	primary = newIndex;
      hasIndices = true;
      if (!(newIndex->Duplicates()))
	hasUniqueIndices = true;
      retVal = true;
    }
  return retVal;
}
