/*
 * CDLFileDef:
 *
 *    This class represents a file defined in the .cfd file. The file record
 * has a name, a few parameters (type, whether duplicate records are allowed)
 * and a list of indices (and later, a list of views).
 *
 * cdlfiledef.H,v 1.5 1994/04/07 19:25:36 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_FILE_REC_DEF_H
#define CDL_FILE_REC_DEF_H

extern "C"
{
#include <pthread.h>
}
#include <cdl/cdldefs.H>
#include <cdl/cdlbasedefs.H>
#include <Path.h>
#include <List.h>
#include <String.h>
#include <namedobject.H>
#include <objdce/objtranparams.H>
#include <cdlindexdef.H>

class CDLFileDef : public NamedObject

{
public:
  CDLFileDef();
  CDLFileDef(const String& name, const String& recName,
	     CDLFileType type, Boolean duplicates,
	     const String& acl);
  CDLFileDef(const CDLFileDef&);
  ~CDLFileDef();
  void DeleteIndices();
  CDLFileDef& operator = (const CDLFileDef&);
  Boolean AddIndex(CDLIndexDef*);
  Boolean Duplicates() const;
  CDLFileType Type() const;
  int NumIndices() const;
  CDLIndexDef* Index(const int num);
  CDLIndexDef* PrimaryIndex() const;
  const String& ACLName() const;
  const String& RecName() const;
  Boolean HasIndices() const;
  Boolean HasUniqueIndices() const;
private:
  Boolean allowDuplicates;
  CDLFileType myType;
  List_of_p<CDLIndexDef> indices;
  CDLIndexDef* primary;
  String aclName;
  String recName;
  Boolean hasIndices;
  Boolean hasUniqueIndices;
};

inline Boolean CDLFileDef::Duplicates() const

{ return allowDuplicates; }

inline CDLFileType CDLFileDef::Type() const

{ return myType; }

inline CDLIndexDef* CDLFileDef::Index(const int num)
     
{ return indices[num]; }

inline CDLIndexDef* CDLFileDef::PrimaryIndex() const

{ return primary; }

inline const String& CDLFileDef::ACLName() const

{ return aclName; }

inline const String& CDLFileDef::RecName() const

{ return recName; }

inline Boolean CDLFileDef::HasIndices() const

{ return hasIndices; }

inline Boolean CDLFileDef::HasUniqueIndices() const

{ return hasUniqueIndices; }

inline int CDLFileDef::NumIndices() const

{ return indices.length(); }

#endif
