/*
 * CDLFileGenerator:
 *
 *    Here, we create the file superclass for our record type. We execute
 * the parser for the auxiliary file, then turn it in to our superclass.
 * 
 * cdlfilegenerator.C,v 1.9 1994/02/01 22:32:52 gjb Exp 
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <cdlfilegenerator.H>
#include <cdlclassgenerator.H>
#include <cdlfield.H>
#include <unistd.h>

CDLFileGenerator::CDLFileGenerator(const ArgList& args)
: CDLDataGenerator(args)
{
  mainName = "";
  inclName = "";
}

CDLFileGenerator::~CDLFileGenerator()

{
}

Boolean CDLFileGenerator::Start(const String& fileName)

{
  String symbolName;
  String recFileName;
  

  CDLReport.Report(ErrorLevels::TraceMsg, "CDLFileGenerator: Start\n");
  MakeFileName(fileName, mainName, "_cdl_file.C");
  CDLReport.VReport(ErrorLevels::TraceMsg, "Making File: %s\n", 
		    (const char*)mainName);
  MakeFileName(fileName, inclName, "_cdl_file.H");
  CDLReport.VReport(ErrorLevels::TraceMsg, "Making File: %s\n", 
		    (const char*)inclName);
  outFile.open(mainName);
  if (outFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1002, (const char*)mainName);
      return false;
    }
  inclFile.open(inclName);
  if (inclFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1002, (const char*)inclName);
      return false;
    }
  else
    {
      MakeFileName(fileName, symbolName, "_CDL_FILE_H_INCLUDED");
      CDLReport.VReport(ErrorLevels::TraceMsg, "Making File: %s\n", 
			(const char*)symbolName);
      MakeFileName(fileName, recFileName, "_cdl.H");
      CDLReport.VReport(ErrorLevels::TraceMsg, "Making File: %s\n", 
			(const char*)recFileName);
      inclFile << "/*" << endl;
      inclFile << " * Class generated by Citi-DDL:" << endl;
      inclFile << " * cdlfilegenerator.C,v 1.9 1994/02/01 22:32:52 gjb Exp" << endl;
      inclFile << " */\n" << endl;
      inclFile << "#ifndef " << symbolName << endl;
      inclFile << "#define " << symbolName << "\n" << endl;
      inclFile << "#include <" << recFileName << ">" << endl;
      inclFile << "#include <List.h>" << endl;
      inclFile << "#include <String.h>" << endl;
      inclFile << "#include <cdl/cdlbasedefs.H>" << endl;
      inclFile << "#include <objdce/objtranparams.H>\n" << endl;
      
      outFile << "/*" << endl;
      outFile << " * Class generated by Citi-DDL:" << endl;
      outFile << " * cdlfilegenerator.C,v 1.9 1994/02/01 22:32:52 gjb Exp" << endl;
      outFile << " */\n" << endl;
      outFile << "#include <" << inclName << ">\n" << endl;

      return true;
    }
}

Boolean CDLFileGenerator::End()

{
  inclFile << "#endif" << endl;
  inclFile << flush;
  inclFile.close();
  outFile << flush;
  outFile.close();
  return true;
}

void CDLFileGenerator::DeleteFiles()

{
  CDLReport.Report(ErrorLevels::TraceMsg, "CDLFileGenerator: DeleteFiles\n");
  outFile.close();
  inclFile.close();
  unlink(mainName);
  unlink(inclName);
}

Boolean CDLFileGenerator::Generate(CDLRecord& rec, CDLFileDef& file)

{
  Boolean retVal;

  CDLReport.Report(ErrorLevels::TraceMsg, "CDLFileGenerator: Generate\n");
  retVal = GenHeader(rec, file);
  if (retVal)
    retVal = GenMain(rec, file);
  return retVal;
}

Boolean CDLFileGenerator::GenHeader(CDLRecord& rec, CDLFileDef& file)

{
  int inc;
  CDLField* curField;
  String tmpName;
  String recName;
  String fileName;
  
  CDLReport.Report(ErrorLevels::TraceMsg, "CDLFileGenerator: GenHeader\n");
  recName = rec.Name();
  fileName = file.Name();
  inclFile << indent << "class " << file.Name() << "\n" << endl;
  inclFile << indent << "{" << endl;
  if (file.HasIndices())
    {						// Create index enum
      inclFile << indent << "public:" << endl;
      Indent();
      inclFile << indent << "enum Index { ";
      for (inc = 0; inc < file.NumIndices(); inc++)
	{
	  tmpName = file.Index(inc)->Name();
	  inclFile << tmpName.upper() << ", ";
	}
      inclFile.seekp(-2, ios::cur);	// Back over comma
      inclFile << " };" << endl;
      Outdent();
    }
  inclFile << indent << "protected:" << endl;
  Indent();
  inclFile << indent << fileName << "();" << endl;
  Outdent();
  inclFile << indent << "public:" << endl;
  Indent();
  inclFile << indent << "virtual ~" << fileName << "();" << endl;
  inclFile << indent << "virtual CDLStatus OpenFile(const String& dbName, ";
  inclFile << "const String& fileName," << endl;
  inclFile << indent << indent << "unsigned short flags) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus CloseFile() = 0;" << endl;
  inclFile << indent << "virtual CDLStatus CreateFile(const String& dbName, ";
  inclFile << "const String& fileName, const String& volName, " << endl;
  inclFile << indent << indent << "unsigned long numRecs, ";
  inclFile << "Boolean makeIndices = true) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus DestroyFile() = 0;" << endl;
  inclFile << indent << "virtual CDLStatus EmptyFile() = 0;" << endl; 
  if (file.HasIndices())
    {
      inclFile << indent << "virtual CDLStatus Select(" << fileName;
      inclFile << "::Index, " << recName << "* start, ";
      inclFile << recName << "* end, CDLIndexDir) = 0;" << endl;
    }
  inclFile << indent << "virtual CDLStatus Insert(" << recName;
  inclFile << "& newRec) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus InsertList(List<";
  inclFile << recName << ">&) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus ReadCurrent(CDLReadLock, ";
  inclFile << "CDLRecordDir, " << recName;
  inclFile << "& out) = 0;" << endl;
  if (file.HasUniqueIndices())
    {
      inclFile << indent << "virtual CDLStatus ReadUnique(CDLReadLock, ";
      inclFile << fileName << "::Index, " << recName << "& key, ";
      inclFile << recName << "& out) = 0;" << endl;
    }
  if (file.HasIndices())
    {
      inclFile << indent << "virtual CDLStatus ReadAll(CDLReadLock, ";
      inclFile << fileName << "::Index, " << recName;
      inclFile << "& key, CDLIndexDir, List<";
      inclFile << recName << ">& out, unsigned int maxHits = 0) = 0;" << endl;
      inclFile << indent << "virtual CDLStatus ReadAll(CDLReadLock, ";
      inclFile << fileName << "::Index, " << recName << "* start," << endl;
      inclFile << indent << indent << recName << "* end, ";
      inclFile << "CDLIndexDir, List<" << recName << ">& out, ";
      inclFile << "unsigned int maxHits = 0) = 0;" << endl;
    }
  inclFile << indent << "virtual CDLStatus DeleteCurrent(CDLRecordDir) = 0;";
  inclFile << endl;
  if (file.HasUniqueIndices())
    {
      inclFile << indent << "virtual CDLStatus DeleteUnique(" << fileName;
      inclFile << "::Index, " << recName << "& key) = 0;" << endl;
    }
  if (file.HasIndices())
    {
      inclFile << indent << "virtual CDLStatus DeleteAll(" << fileName;
      inclFile << "::Index, " << recName << "& key) = 0;" << endl;
      inclFile << indent << "virtual CDLStatus DeleteAll(" << fileName;
      inclFile << "::Index, " << recName << "* start," << endl;
      inclFile << indent << indent << recName << "* end) = 0;" << endl;
    }
  inclFile << indent << "virtual CDLStatus UpdateCurrent(CDLRecordDir, ";
  inclFile << recName << "& newVal) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus SaveContext(void*&) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus RestoreContext(void*) = 0;" << endl;
  if (file.HasUniqueIndices())
    {
      inclFile << indent << "virtual CDLStatus UpdateUnique(" << fileName;
      inclFile << "::Index, " << recName << "& key, ";
      inclFile << recName << "& newVal) = 0;" << endl;

      for (inc = 0; inc < rec.NumFields(); inc++)	// Do modifies & increments
	{
	  curField = rec.Field(inc);
	  if (IsAtomic(curField->Type()) && (curField->ArraySize() == 1))
	    {
	      CDLClassGenerator::GetTypeName(curField, tmpName, false);
	      inclFile << indent << "virtual CDLStatus Modify_" << curField->Name();
	      inclFile << "(" << fileName << "::Index, ";
	      inclFile << recName << "& key, " << tmpName << " newVal, ";
	      inclFile << recName << "& out) = 0;" << endl;
	      inclFile << indent << "virtual CDLStatus Increment_";
	      inclFile << curField->Name() << "(" << fileName;
	      inclFile << "::Index, " << recName << "& key, ";
	      inclFile << tmpName << " incrementBy, ";
	      inclFile << recName << "& out, ";
	      inclFile << "Boolean incrementLock = false) = 0;" << endl;
	    }
	}
    }
  inclFile << indent << "virtual CDLStatus ActivateIndex(";
  inclFile << fileName << "::Index, const String& volume, ";
  inclFile << recName << "* nullKey = (" << recName << "*)0) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus DeactivateIndex(";
  inclFile << fileName << "::Index) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus DestroyIndex(";
  inclFile << fileName << "::Index) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus ActivateAllIndices";
  inclFile << "(const String& volume) = 0;" << endl;
  inclFile << indent << "virtual CDLStatus DeactivateAllIndices() = 0;" << endl;
  inclFile << indent << "virtual CDLStatus DestroyAllIndices() = 0;" << endl;
  Outdent();
  inclFile << indent << "};\n" << endl;
  return true;
}

Boolean CDLFileGenerator::GenMain(CDLRecord&, CDLFileDef& file)

{

  CDLReport.Report(ErrorLevels::TraceMsg, "CDLFileGenerator: GenMain\n");
  outFile << indent << file.Name() << "::" << file.Name() << "()\n" << endl;
  outFile << indent << "{}\n" << endl;
  outFile << indent << file.Name() << "::~" << file.Name() << "()\n" << endl;
  outFile << indent << "{}\n" << endl;
  return true;
}
