/*
 * CDLIndexDef
 *
 *    This object contains information on an index in a CDL file.
 *
 * cdlindexdef.C,v 1.6 1994/02/01 22:33:04 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <cdlindexdef.H>
#include <cdlindexfielddef.H>
#include <parselist.H>

#ifdef _AIX
#include <List.c>

#pragma define (Const_list_of_piter<CDLIndexDef>)
#pragma define (List_of_p<CDLIndexDef>)
#pragma define (List_of_piter<CDLIndexDef>)
#endif

CDLIndexDef::CDLIndexDef() : NamedObject()

{
  allowDuplicates = false;
  theDir = FORWARDS;
  buildMe = true;
}

CDLIndexDef::CDLIndexDef(const String& myName, CDLIndexDir direction,
			 Boolean duplicates, Boolean buildIt)
    : NamedObject(myName)
{
  allowDuplicates = duplicates;
  theDir = direction;
  buildMe = buildIt;
}

CDLIndexDef::CDLIndexDef(const CDLIndexDef& def)

{
  this->operator = (def);
}

CDLIndexDef::~CDLIndexDef()

{
}

void CDLIndexDef::DeleteFields()

{
  List_of_piter<CDLIndexFieldDef> iter(fields);
  CDLIndexFieldDef* dead;
  iter.reset();
  while (iter.remove_next(dead))
    delete dead;
}

CDLIndexDef& CDLIndexDef::operator = (const CDLIndexDef& def)

{
  NamedObject::operator = (def);
  allowDuplicates = def.allowDuplicates;
  theDir = def.theDir;
  buildMe = def.buildMe;
  fields = def.fields;
  return *this;
}

Boolean CDLIndexDef::AddField(CDLIndexFieldDef* newField)

{
  Boolean found = false;
  Boolean retVal;
  List_of_piter<CDLIndexFieldDef> iter(fields);
  CDLIndexFieldDef* curField;
  String errMsg;

  while (!found && iter.next(curField))
    if (*curField == *newField)
      found = true;
  if (found)
    {
      CDLReport.VGetMsg(1102, errMsg, (const char*)newField->Name(), 
			(const char*)name);
      CDLPError(errMsg);
      retVal = false;
    }
  else
    {
      fields.put(newField);
      retVal = true;
    }
  return retVal;
}

CDLIndexFieldDef* CDLIndexDef::Find(const String& findName)

{
  Boolean found = false;
  CDLIndexFieldDef* retVal;
  Const_list_of_piter<CDLIndexFieldDef> iter(fields);


  CDLReport.VReport(ErrorLevels::TraceMsg,
		    "CDLIndexDef::Find - Name: %s\n", 
		    (const char*)findName);
  while (!found && iter.next(retVal))
    {
      if (retVal->Name() == findName)
	found = true;
    }
  if (!found)
    retVal = (CDLIndexFieldDef*)0;
  return retVal;
}
