/*
 * CDLIndexDef:
 *
 *    This class represents a single index in a CDL file. It, in turn,
 * represents a list of index fields.
 *
 * cdlindexdef.H,v 1.5 1994/04/07 19:25:51 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_INDEX_DEF_H
#define CDL_INDEX_DEF_H

extern "C"
{
#include <pthread.h>
}
#include <iostream.h>
#include <cdl/cdldefs.H>
#include <cdl/cdlbasedefs.H>
#include <Path.h>
#include <List.h>
#include <String.h>
#include <objdce/objtranparams.H>
#include <namedobject.H>

class CDLIndexFieldDef;

class CDLIndexDef : public NamedObject

{
public:
  CDLIndexDef();
  CDLIndexDef(const String& myName, CDLIndexDir dir,
	      Boolean duplicates, Boolean buildIt);
  CDLIndexDef(const CDLIndexDef&);
  virtual ~CDLIndexDef();
  void DeleteFields();
  CDLIndexDef& operator = (const CDLIndexDef&);
  Boolean AddField(CDLIndexFieldDef*);
  Boolean Duplicates() const;
  Boolean BuildMe() const;
  CDLIndexDir Direction() const;
  int NumFields() const;
  CDLIndexFieldDef* Field(const int num);
  CDLIndexFieldDef* Find(const String& name);
private:
  Boolean allowDuplicates;
  CDLIndexDir theDir;
  Boolean buildMe;
  List_of_p<CDLIndexFieldDef> fields;
};

inline Boolean CDLIndexDef::Duplicates() const

{ return allowDuplicates; }

inline Boolean CDLIndexDef::BuildMe() const

{ return buildMe; }

inline CDLIndexDir CDLIndexDef::Direction() const

{ return theDir; }

inline CDLIndexFieldDef* CDLIndexDef::Field(const int num)
     
{ return fields[num]; }

inline int CDLIndexDef::NumFields() const

{ return fields.length(); }

#endif
