/*
 * CDLMain:
 *
 *    This file contains the main() for CDL. The main creates a CDLMainParser
 * object and passes it a list of CDLGenerator objects with which to parse
 * the file. It builds this list based on the arguments it is given.
 *
 * cdlmain.C,v 1.14 1994/04/07 19:25:57 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/statics.H>
#include <sys/types.h>
#include <sys/socket.h>
#include <objdce/objtranparams.H>
#include <cdl/cdldefs.H>
#include <iostream.h>
#include <cdlrecord.H>
#include <cdlmainparser.H>
#include <cdlauxparser.H>
#include <cdlidlgenerator.H>
#include <cdlclassgenerator.H>
#include <cdlfilegenerator.H>
#include <cdlsfsgenerator.H>
#include <cdlrqsgenerator.H>
#include <cdlnidlgenerator.H>
#include <parselist.H>
#include <arglist.H>
#include <objdce/error.H>
#include <mainparsedefs.H>
#include <sfsparsedefs.H>

/* Results from argument parsing. These are set in ParseArgs. */

static Boolean useSFS;
static Boolean useRQS;
static Boolean useClass;
static String inFileName;
static Boolean runIDL;
static Boolean makeStubs;
static Boolean nidlMode;

static Boolean ParseArgs(const ArgList& args)

{
  int curArgNo;
  Boolean worked = true;
  String curArg;
  String firstTwo;
  Boolean gotGenerate;

  /* Here, set the defaults for the argument parsing flags */
  useSFS = false;
  useRQS = false;
  useClass = true;
  runIDL = true;
  nidlMode = false;
  makeStubs = true;
  inFileName = "";

  if ((args.GetArgc() < 2) || (args.Get(1) == "-h") || 
      (args.Get(1) == "-help"))
    worked = false;
  else
    {
      gotGenerate = false;
      for (curArgNo = 1; curArgNo < args.GetArgc(); curArgNo++)
	{
	  curArg = args.Get(curArgNo);
	  firstTwo = curArg.chunk(0, 2);
	  if (gotGenerate)			// Last arg was "-generate"
	    {
	      if (curArg == "sfs")
		{
		  useSFS = true;
		  gotGenerate = false;
		}
	      else
	      if (curArg == "rqs")
		{
		  useRQS = true;
		  gotGenerate = false;
		}
	      else

		worked = false;
	    }
	  else if ((firstTwo != "-I") && (firstTwo != "-D") &&
		   (firstTwo != "-U") && (firstTwo != "-Y"))
	    {
	      if (curArg == "-generate")
		{
		  gotGenerate = true;
		}
	      else if (curArg == "-d")
		{
		  CDLMaindebug = 1; 		// Turn on yacc debugging
		  CDLSfsdebug = 1;
		}
	      else if (curArg == "-no_class")
		{
		  useClass = false;
		}
	      else if (curArg == "-no_idl")
		{
		  runIDL = false;
		}
	      else if (curArg == "-no_stubs")
		{
		  makeStubs = false;
		}
	      else if (curArg == "-nidl")
		{
		  nidlMode = true;
		}
	      else if (curArgNo == (args.GetArgc() - 1))
		{
		  inFileName = curArg;		// This is the file name
		}
	      else
		{
		  worked = false;
		}
	    }
	}
      if (gotGenerate || (inFileName == ""))
	worked = false;
    }
  if (!worked)
    {			// Print "Usage:"
      String msg;
      CDLReport.GetMsg(300, msg);
      fprintf(stderr, "%s", (const char*)msg);
    }
  return worked;
}

int main(int argc, char** argv)

{	
  int retVal = 0;
  CDLIdlGenerator* genIdl = (CDLIdlGenerator*)0;
  CDLClassGenerator* genClass = (CDLClassGenerator*)0;
  CDLFileGenerator* genFile = (CDLFileGenerator*)0;
  CDLSfsGenerator* genSFS = (CDLSfsGenerator*)0;
  CDLRqsGenerator* genRQS = (CDLRqsGenerator*)0;
  CDLNidlGenerator* genNidl = (CDLNidlGenerator*)0;
  CDLMainParser* mainParse = (CDLMainParser*)0;
  CDLAuxParser* auxParse = (CDLAuxParser*)0;
  Boolean doFile = false;
  Boolean doSFS = false;
  Boolean doRQS = false;
  CDLGenerator* genPtr;
  CDLDataGenerator* dataPtr;

  CDLFirstInit();

  ArgList mainArgs((const int)argc, (const char**) argv);

  if (ParseArgs(mainArgs))
    {
      ArgList includeArgs(mainArgs, "I");
      MakeIncludeList(mainArgs.GetArgc(), mainArgs.GetArgv());
      String auxName;
      String fileName = mainArgs.Get(mainArgs.GetArgc() - 1);
      CDLGenericGenerator::MakeFileName(inFileName, auxName, ".cfd", true);

      if (useRQS)
	doRQS = true;
      if (useSFS)
	doFile = doSFS = true;

      /* Lists */
      List_of_p<CDLGenerator> generators;
      List_of_p<CDLDataGenerator> dataGenerators;
      List_of_piter<CDLGenerator> genIter(generators);
      List_of_piter<CDLDataGenerator> dataIter(dataGenerators);

      /* Parsers */
      mainParse = new CDLMainParser(mainArgs);	  // Parser for the .cdl file
      CDLMainSetParser(mainParse);
      if (doFile || doRQS)
	{
	  auxParse = new CDLAuxParser(mainArgs);  // Parser for the .cfd file
	  CDLSfsSetParser(auxParse);
	}

      /* Generators */
      if (useClass)
	{
	  genIdl = new CDLIdlGenerator(mainArgs, nidlMode);  // Generate IDL code
	  generators.put(genIdl);
	  genClass = new CDLClassGenerator(mainArgs, true);  // C++ classes
	  generators.put(genClass);
	}
      if (nidlMode)
	{
	  genNidl = new CDLNidlGenerator(mainArgs);
	  generators.put(genNidl);
	}
      if (doFile)
	{
	  genFile = new CDLFileGenerator(mainArgs);  // Generic file code
	  dataGenerators.put(genFile);
	}
      if (doSFS)
	{
	  genSFS = new CDLSfsGenerator(mainArgs);    // SFS file code
	  dataGenerators.put(genSFS);
	}
      if (doRQS)
	{
	    genRQS = new CDLRqsGenerator(mainArgs); // RQS code
	    dataGenerators.put(genRQS);
	}

      if (mainParse->Parse(fileName, generators))    // Parse the .cdl file
	{
	  if (useClass && runIDL)
	    {
	      genIdl->RunIdl(makeStubs);	// Run IDL on the .idl output
	    }
	  if (useClass && nidlMode && runIDL)
	    {
	      genNidl->RunNidl(makeStubs);	// Run NIDL on the same
	    }
	  if (doFile || doRQS)
	    {
	      if (auxParse->Parse(auxName, dataGenerators))  // Parse .cfd file
		retVal = 0;
	      else
		retVal = -1;
	    }
	}
      else
	retVal = -1;
      if (retVal != 0)		// Delete all output files if parsing failed
	{
	  dataIter.reset();
	  while (dataIter.next(dataPtr))
	    dataPtr->DeleteFiles();
	  genIter.reset();
	  while (genIter.next(genPtr))
	    genPtr->DeleteFiles();
	}
      delete genIdl;
      delete genClass;
      delete genFile;
      delete genSFS;
      delete genRQS;
      delete genNidl;
      delete mainParse;
      delete auxParse;
    }
  return retVal;
}

