/*
 * CDLMainParser:
 *
 *    Here, we actually process the file using cpp and the yacc parser. We
 * then feed it to several generators. 
 *
 * cdlmainparser.C,v 1.7 1993/12/03 14:43:31 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <cdlmainparser.H>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <arglist.H>
#include <cdlgenerator.H>
#include <mainparsedefs.H>

CDLMainParser::CDLMainParser(const ArgList& args)
    : CDLParser(args)

{
}

/*
 * ~CDLMainParser
 * 
 *    Clear out the list of records we generated while parsing.
 */

CDLMainParser::~CDLMainParser()

{
  List_of_piter<CDLRecord> iter(recordList);
  CDLRecord* dead;
  
  while (iter.remove_next(dead))
    {
      dead->DeleteFields();
      delete dead;
    }
}

/*
 * Parse
 *    
 *    Pre-process the file, then open the file. If that works, initialize the
 * global variables (in cdlparsedefs.H) and run the parser (in mainparse.y).
 * If parsing succeeded, feed the output to each of the code generators on the
 * list.
 */

Boolean CDLMainParser::Parse(const String& fileName,
			     List_of_p<CDLGenerator>& generators)
{
  String tmpName;
  Boolean stillOK;
  List_of_piter<CDLGenerator> iter(generators);
  CDLGenerator* curGen;
  List_of_piter<CDLRecord> recIter(recordList);
  CDLRecord* curRec;
  
  if (PreProcess(fileName, tmpName))
    {
      CDLMainin = fopen(tmpName, "r");
      if (CDLMainin == (FILE*)0)
        {
	  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1000, (const char*)tmpName);
          return false;
        }
      else
        {
	  CDLMainSetName(fileName);
	  CDLMainFailed = 0;
	  CDLMainMakeIdlArray = 0;
          int status = CDLMainparse();
          fclose(CDLMainin);
          unlink(tmpName);
          if ((status != 0) || CDLMainFailed)
            {
	      CDLReport.ReportMsg(ErrorLevels::ErrorMsg, 500);
              return false;   
	    }
	  else
	    {
              stillOK = true;
              while (stillOK && iter.next(curGen))
                {
		  if (!(curGen->Start(fileName)))
                    stillOK = false;
                  else
                    {
		      recIter.reset();
                      while (recIter.next(curRec))
                        {
			  if (!(curRec->Imported()))
			    if (!curGen->Generate(*curRec))
			      stillOK = false;
                        }
                    }
		  curGen->End();
                }
              return stillOK;
            }
        }
    }
}

/*
 * AddRecord
 * 
 *    Called by the wrapper routines in cdlparsewrap.C. These routines
 * create a new CDLRecord object. Put it on our list of records.
 */

Boolean CDLMainParser::AddRecord(CDLRecord* rec)

{
  Boolean found = false;
  Boolean retVal;
  List_of_piter<CDLRecord> iter(recordList);
  CDLRecord* curRec;

  while (!found && iter.next(curRec))
    if (*curRec == *rec)
      found = true;
  if (found)
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1103, (const char*)rec->Name());
      retVal = false;
    }
  else
    {
      recordList.put(rec);
      retVal = true;
    }
  return retVal;
}

/*
 * Find
 *
 *    Go through the list of records until we find one with the
 * specified name. Used when parsing so we can associate a record
 * name with a pointer to an actual object.
 */

CDLRecord* CDLMainParser::Find(const String& name) const

{
  Boolean found = false;
  CDLRecord* retVal;
  Const_list_of_piter<CDLRecord> iter(recordList);

  while (!found && iter.next(retVal))
    if (retVal->Name() == name)
      found = true;
  if (!found)
    retVal = (CDLRecord*)0;
  return retVal;
}
