/*
 * CDLParser:
 * 
 *    Here, we put the code that all parser objects will have to use.
 *
 * cdlparser.C,v 1.10 1994/04/07 19:25:59 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <cdlparser.H>
#include <stdio.h>
#include <stdlib.h>
#include <Search_path.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <unistd.h>
#include <arglist.H>
#include <objdce/defs.H>
#include <cdl/cdlbasedefs.H>
#include <cdlgenerator.H>
#include <cdlsuffixes.H>

CDLParser::CDLParser(const ArgList& args)
    : origArgs(args), 
      cppArgs(args, "DIUY")	// Only copy args that start with "-" that letter

{
//  cppArgs.PutFront("-P");
#ifdef sun
  cppArgs.PutFront("-B");
#endif
}

CDLParser::~CDLParser()

{
}

/*
 * FindCpp
 *    Using the Standard Components "Search_path" class, return the full
 * pathname of CPP in "cppNameOut". However, if CDL_CPP is set, just
 * use that name.
 */

Boolean CDLParser::FindCpp(String& cppNameOut)

{
  Boolean retVal = false;

  char* envVal = getenv("CDL_CPP");
  if (envVal != (char*)0)
    {
      cppNameOut = envVal;
      retVal = true;
    }
  else
    {
      Path foundIt;
      Search_path cppPath(CDLCppSearchPath);
      Path cppName = "cpp";
      if (cppPath.find(cppName, foundIt, Ksh_test::x))
	{
	  retVal = true;
	}
      else
	{
	  cppName = "cpp.ansi";
	  if (cppPath.find(cppName, foundIt, Ksh_test::x))
	    {
	      retVal = true;
	    }
	}
      if (retVal)
	{
	  CDLReport.VReport(ErrorLevels::InfoMsg,
			    "Found cpp as %s\n", (const char*)foundIt);
	  cppNameOut = (const String&)foundIt;
	}
      else
	{
	  CDLReport.Report(ErrorLevels::ErrorMsg,
			   "Couldn't locate cpp\n");
	  cppNameOut = "";
	}
    }
  return retVal;
}

Boolean CDLParser::PreProcess(const String& inFile, String& outFile)

{
  Boolean retVal;
  ArgList tmpCppArgs = cppArgs;
  char* tmpEnv = getenv("TMPDIR");
  String tmpDir;
  if (tmpDir == (char*)0)
    {
      tmpDir = CDLDefaultTmp;
    }
  else
    {
      tmpDir = tmpEnv;
    }
  int status = 0;
#if HAVE_UNION_WAIT
  union wait wStatus;
  union wait* waiter;
  wStatus.w_status = 0;
#else
  int* statusP;
#endif
  String cppNameStr;
  if (!FindCpp(cppNameStr))
    {
      return false;
    }
  char* cppName = (char*)((const char*)cppNameStr);
  char* realTmpDir = (char*)((const char*)tmpDir);
  
  outFile = tempnam(realTmpDir, "cdl");
  tmpCppArgs.PutEnd("-I.");
  tmpCppArgs.PutEnd(CDLDCEIncludeCmd);
  tmpCppArgs.PutEnd(inFile);
  tmpCppArgs.PutEnd(outFile);
  tmpCppArgs.PutFront(cppName);
  if (vfork() == 0)
    {
#if CONST_EXEC_ARGV
      status = execv(cppName, (char* const*)(tmpCppArgs.GetArgv()));
#else
      status = execv(cppName, (const char **)(tmpCppArgs.GetArgv()));
#endif
      if (status == -1)
	{
	  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 501, "cpp");
	  return false;
	}
      retVal = true;
    } 
  else 
    {
#if HAVE_UNION_WAIT
      waiter = &wStatus;
      wait(waiter);
      status = wStatus.w_status;
#else
      statusP = &status;
      wait(statusP);
#endif
      if (WIFEXITED(status) && (WEXITSTATUS(status) != 0))
        {
	  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 502, "cpp", WEXITSTATUS(status));
          retVal = false;
        }
      else if (WIFSIGNALED(status))
        {
	  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 503, "cpp", WTERMSIG(status));
          retVal =  false;
        }
      else
	{
	  retVal = true;
	}
    }
  return retVal;
}

