/* 
 * CDLRecord:
 *
 *    This object represents a single record in the main CDL file. The record
 * is very simple -- it consists of a name, a documentation string, and a
 * list of CDLField objects, each of which represents a single field.
 * There are also various flags.
 *
 * cdlrecord.H,v 1.7 1994/04/07 19:26:05 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_RECORD_H
#define CDL_RECORD_H

extern "C"
{
#include <pthread.h>
}
#include <iostream.h>
#include <cdl/cdldefs.H>
#include <Path.h>
#include <List.h>
#include <String.h>
#include <objdce/objtranparams.H>
#include <cdlfield.H>
#include <namedobject.H>

class CDLRecord : public NamedObject

{
public:
  CDLRecord();
  CDLRecord(const String& name, const String& docs,
	    Boolean fromIncludeFile, Boolean makeIdlArray);
  CDLRecord(const CDLRecord&);
  virtual ~CDLRecord();
  void DeleteFields();
  CDLRecord& operator = (const CDLRecord&);
  Boolean AddField(CDLField*);
  const String& Documentation() const;
  int NumFields() const;
  CDLField* Field(const int index);
  CDLField* Find(const String& name);
  Boolean Imported() const;
  Boolean MakeIdlArray() const;
private:
  String documentation;			// Documentation for this type
  List_of_p<CDLField> fields;		// All the fields in the record
  Boolean imported;			// Was it read from an imported file?
  Boolean makeIdlArray;			// Generate array code in IDL?
};

inline const String& CDLRecord::Documentation() const

{ return documentation; }

inline int CDLRecord::NumFields() const
     
{ return fields.length(); }

inline CDLField* CDLRecord::Field(const int index) 

{ return fields[index]; }

inline Boolean CDLRecord::Imported() const

{ return imported; }

inline Boolean CDLRecord::MakeIdlArray() const

{ return makeIdlArray; }

#endif

