#include <dce/idlbase.h>
#include <cdl/cdlerrors.H>
#include <cdlsfsgenerator.H>
#include <cdlrqsgenerator.H>
#include <cdlclassgenerator.H>
#include <cdlfield.H>
#include <cdlindexdef.H>
#include <cdlindexfielddef.H>
#include <cdlsuffixes.H>
#include <unistd.h>

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


/**** gjb -- take this out so people can compile without buying RQS ***
 extern "C"
    {
#include <rqs/rqs.h>
    }
 ****/

/* Map cdl types to rqs stype names */
const String & RqsTypeSymbol(const CDLField::FieldType t, 
			     const CDLField::ModifierType m)
    {
	static const String r("rqs_double");
	static const String b("rqs_byteArray");
	static const String vb("rqs_varLenByteArray");
	static const String s("rqs_string");
	static const String sh("rqs_signedInt64");
	static const String uh("rqs_unsignedInt64");
	static const String fubar("CATASTROPHIC SCREWUP IN CONFIGURATION PROCESSING");
	static const String sl("rqs_signedInt32");
	static const String ul("rqs_unsignedInt32");
	static const String ss("rqs_signedInt16");
	static const String us("rqs_unsignedInt16");
	static const String e("rqs_unsignedInt16");
	switch(t)
	    {
		case CDLField::REAL:
		    return(r);
		case CDLField::BYTES:
	        case CDLField::CHARRAY:
		    return(b);
		case CDLField::VARBYTES:
		    return(vb);
		case CDLField::STRING:
		    return(s);
		case CDLField::HYPER:
		    switch(m)
			{
			    case CDLField::SIGNED:
				return(sh);
			    case CDLField::UNSIGNED:
			    case CDLField::NONE:
				return(uh);
			    default:
				return(fubar);
			}
		case CDLField::LONG:
		    switch(m)
			{
			    case CDLField::SIGNED:
				return(sl);
			    case CDLField::UNSIGNED:
			    case CDLField::NONE:
				return(ul);
			    default:
				return(fubar);
			}
		case CDLField::CHAR:
		case CDLField::SHORT:
		    switch(m)
			{
			    case CDLField::SIGNED:
				return(ss);
			    case CDLField::UNSIGNED:
			    case CDLField::NONE:
				return(us);
			    default:
				return(fubar);
			}
	    case CDLField::ENUM:
		    return(e);
		default:
		    return(fubar);
	    }
    }

/* 
 * CDLRqsGenerator:
 *    This is the main class.
 */

CDLRqsGenerator::CDLRqsGenerator(const ArgList& args)
   : CDLDataGenerator(args)
{
  mainName = "";
  inclName = "";
}

CDLRqsGenerator::~CDLRqsGenerator()

{
}

/* 
 * Start
 *    Create the output files and write there whatever we need to put
 * at the beginning of every file, like #include statements.
 */

Boolean CDLRqsGenerator::Start(const String& fileName)

{
  String symbolName;
  String baseName;
  
  MakeFileName(fileName, mainName, "_cdl_rqs.C");
  MakeFileName(fileName, inclName, "_cdl_rqs.H");
  outFile.open(mainName);
  if (outFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1002, (const char*)mainName);
      return false;
    }
  inclFile.open(inclName);
  if (inclFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1001, (const char*)inclName);
      return false;
    }
  else
    {
      MakeFileName(fileName, symbolName, "_CDL_RQS_H_INCLUDED");
      MakeFileName(fileName, baseName, "_cdl.H");
      inclFile << "/*" << endl;
      inclFile << " * Class generated by Citi-DDL:" << endl;
      inclFile << " * cdlrqsgenerator.C,v 1.19 1994/04/07 19:26:10 gjb Exp" << endl;
      inclFile << " */\n" << endl;
      inclFile << "#ifndef " << symbolName << endl;
      inclFile << "#define " << symbolName << "\n" << endl;
      inclFile << "#include <" << baseName << ">" << endl;
//      inclFile  << "#include <.H>" << endl;
      inclFile << "#include <String.h>" << endl <<
	"#include <List.h>" << endl << 
	"#include <cdl/yqbase.H>" << endl <<
	"#include <cdl/yqname.H>" << endl <<
	"#include <cdl/pack.H>" << endl <<
	"#include <cdl/rqsutil.H>" << endl <<
	"#include <hyperint.H>" << endl;
      inclFile << "extern \"C\" {" << endl;
      inclFile << "#include <rqs/rqs.h>" << endl;
      inclFile << "}" << endl;
      
      outFile << "/*" << endl;
      outFile << " * Class generated by Citi-DDL:" << endl;
      outFile << " */\n" << endl;
      outFile << "#include <" << inclName << ">" << endl;
//      outFile << "CatReport CDLRQSReport("cdl.cat");
      outFile << "extern \"C\" {" << endl;
      outFile << "#include <rqs/rqs.h>" << endl;
      outFile << "}" << endl;
      outFile << "extern CatReport CDLReport;" << endl;
      
      return true;
    }
}

/*
 * End
 *    Finish up the output files and close them.
 */ 

Boolean CDLRqsGenerator::End()

{
  inclFile << "#endif" << endl;
  inclFile << flush;
  inclFile.close();
  outFile << flush;
  outFile.close();
  return true;
}

/* 
 * DeleteFiles
 *    Close and remove the output files. This is used when parsing fails
 * and files should be deleted.
 */

void CDLRqsGenerator::DeleteFiles()

{
  inclFile.close();
  outFile.close();
  unlink(inclName);
  unlink(mainName);
}

/*
 * GetSize
 *    This figures out how much space a particular field type will use
 * in RQS, and thus the value for the "dataSize" and "bufferSize" fields
 * of the various RQS structures. 
 *    gjb -- These sizes must, must, must be the same as the "rqs_" type
 * declarations. These are defined and documented to be a certain number
 * of bits, not the size returned by sizeof, which depends on the machine
 * type and the compiler.
 */

int CDLRqsGenerator::GetSize(CDLField& field) const

{
  int retVal;
  switch (field.Type())
    {
    case CDLField::REAL:
      retVal = 8;
      break;
    case CDLField::HYPER:
      retVal = 8;
      break;
    case CDLField::LONG:
      retVal = 4;
      break;
    case CDLField::SHORT:
    case CDLField::CHAR:
    case CDLField::ENUM:
      retVal = 2;
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      retVal = field.Length();
      break;
    case CDLField::VARBYTES:
      retVal = field.Length() + 4;
      break;
    case CDLField::STRING:
      retVal = field.Length() + 1;
      break;
    case CDLField::RECORD:
      retVal = -1;
      break;
    }
  return retVal;
}

/*
 * BuildFileList
 *    Build a list of all the files in a record, and expand all the nested
 * records as well. The resulting field names, stored in FieldNode objects,
 * name each record from the top level separated by '.', like this: 
 * "record1.record2.field".
 */

Boolean CDLRqsGenerator::BuildFieldList(CDLRecord& rec, 
                                        List<FieldNode>& out)

{
  List<FieldNode> varFields;
  List<FieldNode> nonVarFields;

  ExpandOneRecord(&rec, "", varFields, nonVarFields);
  out = nonVarFields;
  out.put(varFields);
  return true;
}

Boolean CDLRqsGenerator::ExpandOneRecord(CDLRecord* rec, 
                                         const String& name,
                                         List<FieldNode>& varFields,
                                         List<FieldNode>& nonVarFields)
{
  CDLField* curField;
  CDLField* curLengthField;
  FieldNode* newField;
  int inc;
  int arrayPos;
  int realArrayPos;
  Boolean isArray;
  String thisFieldName;
  String varName;
  String lengthName;
  
  for (inc = 0; inc < rec->NumFields(); inc++)
    {
      curField = rec->Field(inc);
      isArray = (curField->ArraySize() > 1);
      for (arrayPos = 0; arrayPos < curField->ArraySize(); arrayPos++)
	{
	  if (isArray)
	    {
	      realArrayPos = arrayPos;
	    }
	  else
	    {
	      realArrayPos = -1;
	    }
	  GenWholeFieldName('.', curField->Name(), name, 32,
			    thisFieldName, realArrayPos);
	  GenWholeFieldName('.', curField->Name(), name, 0,
			    varName, realArrayPos);
	  if (IsRecord(curField->Type()))
	    {
	      thisFieldName += ".";
	      ExpandOneRecord(curField->Record(), thisFieldName,
			      varFields, nonVarFields);
	    }
	  else
	    {
	      newField = new FieldNode(thisFieldName, curField);
	      newField->VariableName(varName);
	      if (curField->Type() == CDLField::VARBYTES)
		varFields.put(*newField);
	      else
		nonVarFields.put(*newField);
	      delete newField;
	      if (curField->Type() == CDLField::BYTES)
		{
		  lengthName = curField->Name() + CDLByteArrayLengthSuffix;
		  curLengthField = new CDLField(CDLField::LONG,
						CDLField::UNSIGNED,
						lengthName, 1);
		  GenWholeFieldName('.', lengthName, name, 32,
				    thisFieldName, realArrayPos);
		  GenWholeFieldName('.', lengthName, name, 1024,
				    varName, realArrayPos);
		  newField = new FieldNode(thisFieldName, curLengthField);
		  newField->VariableName(varName);
		  nonVarFields.put(*newField);
		  delete curLengthField;
		  delete newField;
		}
	    }
	}
    }
  return true;
}

/* 
 * Generate
 *    Generate all the files by calling the other generation functions 
 * in turn.
 */

Boolean CDLRqsGenerator::Generate(CDLRecord& rec, CDLFileDef& file)

{
  Boolean retVal;
  retVal = GenSubclasses(rec, file);
  if (retVal)
    retVal = GenHeader(rec, file);
#if 0
  if (retVal)
    retVal = GenMain(rec, file);
  if (retVal)
    retVal = GenModifiers(rec, file);
#endif
  return retVal;
}

#if 0
/*
 * GenField
 *    Write to the output file the code necessary to make the external
 * field structures in the array named by prefix point to the corresponding
 * fields in the main IDL struct (pointed to by a pointer by a pointer
 * called "base"). 
 */

Boolean CDLRqsGenerator::GenField(CDLField& curField,
				  const String& prefix, 
				  const String& rqsFieldName, int ent)
{
  String fieldName;
  int dataLen;

  fieldName = "base->";
  fieldName += rqsFieldName;
  dataLen = GetSize(curField);
  
  outFile << indent << prefix << "[" << ent;
  outFile << "].name = \"" << rqsFieldName << "\";" << endl;
  switch(curField.Type())
    {
    case CDLField::STRING:
      outFile << indent << prefix << "[" << ent;
      outFile << "].buffer = (rqs_pointer_t)(" << fieldName;
      outFile << ");" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].bufferLength = " << dataLen << ";" << endl;
      /* Replace when RQS string bug is fixed *
      outFile << indent << prefix << "[" << ent;
      outFile << "].dataLength = strlen((const char*)";
      outFile << fieldName << ") + 1;" << endl;
      */
      outFile << indent << prefix << "[" << ent;
      outFile << "].dataLength = " << dataLen << ";" << endl;
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << indent << prefix << "[" << ent;
      outFile << "].buffer = (rqs_pointer_t)(" << fieldName;
      outFile << ");" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].bufferLength = " << dataLen << ";" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].dataLength = " << dataLen << ";" << endl;
      break;
    case CDLField::VARBYTES:
      outFile << indent << prefix << "[" << ent;
      outFile << "].buffer = (rqs_pointer_t)(" << fieldName;
      outFile << ");" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].bufferLength = " << dataLen << ";" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].dataLength = " << fieldName << CDLByteArrayLengthSuffix;
      outFile << ";" << endl;
      break;
    default:
      outFile << indent << prefix << "[" << ent;
      outFile << "].buffer = (rqs_pointer_t)&(" << fieldName;
      outFile << ");" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].bufferLength = " << dataLen << ";" << endl;
      outFile << indent << prefix << "[" << ent;
      outFile << "].dataLength = " << dataLen << ";" << endl;
      break;
    }
  return true;
}

/*
 * GenFieldSpec
 *    In a similar manner to GenField, set the fields of a an RQS field
 * specification structure to refer to a particuar CDL field.
 */

Boolean CDLRqsGenerator::GenFieldSpec(CDLField& curField, 
				      const String& prefix,
				      const String& rqsFieldName,
				      int ent)
{
  int dataLen;
  dataLen = GetSize(curField);

  outFile << indent << prefix << "[" << ent << "].fieldName = \"";
  outFile << rqsFieldName << "\";" << endl;
  outFile << indent << prefix << "[" << ent << "].fieldType = ";
  switch(curField.Type())
    {
    case CDLField::REAL:
      outFile << "rqs_double;" << endl;
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << "rqs_byteArray;" << endl;
      break;
    case CDLField::VARBYTES:
      outFile << "rqs_varLenByteArray;" << endl;
      break;
    case CDLField::STRING:
      outFile << "rqs_string;" << endl;
    break;
    case CDLField::HYPER:
      if (curField.Modifier() == CDLField::SIGNED)
	outFile << "rqs_signedInt64;" << endl;
      else
	outFile << "rqs_unsignedInt64;" << endl;
      break;
    case CDLField::LONG:
      if (curField.Modifier() == CDLField::SIGNED)
	outFile << "rqs_signedInt32;" << endl;
      else
	outFile << "rqs_unsignedInt32;" << endl;
      break;
    case CDLField::SHORT:
    case CDLField::CHAR:
      if (curField.Modifier() == CDLField::SIGNED)
	outFile << "rqs_signedInt16;" << endl;
      else
	outFile << "rqs_unsignedInt16;" << endl;
      break;
    case CDLField::RECORD:
      break;
    }
  outFile << indent << prefix << "[" << ent << "].fieldSize = ";
  outFile << dataLen << ";" << endl;
  return true;
}

/*
 * GenIndexField
 *    Set the fields of a secondary index field specification to refer
 * to the appropriate field in the index.
 */

Boolean CDLRqsGenerator::GenIndexField(CDLIndexFieldDef& curIField,
				       const String& prefix, int ent)
{
  outFile << indent << prefix << "[" << ent << "].fieldName = \"";
  outFile << curIField.Name() << "\";" << endl;
  outFile << indent << prefix << "[" << ent << "].indexFieldOrdering = ";
  if (curIField.Direction() == BACKWARDS)
    outFile << "sfs_descending;" << endl;
  else
    outFile << "sfs_ascending;" << endl;
  return true;
}
#endif
/*
 * GenSubclasses
 *    Generate the definitions and implementations of the classes declared
 *in GenHeader.
 */
      
Boolean CDLRqsGenerator::GenSubclasses(CDLRecord& rec, CDLFileDef& file)

{
  int inc;
  int ent;
  int recursiveCount;
  String recName;
  String recBaseName;
  String fileName;
  String className;
  String tmpName;
  List<FieldNode> recursiveFields;
  Listiter<FieldNode> fieldIter(recursiveFields);
  FieldNode curNode;
  
  recName = rec.Name();
  recBaseName = rec.Name();
  recBaseName += "Base";
  fileName = file.Name();
  className = "RQS";
  className += fileName;
  className += "Record";
  
  BuildFieldList(rec, recursiveFields);
  recursiveCount = recursiveFields.length();
    /* output index field name arrays */
    for (inc = 0; inc < file.NumIndices(); inc++)
	{
	    CDLIndexDef & idx = *file.Index(inc);
	    outFile << "const char * const YQ_" << recName << "_" << 
	    idx.Name() << "_Name[" << idx.NumFields() << "] = {" << endl;
	    for (ent = 0; ent < idx.NumFields(); ent++)
		{
		    CDLIndexFieldDef & fld = *idx.Field(ent);
		    if (ent) outFile << "\"," << endl;
		    outFile << "    \"" << fld.Name();
		} /* for (ent = 0; ent < idx.NumFields(); ent++) */
	    outFile << "\"};" << endl;
	} /* for (inc = 0; inc < file.NumIndices(); inc++) */

    /* output global record specification */
    outFile << "static const rqs_recordFieldSpec_t " <<
	recName << "_fieldSpecArray[" << recursiveCount << "] =" << endl <<
"    {" << endl;
    inc = 0;
    for (fieldIter.reset(); fieldIter.next(curNode);)
	{
	    if (inc++) /* put comma after preceeding array element */
		outFile << ","; 
	    outFile << endl << "	    {" << endl <<
		"		\"" << curNode.Name() << 
		"\", /* fieldName */" << endl <<
		"		" << 
		RqsTypeSymbol(curNode.Field()->Type(), 
			      curNode.Field()->Modifier());
	    outFile << ",  /* fieldType */" << endl << "		" << 
#if 0
		curNode.Field()->Length() <<
#else
		GetSize(*curNode.Field()) <<
#endif
		",		    /* fieldSize */" << endl <<
		"		(char *)0	    /* collatingLanguage */" 
		<< endl << "	    } /* [" << (inc-1) << "] */";
	}
    outFile << "    };" << endl;
    /* output global key specification */
    outFile <<
"static const rqs_keySpec_t " << recName << "_keySpec[" <<  
	file.NumIndices() << "] =" << endl << "    {" << endl;
    for (inc = 0; inc < file.NumIndices(); inc++)
	{
	    CDLIndexDef & idx = *file.Index(inc);
	    if (inc) outFile << ","; /*	output comma to separate last entry
				      *	from this entry */
	    outFile << endl << "	    {   " << "\"";
	    outFile << idx.Name() << "\", /* keyName */" << endl;
	    outFile << "		" << idx.NumFields();
	    outFile << ", /* numFields */" << endl << "		(char**)YQ_";
	    outFile << recName << "_" << idx.Name() << "_Name"
			 << endl << "	    }";
	} /* for inc */
  outFile << "    };" << endl;
  outFile << "static const rqs_typeSpec_t " << recName;
  outFile << "_typeSpec =" << endl << "    {" << endl;
  outFile << "	" << recursiveCount << ", /* recordSpec.numFields */" << endl;
  outFile << "	(rqs_recordFieldSpec_t*)&" <<
	recName <<
	"_fieldSpecArray, /* recordSpec.fieldSpecArray*/" << endl;
  outFile << "	" << file.NumIndices() << ", /* numKeys */" << endl <<
    "	(rqs_keySpec_t*)&" << recName;
  outFile << "_keySpec" << endl << "    };" << endl;
  outFile << endl << recName << "_YQtype global_" << recName << "_YQtype;" << 
	endl;

    /* output code */

   // Cursor::DropLock
  outFile << "CDLStatus " << recName;
  outFile << "_YQcursor::DropLock()" << endl << "    {" << endl;
  outFile << "  ExistingTransaction tran;" << endl;
  outFile << "	RqsStatusValue = rqs_CursorDropLock(cursor);" << endl <<
    "	return CheckError(20001, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "    }" << endl << endl;

   // constructors for cursor
  outFile << recName << "_YQcursor::" << recName << "_YQcursor(" << endl;
  outFile << "	const String&ApplicationName," << endl;
  outFile << "	const CursorPolicy cp," << endl <<
    "	const ElementLockMode elm," << endl;
  outFile << "	const char* server) : YQbase(ApplicationName)," << endl;
  outFile << "			 yQtype(global_" <<
    recName <<
    "_YQtype)," << endl <<
    "			 RqsBinding(server)" << endl;
  outFile << "    {" << endl <<
    "	YQname queue_name(aName, yQtype.TypeName);" << endl <<
    "	Boundless();" << endl;
  outFile << "ExistingTransaction tran;" << endl;
  outFile << "	CDLReport.Report(ErrorLevels::TraceMsg, \"Calling rqs_CursorCreate\\n\");" << endl;
  outFile << "	RqsStatusValue = rqs_CursorCreate(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl;
  outFile << "			    (char *)(const char *)queue_name.FullName()," << endl;
  outFile << "			    (rqs_cursorPolicy_t)cp," << endl <<
    "			    (rqs_elementLockMode_t)elm," << endl;
  outFile << "			    &cursor);" << endl <<
    "	CheckError(20002, RqsStatusValue, tran, ErrorLevels::InfoMsg);" ;
  outFile <<   "    }" << endl;

   // constructor 
  outFile << endl << recName << "_YQcursor::" << recName << "_YQcursor(" << endl;
  outFile << "			const " << recName;
  outFile << "_YQcursor & src) :" << endl <<
    "			    YQbase(src)," << endl;
  outFile << "			    yQtype(src.yQtype)," << endl <<
    "			    RqsBinding(src)" << endl;
  outFile << "    {" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile <<
    "	RqsStatusValue = rqs_CursorCopy(cursor, (rqs_cursor_t*)&src.cursor);" << endl;
  outFile << "	CheckError(20002, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "    }" << endl << endl;

   // cursor Next 
  outFile << "CDLStatus " << recName << "_YQcursor::Next(" << endl;
  outFile << "	" << recName << " & data," << endl;
  outFile << "	HyperInt & work)" << endl <<
    "    {" << endl <<
    "	CDLStatus ret;" << endl <<
    "	rqs_elementDescriptor_t *elementP;" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	CDLReport.Report(ErrorLevels::TraceMsg, \"Calling rqs_CursorReadNext\\n\");" << endl;
  outFile << "	RqsStatusValue = rqs_CursorReadNext(cursor, &elementP);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    {" << endl <<
    "		/* sanity check on type name */" << endl;
  outFile << "		if (strcmp(yQtype.TypeName, elementP->elementType))" << endl;
  outFile << "		    {" << endl;
  outFile << "		    ThrowError(20004, CDL_TYPE_MISMATCH, tran, ErrorLevels::ErrorMsg);" 
    << endl;
  outFile << "          return CDL_TYPE_MISMATCH; " << 
    endl << "		    }" << endl;
  outFile << "		Unpack(elementP->value, data);" << endl <<
    "		ndr_uhyper_int_rep x;" << endl;
  outFile << "		Assign(x, elementP->work);" << endl <<
    "		work = x;" << endl;
  outFile << "		rqs_Free((char*)elementP);" << endl <<
    "	    }" << endl;
  outFile << "	else ret = CheckError(20005, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "return ret;" << endl;
  outFile << "    }" << endl << endl ;

   // next operation for multiple elements
  outFile << "CDLStatus ";
  outFile << recName << "_YQcursor::Next(" << endl << 
    "	const unsigned int DesiredNumberOfElements," << endl <<
    "	List<";
  outFile << recName << "> & ElementList," << endl << 
    "	List<HyperInt>& WorkList)" << endl;
  outFile << "    {" << endl <<
    "	rqs_elementDescriptor_t *elementP;" << endl <<
    "	unsigned int ReturnedNumberOfElements;" << endl <<
    "	rqs_status_t lastStatusP;" << endl;
  outFile << "  ExistingTransaction tran;" << endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Calling rqs_CursorReadSequence for %u elements\\n\",DesiredNumberOfElements);" << endl;
  outFile << "	RqsStatusValue = rqs_CursorReadSequence(" << endl <<
    "			    cursor," << endl;
  outFile << "			    DesiredNumberOfElements," << endl <<
    "			    &ReturnedNumberOfElements," << endl;
  outFile << "			    &elementP," << endl <<
    "			    &lastStatusP );" << endl <<
    "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "	    {	/* clean return lists then" << endl <<
    " *		   copy from elementP to return lists */" << endl;
outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"rqs_CursorReadSequence read in %u elements\\n\",ReturnedNumberOfElements);" << endl;
  outFile << "		while (ElementList)" << endl <<
    "		    ElementList.unput();" << endl;
  outFile << "		while (WorkList)" << endl <<
    "		    WorkList.unput();" << endl <<
    "		int i;" << endl;
  outFile << "		for (i = 0; i < ReturnedNumberOfElements &&" << endl <<
    "			    i <	DesiredNumberOfElements; i++)" << endl;
  outFile << "		    {" << endl <<
    "			" << recName << " e;" << endl;
  outFile << "			HyperInt h;" << endl <<
    "			Unpack(elementP[i].value, e);" << endl;
  outFile << "			ndr_uhyper_int_rep x;" << endl <<
    "			Assign(x, elementP[i].work);" << endl <<
    "			h = x;" << endl;
  outFile << "			ElementList.put(e);" << endl <<
    "			WorkList.put(h);" << endl <<
    "		    }" << endl;
  outFile << endl << endl << "		rqs_Free((char*)elementP);" << endl;
  outFile << "        return CDL_SUCCESS;" << endl;
  outFile << "	    }" << endl;
  outFile << "	else " << endl;
  outFile << "      return CheckError(20005, RqsStatusValue, tran, ErrorLevels::InfoMsg);" 
    << endl ;
  outFile << "    }" << endl << endl;
  
   // Cursor reset 
  outFile << "CDLStatus ";
  outFile << recName << "_YQcursor::Reset()" << endl << "    {" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	if ((RqsStatusValue = rqs_CursorReset(cursor)) == RQS_SUCCESS)" << endl;
  outFile << "	    return CDL_SUCCESS;" << endl;
  outFile << "  else" << endl;
  outFile << "      return CheckError(20006, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl;

/* cursor destructor */
  outFile << endl << recName << "_YQcursor::~" << recName << "_YQcursor()" << endl;
  outFile << "    {" << endl << "	rqs_Free((char*)cursor);" << endl;
  outFile << "    }" << endl << endl;

/* delete cursor */
  outFile << "CDLStatus " ;
  outFile << recName << "_YQelement::Delete(" << endl;
  outFile << "	const YQelementID& eid," << endl <<
    "	const DeleteOption delopt," << endl <<
    "	" <<
    recName;
  outFile << "& e)" << endl << "    {" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	rqs_elementDescriptor_t * q;" << endl;
  outFile << "	CDLReport.Report(ErrorLevels::TraceMsg, \"Calling rqs_ElementDelete\\n\");" << endl;
  outFile << "	RqsStatusValue = rqs_ElementDelete(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl;
  outFile << "			    (rqs_elementId_t*)&eid," << endl <<
    "			    (rqs_elementDeleteOption_t)delopt," << endl;
  outFile << "			    &q);" << endl <<
    "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "		{" << endl;
  outFile << "		ret = CDL_SUCCESS;" << endl;
  outFile << "		Unpack(q->value, e);" << endl;
  outFile << "		}" << endl;
  outFile << "	else" << endl;
  outFile <<
    "		ret = CheckError(20007, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl << endl;
  outFile << "	rqs_Free((char*)q);" << endl;
  outFile << "	return ret;" << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName;
  outFile << "_YQelement::Modify(" << endl <<
    "	const YQelementID& eid," << endl <<
    "	const ";
  outFile << recName << "& src)" << endl << "    {" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	unsigned long qlen =" << endl;
  outFile << "	    FieldSizeTotal(" << recName <<
    "_typeSpec.recordSpec);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	rqs_pointer_t q = (rqs_pointer_t)new char[qlen];" << endl;
  outFile << "	void * buffer_end = Pack(q, src);" << endl;
  outFile << "   unsigned long buffer_len = (char *)buffer_end - (char *)q;"<<endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Calling rqs_ElementModify. Type: %s\\n\",(char*)(const char*)yQtype.TypeName);" << endl;
  outFile << "	RqsStatusValue = rqs_ElementModify(" << endl;
  outFile << "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (rqs_elementId_t*)&eid," << endl <<
    "			    (char*)(const char*)yQtype.TypeName," << endl;
  outFile << "			    buffer_len," << endl <<
    "			    q);" << endl <<
    "	delete [] q;" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    return CDL_SUCCESS;" << endl;
  outFile << "	else" << endl;
  outFile << "	    return CheckError(20008, RqsStatusValue, tran, ErrorLevels::InfoMsg);" 
    << endl;
  outFile << "    }" << endl << endl;

  /* constructor for YQelement */
  outFile << recName << "_YQelement::" << recName << "_YQelement(" << endl;
  outFile << "	const String&ApplicationName," << endl;
  outFile << "	const " << recName << "_YQtype&," << endl <<
    "	const char* Server) : YQbase(ApplicationName)," << endl;
  outFile << "			       yQtype(global_" <<
    recName << "_YQtype)," << endl;
  outFile << "			       RqsBinding(Server)" << endl;
  outFile << "    {" << endl << endl << endl << "    }" << endl << endl;
  
  /* element read */
  outFile << "CDLStatus " << recName <<"_YQelement::Read(" << endl <<
    "	const YQelementID& eid ," << endl;
  outFile << "	const ElementLockMode elm ," << endl <<
    "	const Boolean retainLock," << endl;
  outFile << "	" << recName << "& dst)" << endl <<
    "    {" << endl << "	Boundless();" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	CDLReport.Report(ErrorLevels::TraceMsg, \"Calling rqs_ElementRead in YQelement::Read\\n\"); " << endl;
  outFile << "	rqs_elementDescriptor_t * elementP;" << endl <<
    "	RqsStatusValue = rqs_ElementRead(" << endl;
  outFile << "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (rqs_elementId_t*)&eid," << endl;
  outFile << "			    (rqs_elementLockMode_t)elm," << endl;
  outFile << "			    retainLock," << endl <<
    "			    &elementP);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    {" << endl;
  outFile << "		Unpack(elementP->value, dst);" << endl;
  outFile << "		ret = CDL_SUCCESS;" << endl;
  outFile << "	    }" << endl;
  outFile << "	else" << endl;
  outFile <<
    "		ret = CheckError(20009, RqsStatusValue, tran, ErrorLevels::InfoMsg);" <<
    endl << endl;
  outFile << "	rqs_Free((char*)elementP);" << endl;
  outFile << "	return ret;" << endl;
  outFile << "    }" << endl << endl;

  /* element Unlock */
  outFile << "CDLStatus " << recName << "_YQelement::Unlock(" << endl;
  outFile << "	const YQelementID& eid )" << endl << "    {" << endl <<
    "	rqs_status_t result;" << endl ;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	CDLReport.Report(ErrorLevels::TraceMsg, \"Calling rqs_ElementListDropLocks in YQelement:Unlock\\n\");" << endl;
  outFile << "	RqsStatusValue = rqs_ElementListDropLocks(" << endl;
  outFile << "				(rqs_serverHandle_t)Handle()," << endl;
  outFile <<
    "				rqs_readLock, 1," << endl <<
    "				(rqs_elementId_t*)&eid," << endl <<
    "				&result);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    return CDL_SUCCESS;" << endl <<
    "	else" << endl;
  outFile <<
    "	    return CheckError(20010, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  /* element destructor */
  outFile << recName << "_YQelement::~" <<
    recName << "_YQelement()" << endl << "    {" << endl;
  outFile << endl << endl << "    }" << endl << endl << endl;


// key constructor
  outFile <<
    endl << recName << "_YQkey::" << recName << "_YQkey(" << endl;
  outFile << "	const char* Server) : YQbase(\"\")," << endl <<
    "			       yQtype(global_" <<
    recName << "_YQtype)," << endl ;
  outFile << "			       RqsBinding(Server)" << endl <<
    "    {" << endl << endl << endl << "    }" << endl << endl;

// key:: Read
  outFile << "CDLStatus " << recName << "_YQkey::Read(" << endl;
  outFile << "	const Index i," << endl << "	const " << recName <<
    "& keydata," << endl <<
    "	const ElementLockMode elm," << endl;
  outFile << "	const unsigned int maxReturnCount," << endl;
  outFile << "	List<YQelementID>& eid," << endl <<
    "	List<" << recName << ">& data, " << endl <<
    "   char*  QueueName) " << endl <<
    "    {" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	if (i < 0 || i >= yQtype.TypeSpecification.numKeys)" << endl;
  outFile << "	    return ThrowError(20011, CDL_INVALID_INDEX, tran, ErrorLevels::ErrorMsg);"
    << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	rqs_key_t key;" << endl <<
    "	key.typeName=(char*)(const char*)yQtype.TypeName;" << endl <<
    "	key.keyName = yQtype.TypeSpecification.keySpecArray[i].keyName;" << endl <<
    "   key.valueLen = FieldSizeTotal(yQtype.TypeSpecification, i);" << endl;
  outFile << "	key.value = new char[key.valueLen];" << endl <<
    "	Pack(key.value, keydata, i);" << endl <<
    "   key.valueLen = buff_length;" << endl <<
    "   unsigned int addMatchCount;" << endl <<
    "	unsigned int numMatches;" << endl;
  outFile << "	rqs_elementDescriptor_t * matchingElementArray;" << endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Calling rqs_RetrieveByKey - Index: %d - Type: %s - Name: %s\\n\"," << endl;
  outFile << "			  i, key.typeName, key.keyName);" << endl;
  outFile << "	RqsStatusValue = rqs_RetrieveByKey(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl;
  outFile << "			    &key," << endl <<
    "			    (rqs_elementLockMode_t)elm," << endl;
  outFile << "			    1, /* always retain lock */" << endl <<
    "			    1, /* always include values */" << endl <<
    "			    maxReturnCount," << endl;
  outFile << "		    &numMatches,"  <<endl <<
    "			    &matchingElementArray);" << endl <<
    "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "	    {" << endl <<
    "		/* unpack returned data */" << endl <<
    "		while(eid)" << endl;
  outFile << "		    eid.unput();" << endl <<
    "		while(data)" << endl <<
    "		    data.unput();" << endl <<
    "		int i;" << endl <<
    "           String str = \"\";" << endl <<
    "     if((strcmp(QueueName,str))==0)" << endl <<
    "        {" << endl <<
    "      		for (i = 0; i < numMatches; i++)" << endl;
  outFile << "		    {" << endl <<
    "			YQelementID eltid(matchingElementArray[i].eltId);" << endl <<
    "			eid.put(eltid);" << endl <<
    "			" << recName << " rec;" << endl <<
    "			Unpack(matchingElementArray[i].value, rec);" << endl <<
    "			data.put(rec);" << endl;
  outFile << "		    }" << endl;
  outFile << " }" << endl;
  outFile << " else   { "<< endl <<
    "      		for (i = 0; i < numMatches; i++)" << endl <<
    "                     {" << endl;
  outFile << "if((strcmp(QueueName,matchingElementArray[i].containingQueue))==0)" << endl;

// gjb 11/29/93 -- The next three lines seem like they don't belong.
// They certainly don't compile.
//    outFile<<"		int i;" << endl;
//    outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Number Of Matches: %u\\n\", numMatches);" << endl;
//    outFile << "      		for (i = 0; i < numMatches; i++)" << endl;

  outFile << "		    {" << endl <<
    "			YQelementID eltid(matchingElementArray[i].eltId);" << endl <<
    "			eid.put(eltid);" << endl <<
    "			" << recName << " rec;" << endl <<
    "			Unpack(matchingElementArray[i].value, rec);" << endl <<
    "			data.put(rec);" << endl;
  outFile << "		    }" << endl;
  outFile << "   }" << endl;      
  outFile << " }" << endl;      
  outFile << "		ret = CDL_SUCCESS;" << endl;
  outFile << "	    }" << endl << "	else" << endl;
  outFile << "		ret = CheckError(20033, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "	rqs_Free((char*)matchingElementArray);" << endl;
  outFile << "	delete [] key.value;" << endl;
  outFile << "	return ret;" << endl;
  outFile << "    }" << endl << endl;

// key:: Read
  outFile << "CDLStatus " << recName << "_YQkey::Read(" << endl;
  outFile << "	const List<Index> idxlst," << endl <<
    "	const List<" << recName << ">& keydata," << endl;
  outFile << "	const ElementLockMode elm," << endl;
  outFile << "	const unsigned int maxReturnCount," << endl;
  outFile << "	List<YQelementID>& eid," << endl;
  outFile << "	List<" << recName << ">& data)" << endl;
  outFile << "    {" << endl <<
    "	const unsigned int numKeys = idxlst.length();" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	unsigned int numMatches;" << endl;
  outFile << "	Const_listiter<Index> idxlst_iter(idxlst);" << endl;
  outFile << "	Const_listiter<" << recName << "> kdlst_iter(keydata);" <<endl;
  outFile << "	rqs_elementDescriptor_t * matchingElementArray;" << endl;
  outFile << "  unsigned int addMatchCount;"<< endl;
  outFile << "	while (!idxlst_iter.at_end())" << endl;
  outFile << "	    {" << endl;
  outFile << "		Index i = *idxlst_iter.next();" << endl;
  outFile << "		if (i < 0 ||" << endl;
  outFile << "		    i >= yQtype.TypeSpecification.numKeys)" << endl;
  outFile << 
    "		return ThrowError(20011, CDL_INVALID_INDEX, tran, ErrorLevels::ErrorMsg);"
    << endl;
  outFile << "	}" << endl;
  outFile << "	Boundless();" << endl <<
    "	rqs_key_t * keyArrayP = new rqs_key_t[keydata.length()];" << endl;
  outFile << "	int loop = 0;" << endl;
  outFile << "	idxlst_iter.reset();" << endl <<
    "	while(!kdlst_iter.at_end() && !idxlst_iter.at_end())" << endl;
  outFile << "	    {	/* build up the key list */" << endl;
  outFile << "		Index i = *idxlst_iter.next();" << endl;
  outFile << "		" << recName << " & k = *kdlst_iter.next();" << endl;
  outFile << "		rqs_key_t & key = keyArrayP[loop++];" << endl;
  outFile << "		key.typeName=(char*)(const char*)yQtype.TypeName;"
    << endl;
  outFile <<
    "		key.keyName=yQtype.TypeSpecification.keySpecArray[i].keyName;"
    << endl;
  outFile <<
    "		key.valueLen = FieldSizeTotal(yQtype.TypeSpecification, i);"
    << endl;
  outFile << "		key.value = new char[key.valueLen];" << endl;
  outFile << "		Pack(key.value, k, i);" << endl; 
  outFile << "          key.valueLen = buff_length;" << endl;
  outFile << "		CDLReport.VReport(ErrorLevels::TraceMsg, \"Building Key in array - Index: %d - Type: %s - Name: %s\\n\"," << endl;
  outFile << "				  i, key.typeName, key.keyName);" << endl;
  outFile << "	    }" << endl;

  outFile << "	RqsStatusValue = rqs_RetrieveByKeys(" << endl;
  outFile << "		    (rqs_serverHandle_t)Handle()," << endl <<
    "		    numKeys," << endl <<
    "		    keyArrayP," << endl;
  outFile << "		    (rqs_elementLockMode_t)elm," << endl;
  outFile << "		    1, /* always retainLock,*/" << endl;
  outFile << "		    1, /* always include values */" << endl;
  outFile << "		    maxReturnCount," << endl;
  outFile << "              &numMatches," << endl <<
    "		    &matchingElementArray);" << endl;
  outFile << "	/* handle the return values */" << endl <<
    "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "	    {	/* unpack return values */" << endl;
  outFile << "		CDLReport.VReport(ErrorLevels::TraceMsg, \"Read Record by key array -- # Matches: %u\\n\", numMatches);" << endl;
  outFile << "		while(eid)" << endl;
  outFile << "		    eid.unput();" << endl;
  outFile << "		while(data)" << endl;
  outFile << "		    data.unput();" << endl;
  outFile << "		int i;" << endl;
  outFile << "		for (i= 0; i < numMatches; i++)" << endl;
  outFile << "		    {" << endl <<
    "			YQelementID eltid(matchingElementArray[i].eltId);"
    << endl <<
    "			eid.put(eltid);" << endl;
  outFile << "			" << recName << " rec;" << endl;
  outFile << "			Unpack(matchingElementArray[i].value, rec);"
    << endl;
  outFile << "			data.put(rec);" << endl;
  outFile << "		    }" << endl;
  outFile << "		rqs_Free((char*)matchingElementArray);" << endl;
  outFile << "		ret = CDL_SUCCESS;" << endl;
  outFile << "	    }" << endl <<
    "	else" << endl;
  outFile << "	    {" << endl <<
    "		ret = CheckError(20033, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl << "	    }" << endl;
  outFile << "	for (int inc = 0; inc < keydata.length(); ++ inc)" << endl;
  outFile << "	    delete [] keyArrayP[inc].value;" << endl;
  outFile << "	delete [] keyArrayP;" << endl <<
    "	return ret;" << endl <<
    "    }" << endl << endl;

/* key destructor */
  outFile << recName << "_YQkey::~" << recName << "_YQkey()" << endl;
  outFile << "    {" << endl << endl << "    }" << endl << endl ;

  outFile << "CDLStatus " << recName << "_YQqueue::AcquireExclusiveAccess()"
    << endl;
  outFile << "    {" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	RqsStatusValue = rqs_AcquireExclusiveAccess(" << endl;
  outFile << "			    (rqs_serverHandle_t)Handle()," << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName()"
    << endl << "			    );" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "	    return CDL_SUCCESS;" << endl <<
    "	else" << endl;
  outFile << "	    return CheckError(20012, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << 
    "_YQqueue::CreateQueue(const Boolean workAccumulationFlag)" << endl;
  outFile << "    {" << endl <<
    "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Creating Queue: %s\\n\", (char*)(const char *)queue_name.FullName());" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	RqsStatusValue = rqs_QCreate((rqs_serverHandle_t)Handle(),"
    << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName(),"
    << endl;
  outFile << "			    (workAccumulationFlag ?" << endl;
  outFile << "			    rqs_workAccumulationEnabled :" << endl;
  outFile << "			    rqs_workAccumulationDisabled)" << endl;
  outFile << "			    );" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS) return CDL_SUCCESS;"
    << endl;
  outFile << "	else return CheckError(20013, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQqueue::DeleteAllElements()"
    << endl;
  outFile << "    {" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Deleting All Elements from Queue: %s\\n\", (char*)(const char*)queue_name.FullName());" << endl;
  outFile <<
    "	RqsStatusValue = rqs_DeleteAllElements((rqs_serverHandle_t)Handle(),"
    << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName());"
    << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS) return CDL_SUCCESS;"
    << endl;
  outFile << "	else return CheckError(20014, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQqueue::Dequeue(" << endl;
  outFile << "	" << recName << "& dst ," << endl;
  outFile << "	const DeleteOption dltopt," << endl <<
    "	YQelementID& eid," << endl;
  outFile << "	const Boolean immediateReturn," << endl <<
    "	HyperInt& work)" << endl;
  outFile << "    {" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	rqs_elementDescriptor_t *elementP;" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Dequeing From Queue: %s - Element Disposition: %s\\n\"," << endl;
  outFile << "			  (char*)(const char *)queue_name.FullName(), (dltopt == Delete) ? \"Delete\" : \"Orphan\");" << endl;
  outFile << "	RqsStatusValue = rqs_Dequeue((rqs_serverHandle_t)Handle(),"
    << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName(),"
    << endl;
  outFile <<
    "			    (dltopt == Delete ? rqs_deleteElement :" << endl;
  outFile << "						rqs_orphanElement),"
    << endl;
  outFile << "			    immediateReturn," << endl;
  outFile << "			    &elementP);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    {" << endl <<
    "		Unpack(elementP->value, dst);" << endl;
  outFile <<"		ndr_uhyper_int_rep x;" << endl;
  outFile <<"		Assign(x, elementP->work);" << endl <<
    "		work = x;" << endl ;
  outFile << "		eid = (YQelementID)elementP->eltId;" << endl;
  outFile << "		ret = CDL_SUCCESS;" << endl;
  outFile << "	    }" << endl;
  outFile << "	else" << endl;
  outFile <<
    "	    ret = CheckError(20022, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "	rqs_Free((char*)elementP);" << endl;
  outFile << "	return ret;" << endl <<
    "    }" << endl << endl;

// destroy queue
  outFile << "CDLStatus " << recName << "_YQqueue::DestroyQueue()" << endl;
  outFile << "    {" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Destroying Queue: %s\\n\", (char*)(const char *)queue_name.FullName());" << endl;
  outFile << "	RqsStatusValue = rqs_QDestroy((rqs_serverHandle_t)Handle(),"
    << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName());"
    << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS) return CDL_SUCCESS;"
    << endl;
  outFile << "	else return CheckError(20015, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

// enable transactional access
  outFile << "CDLStatus " << recName << "_YQqueue::EnableTransactionalAccess()"
    << endl;
  outFile << "    {" << endl <<
    "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"EnableTransactionalAccess for Queue: %s\\n\", (char*)(const char *)queue_name.FullName());" << endl;
  outFile << "	RqsStatusValue = rqs_EnableTransactionalAccess(" << endl;
  outFile << "			    (rqs_serverHandle_t)Handle()," << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName());"
    << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS) return CDL_SUCCESS;" << endl;
  outFile << "	else return CheckError(20016, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

// queue:: Statistics

  outFile << "CDLStatus " << recName << "_YQqueue::QStat(" << endl ;
  outFile << "	unsigned long& Qlength," << endl;
  outFile << "	 HyperInt& work)" << endl;
  outFile << " { " << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	rqs_queueStat_t *statP;" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	RqsStatusValue = rqs_QStat((rqs_serverHandle_t)Handle(),"<<endl <<  "              (char *)(const char *)queue_name.FullName(),"<< endl <<
     "              &statP);" <<endl;
  outFile << "  if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "	   {" << endl;
  outFile << "	     Qlength = statP->queueStats.currentElementCount;" << endl;
  outFile << "	     ndr_uhyper_int_rep x;" << endl;
  outFile << "	     Assign(x, statP->queueStats.currentWork);" << endl;
  outFile << "	     work = x;" << endl;
  outFile << "	     rqs_Free((char*)statP);" << endl;
  outFile << "	     ret = CDL_SUCCESS;" << endl;
  outFile << "	   }" << endl;
  outFile << "	 else  {" << endl;
  outFile << "            ret = CheckError(20034, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "	       }" << endl;
  outFile << "	 return ret;" << endl;
  outFile << "	}" << endl << endl;
 
// queueu:: Enqueue
  outFile << "CDLStatus " << recName << "_YQqueue::Enqueue(" << endl ;
  outFile << "	YQelementID& eid," << endl;
  outFile << "	const " << recName << "& src," << endl << 
    "	const HyperInt& work)" << endl;
  outFile << "    {" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl <<
    "	unsigned long qlen =" << endl <<
    "	    FieldSizeTotal(" << recName << "_typeSpec.recordSpec);" << endl;
  outFile << "	rqs_pointer_t q = (rqs_pointer_t)new char[qlen];" << endl;
  outFile << "	void * buffer_end = Pack(q, src);" << endl;
  outFile << "	unsigned long buffer_len = (char *)buffer_end - (char *)q;"
    << endl <<
    "	rqs_elementId_t eltId;" << endl;
  outFile << "	rqs_unsignedInt64_t w;" << endl;
  outFile << "	ndr_uhyper_int_rep x = work;" << endl <<
    "	Assign(w, x);" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"rqs_Enqueue - Queue: %s - Type: %s - Buffer Length: %ul\\n\",(char*)(const char *)queue_name.FullName(),(char *)(const char *)yQtype.TypeName,buffer_len);" << endl;
  outFile << "	RqsStatusValue = rqs_Enqueue((rqs_serverHandle_t)Handle(),"
    << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName(),"
    << endl;
  outFile << "			    (char *)(const char *)yQtype.TypeName,"
    << endl <<
    "			    buffer_len," << endl <<
    "			    q," << endl <<
    "			    &w," << endl <<
    "			    &eltId);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    {" << endl;
  outFile << "		eid = (YQelementID)eltId;" << endl;
  outFile << "		ret = CDL_SUCCESS;" << endl;
  outFile << "	    }" << endl << "	else" << endl <<
    "	    {" << endl;
  outFile << "		ret = CheckError(20017, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl << "	    }" << endl;
  outFile << "	delete [] q;" << endl << "	return ret;" << endl;
  outFile << "    }" << endl <<  endl;

// Queue constructor
  outFile << recName << "_YQqueue::" << recName << "_YQqueue(" << endl;
  outFile << "	const String & ApplicationName," << endl <<
    "	const char* Server)" << endl <<
    "	: YQbase(ApplicationName)," << endl <<
    "	  yQtype(global_" << recName << "_YQtype)," << endl;
  outFile << "	  RqsBinding(Server)" << endl;
  outFile << "    {" << endl << "    }" << endl << endl;

// destructor for queue  
  outFile << recName << "_YQqueue::~" << recName << "_YQqueue(){}"
    << endl << endl;

// release exclusive access
  outFile << "CDLStatus " << recName << "_YQqueue::ReleaseExclusiveAccess()"
    << endl;
  outFile << "    {" << endl;
  outFile << "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"ReleaseExclusiveAccess Queue: %s\\n\", (char*)(const char*)queue_name.FullName());" << endl;
  outFile << "	RqsStatusValue = rqs_ReleaseExclusiveAccess(" << endl;
  outFile << "			    (rqs_serverHandle_t)Handle()," << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName());"
    << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS) return CDL_SUCCESS;"
    << endl;
  outFile << "	else return CheckError(20018, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

// queue::Requeue
  outFile << "CDLStatus " << recName << "_YQqueue::Requeue(" << endl;
  outFile << "	const YQelementID& eid," << endl;
  outFile << "	const " << recName << "& src," << endl <<
    "	const HyperInt& work)" << endl;
  outFile << "    {" << endl <<
    "	CDLStatus ret;" << endl <<
    "	YQname queue_name(aName, yQtype.TypeName);" << endl;
  outFile << "	unsigned long qlen =" << endl <<
    "	    FieldSizeTotal(" << recName << "_typeSpec.recordSpec);" << endl;
  outFile << "	rqs_pointer_t q = (rqs_pointer_t)new char[qlen];" << endl;
  outFile << "	Pack(q, src);" << endl <<
    "	rqs_unsignedInt64_t w;" << endl <<
    "	ndr_uhyper_int_rep x = work;" << endl <<
    "	Assign(w, x);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"RequeueAndModify Queue: %s - Type: %s - Queue Length: %ul\\n\"," << endl;
  outFile << "			  (char *)(const char*)queue_name.FullName(), (char*)(const char *)yQtype.TypeName, qlen);" << endl;
  outFile <<
    "	RqsStatusValue = rqs_RequeueAndModify((rqs_serverHandle_t)Handle(),"
    << endl;
  outFile <<
    "			    (char *)(const char *)queue_name.FullName(),"
    << endl;
  outFile << "			    (rqs_elementId_t*)&eid," << endl <<
    "			    &w," << endl ;
  outFile
    << "			    (char*)(const char *)yQtype.TypeName,"
    << endl;
  outFile << "			    qlen," << endl;
  outFile << "			    q);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "      		ret = CDL_SUCCESS;" << endl;
  outFile << "	else" << endl;
  outFile << "		ret = CheckError(20019, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "	delete [] q;" << endl <<
    "	return ret;" << endl;
  outFile << "    }" << endl <<  endl << endl;

// create queueset
  outFile << "CDLStatus " << recName << "_YQqueueset::CreateQueueset("
    << endl;
  outFile <<
    "				List<String> queueApplicationNames,"
    << endl <<
    "				List<unsigned int> queuePriorities)"
    << endl;
  outFile << "    {" << endl <<
    "	/* sanity check on lists */" << endl;
  outFile << "	ExistingTransaction tran;" << endl <<
    "	if (queueApplicationNames.length() !=" << endl;
  outFile << "	    queuePriorities.length())" << endl <<
    "		{" << endl <<
    "		    ThrowError(20020, CDL_LIST_COUNT_MISMATCH, tran, ErrorLevels::ErrorMsg);"
    << endl;
  outFile << "		}" << endl;
  outFile << "	CDLStatus ret;" << endl;
  outFile << "	const unsigned int itmcnt = queuePriorities.length();"
    << endl <<
    "	unsigned int loop = 0;" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl <<
    "	Const_listiter<String> qanames(queueApplicationNames);" << endl;
  outFile << "	Const_listiter<unsigned int> qpriorities(queuePriorities);"
    << endl;
  outFile <<
    "	rqs_queueSetMember_t * rqsm = new rqs_queueSetMember_t[itmcnt];"
    << endl;
  outFile << "	while(!qanames.at_end())" << endl <<
    "	    {" << endl;
  outFile << "		rqsm[loop].priority = *qpriorities.next();" << endl;
  outFile <<
    "		String qname = *qanames.next() + \"_\" + yQtype.TypeName;"
    << endl <<
    "		rqsm[loop].queue = new char[qname.length()+1];" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Adding Queue: %s to QueueSet: %s - Priority: %u for queue set Creation\\n\"," << endl;
  outFile << "			  (char *)(const char *)qname, (char*)(const char*)queueset_name.FullName(), rqsm[loop].priority);" << endl;
  outFile << "		qname.dump(rqsm[loop].queue);" << endl <<
    "		loop++;" << endl <<
    "	    }" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Creating Qset: %s - Item Count: %u\\n\", (char*)(const char*)queueset_name.FullName(), itmcnt);" << endl;
  outFile << "	RqsStatusValue = rqs_QSCreate(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (char*)(const char*)queueset_name.FullName(),"
    << endl;
  outFile << "			    itmcnt," << endl <<
    "			    rqsm);" << endl <<
    "	for(loop = 0; loop < itmcnt; loop++) delete [] rqsm[loop].queue;"
    << endl;
  outFile << "	delete [] rqsm;" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "		ret = CDL_SUCCESS;" << endl;
  outFile << "	else" << endl <<
    "	    ret = CheckError(20021, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "	return ret;" << endl;
  outFile << "    }" << endl << endl;

// queueset dequeue
  outFile << "CDLStatus " << recName << "_YQqueueset::Dequeue(" << endl;
  outFile << "	" << recName << "& dst," << endl;
  outFile << "	const DeleteOption dltopt," << endl <<
    "	YQelementID& eid," << endl <<
    "	const Boolean immediateReturn," << endl <<
    "	HyperInt& work)" << endl;
  outFile << "    {" << endl <<
    "	CDLStatus ret;" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	rqs_elementDescriptor_t *elementP;" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl <<
    "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"QSet Dequeue - Queset: %s - Element Disposition: %s\\n\"," << endl;
  outFile << "			  (char *)(const char *)queueset_name.FullName(), (dltopt == Delete) ? \"Delete\" : \"Orphan\");" << endl;
  outFile <<
    "      	RqsStatusValue = rqs_QSDequeue((rqs_serverHandle_t)Handle(),"
    << endl <<
    "			    (char *)(const char *)queueset_name.FullName(),"
    << endl <<
    "			    (dltopt == Delete ? rqs_deleteElement :" << endl <<
    "						rqs_orphanElement)," << endl <<
    "			    immediateReturn," << endl <<
    "			    &elementP);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl <<
    "	    {" << endl <<
    "		Unpack(elementP->value, dst);" << endl <<
    "		ndr_uhyper_int_rep x;" << endl <<
    "		Assign(x, elementP->work);" << endl <<
    "		work = x;" << endl;
  outFile << "		eid = (YQelementID)elementP->eltId;" << endl <<
    "		rqs_Free((char*)elementP);" << endl <<
    "		ret = CDL_SUCCESS;" << endl <<
    "      	    }" << endl;
  outFile << "	else" << endl <<
    "	    {" << endl <<
    "		ret = CheckError(20022, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "	    }" << endl <<
    "	return ret;" << endl <<
    "    }" << endl << endl;

// queueset DestroyQueueset
  outFile << "CDLStatus " << recName << "_YQqueueset::DestroyQueueset()"
    << endl <<
    "    {" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl <<
    "	ExistingTransaction tran;" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Destroying QSet: %s\\n\", (char*)(const char*)queueset_name.FullName());" << endl;
  outFile << "	Boundless();" << endl <<
    "	RqsStatusValue = rqs_QSDestroy((rqs_serverHandle_t)Handle(),"
    << endl <<
    "			    (char *)(const char *)queueset_name.FullName());"
    << endl <<
    "	if (RqsStatusValue == RQS_SUCCESS) return CDL_SUCCESS;" << endl;
  outFile << "	return CheckError(20023, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

// queueset insert queue
  outFile << "CDLStatus " << recName << "_YQqueueset::InsertQueue(" << endl;
  outFile << "	const String& q," << endl <<
    "	unsigned int p)" << endl <<
    "    {" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl <<
    "   YQname queue_name(q, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "  CDLReport.VReport(ErrorLevels::TraceMsg, \"Inserting Queue: %s in Qset: %s\\n\"," << endl;
  outFile << "			  (char*)(const char*)queue_name.FullName(), (char *)(const char *)queueset_name.FullName());" << endl;
  outFile <<
    "	RqsStatusValue = rqs_InsertQueue((rqs_serverHandle_t)Handle(),"
    << endl <<
    "			    (char *)(const char *)queueset_name.FullName(),"
    << endl <<
    "			    (char *)(const char *)queue_name.FullName()," << endl <<
    "			    p);" << endl <<
    "	return CheckError(20024, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "    }" << endl << endl;


  outFile << recName <<
"_YQqueueset::" <<
recName <<
"_YQqueueset(" << endl <<
"	const String & ApplicationName," << endl <<
"	const char* Server) : YQbase(ApplicationName)," << endl <<
"				 yQtype(global_" <<
recName <<
"_YQtype)," << endl <<
"				 RqsBinding(Server)" << endl <<
"    {" << endl <<
"" << endl <<
"    }" << endl <<
endl <<
recName <<
"_YQqueueset::~";
  outFile << recName <<
"_YQqueueset()" << endl <<
"    {" << endl <<
"" << endl <<
"    }" << endl <<
endl;

  outFile << "CDLStatus " << recName << "_YQqueueset::RemovePriorities("
    << endl <<
    "	List<unsigned int> priorities)" << endl <<
    "    {" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl <<
    "	Boundless();" << endl;
  outFile <<
    "   const unsigned int size = priorities.length();" << endl <<
    "   unsigned int *priorArr = new unsigned int[size];" << endl <<
    "   for(int j=0; j<size; j++)" << endl <<
    "      priorities.get(priorArr[j]);" << endl;
    outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Removing %u Priorities from Qset: %s\\n\", size, (char *)(const char*)queueset_name.FullName());" << endl;
    outFile << "	RqsStatusValue = rqs_RemovePriorities((rqs_serverHandle_t)Handle(),"
    << endl <<
    "			    (char *)(const char *)queueset_name.FullName(),"
    << endl <<
    "			    size," << endl <<
    "			    priorArr );"
    << endl;
  outFile << "	return CheckError(20025, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQqueueset::RemoveQueue(" << endl <<
    "	const String& queue)" << endl <<
    "    {" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);"<<endl <<
    "   YQname queue_name(queue, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Removing Queue: %s from Qset: %s\\n\"," << endl;
  outFile << "			  (char*)(const char *)queue_name.FullName(), (char*)(const char*)queueset_name.FullName());" << endl;
  outFile <<
    "	RqsStatusValue = rqs_RemoveQueue((rqs_serverHandle_t)Handle(),"
    << endl <<
    "			    (char *)(const char *)queueset_name.FullName(),"
    << endl;
  outFile << "			    (char*)(const char *)queue_name.FullName());" << endl;
  outFile <<
    "	return CheckError(20026, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName <<
    "_YQqueueset::SetInfiniteServiceLevels()" << endl;
  outFile << "    {" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl <<
    "	ExistingTransaction tran;" << endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"SetInfiniteServiceLevels on QSet: %s\\n\", (char*)(const char*)queueset_name.FullName());" << endl;
  outFile << "	Boundless();" << endl <<
    "	RqsStatusValue = rqs_SetInfiniteServiceLevels(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (char *)(const char *)queueset_name.FullName());"
    << endl;
  outFile << "	return CheckError(20027, RqsStatusValue, tran, ErrorLevels::InfoMsg);" <<
    endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQqueueset::SetQueuePriority("
    << endl;
  outFile << "	const String&queue," << endl <<
    "	const unsigned int priority)" << endl <<
    "    {" << endl <<
    "	YQname queueset_name(aName, yQtype.TypeName);" << endl;
  outFile << "	ExistingTransaction tran; " << endl;

  outFile << "	Boundless();" << endl;
  outFile <<  "	YQname queue_name(queue, yQtype.TypeName);" << endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"Setting Priority to %u on Queue: %s in Qset: %s\\n\"," << endl;
  outFile << "			  priority, (char*)(const char *)queue_name.FullName(), (char*)(const char*)queueset_name.FullName());" << endl;

  outFile <<   "    RqsStatusValue = rqs_SetQueuePriority(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (char *)(const char *)queueset_name.FullName(),"
    << endl <<
    "			    (char *)(const char *)queue_name.FullName()," << endl <<
    "			    priority);" << endl <<
    "	return CheckError(20028, RqsStatusValue, tran, ErrorLevels::InfoMsg);" << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQqueueset::SetServiceLevels("
    << endl;
  outFile <<
    "				List<unsigned int> serviceLevels,"
    << endl <<
    "				List<unsigned int> priorityArray)"
    << endl <<
    "    {" << endl;
  outFile << "	ExistingTransaction tran;" << endl;
  outFile << "	/* sanity check */" << endl <<
    "	const unsigned int itmcnt = priorityArray.length();" << endl <<
    "	if (serviceLevels.length() != priorityArray.length())" << endl;
  outFile <<
    "\t\treturn ThrowError(20029, CDL_LIST_COUNT_MISMATCH, tran, ErrorLevels::ErrorMsg);"
    << endl;
  outFile <<
    "	rqs_serviceLevelDefn_t *sldt = new rqs_serviceLevelDefn_t[itmcnt];"
    << endl;
  outFile << "	unsigned int i;" << endl <<
    "	for (i = 0; i < itmcnt; i++)" << endl <<
    "	    {" << endl <<
    "		priorityArray.get(sldt[i].priority);" << endl <<
    "		serviceLevels.get(sldt[i].serviceLevel);" << endl <<
    "	    }" << endl;
  outFile << "	YQname queueset_name(aName, yQtype.TypeName);" << endl <<
    "	Boundless();" << endl;
  outFile << "	CDLReport.VReport(ErrorLevels::TraceMsg, \"rqs_SetServiceLevels - Qset: %s\\n\", (char*)(const char*)queueset_name.FullName());" << endl;
  outFile <<   "	RqsStatusValue = rqs_SetServiceLevels((rqs_serverHandle_t)Handle(),"
    << endl <<
    "			    (char *)(const char *)queueset_name.FullName(),"
    << endl <<
    "			    itmcnt," << endl <<
    "			    sldt);" << endl;
  outFile << "	delete [] sldt;" << endl;
  outFile << "	return CheckError(20030, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << "" << endl;

  outFile << "CDLStatus " << recName << "_YQtype::Create()" << endl;
  outFile << "    {" << endl;
  outFile << "	ExistingTransaction tran; " << endl;
  outFile << "	Boundless();" << endl <<
    "	RqsStatusValue = rqs_ElementTypeCreate(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (char*)(const char*)TypeName," << endl <<
    "			    ( rqs_typeSpec_t*)&TypeSpecification);" << endl;
  outFile << "	return CheckError(20031, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQtype::Retrieve(" << endl;
  outFile << "	    rqs_typeSpec_t & dst)" << endl;
  outFile << "    {" << endl;
  outFile << "	rqs_typeSpec_t *typeSpecP;" << endl;
  outFile << "	ExistingTransaction tran; " << endl;
  outFile << "	Boundless();" << endl <<
    "	RqsStatusValue = rqs_ElementTypeRetrieve(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (char*)(const char*)TypeName," << endl <<
    "			    &typeSpecP);" << endl;
  outFile << "		dst = * typeSpecP;" << endl <<
    "		rqs_Free((char*)typeSpecP);" << endl;
  outFile << "	if (RqsStatusValue == RQS_SUCCESS)" << endl;
  outFile << "	    dst = * typeSpecP;" << endl ;
  outFile << "	rqs_Free((char*)typeSpecP);" << endl;
  outFile << "	return CheckError(20031, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << "CDLStatus " << recName << "_YQtype::Destroy()"
    << endl;
  outFile << "    {" << endl;
  outFile << "	ExistingTransaction tran; " << endl;
  outFile << "	Boundless();" << endl;
  outFile << "	RqsStatusValue = rqs_ElementTypeDestroy(" << endl <<
    "			    (rqs_serverHandle_t)Handle()," << endl <<
    "			    (char*)(const char*)TypeName);" << endl;
  outFile << "	return CheckError(20032, RqsStatusValue, tran, ErrorLevels::InfoMsg);"
    << endl;
  outFile << "    }" << endl << endl;

  outFile << recName <<
"_YQtype::" <<
recName <<
"_YQtype(const char* Server)" << endl <<
"    : YQbase(\"\")," << endl <<
"      TypeName(\"" <<
recName <<
"\")," << endl <<
"      TypeSpecification(" <<
recName <<
"_typeSpec)," << endl <<
"      RqsBinding(Server){}" << endl <<
endl <<
recName <<
"_YQtype::~" <<
recName <<
"_YQtype()" << endl <<
"    {" << endl <<
"" << endl <<
"    }" << endl <<
"" << endl <<
"/* ostream friend functions */" << endl <<
"ostream& operator <<(ostream&o,const " <<
recName <<
"_YQcursor&src )" << endl <<
"    {" << endl <<
"	o << (YQbase)src << endl;" << endl <<
"	o << src.cursor << endl;" << endl <<
"	o << src.yQtype << endl;" << endl <<
"	return o;" << endl <<
"    }" << endl <<
"ostream& operator <<(ostream&o,const " <<
recName <<
"_YQelement&src )" << endl <<
"    {" << endl <<
"	o << (YQbase)src << endl;" << endl <<
"	o << src.yQtype << endl;" << endl <<
"	return o;" << endl <<
"    }" << endl <<
"ostream& operator <<(ostream&o,const " <<
recName <<
"_YQkey&src )" << endl <<
"    {" << endl;
  outFile << "	o << (YQbase)src << endl;" << endl <<
"	o << src.yQtype << endl;" << endl <<
"	return o;" << endl <<
"    }" << endl <<
"ostream& operator <<(ostream&o,const " <<
recName <<
"_YQqueue&src )" << endl <<
"    {" << endl <<
"	o << (YQbase)src << endl;" << endl <<
"	o << src.yQtype << endl;" << endl <<
"	return o;" << endl <<
"    }" << endl <<
"ostream& operator <<(ostream&o,const " <<
recName <<
"_YQqueueset&src )" << endl <<
"    {" << endl <<
"	o << (YQbase)src << endl;" << endl <<
"	o << src.yQtype << endl;" << endl <<
"	return o;" << endl <<
"    }" << endl <<
"ostream & operator<<(ostream & o,const " <<
recName <<
"_YQtype & src)" << endl <<
"    {" << endl <<
"	o << src.TypeName << endl;" << endl <<
"	o << src.TypeSpecification << endl;" << endl <<
"	return o;" << endl <<
"    }" << endl <<
    /* now do the Pack and Unpack routines for this specific type */
    "/*" << endl <<
"   This code packs and unpack data between the buffers that rqs uses and the" << endl <<
"   yaddl genereated class " <<
    recName <<
".  The return value is the value of the" << endl <<
"   cursor (pointer) that is used to step though the output buffer.  The return" << endl <<
"   value should be one past the end of the input buffer address." << endl;
  outFile << "*/" << endl <<
"void * " <<
    recName << 
"_YQkey::Pack(void * buffer," << endl <<
"				const " <<
    recName <<
" & src," << endl <<
"				const Index idx)" << endl <<
"    {	/* pack a buffer with key data.  no attempt is made to check the range" << endl <<
"	 * on the index */" << endl <<
"	void * p = buffer;" << endl <<
"       unsigned long start_addr = (unsigned long)p;" << endl << 
"	const rqs_keySpec_t & rks =" << endl <<
"	    yQtype.TypeSpecification.keySpecArray[idx];" << endl <<
"	const rqs_recordSpec_t & rs = yQtype.TypeSpecification.recordSpec;" << endl <<
"	int i;" << endl;
  outFile << "	" << recName << "Base& base = *(" << recName << "Base*)(" <<
	recName << ")src;"
   	<< endl;
outFile <<
"	for (i = 0; i < rks.numFields; i++)" << endl <<
"	    {	/* for each field in the key */" << endl <<
"		const char * const fieldName = rks.fieldNameArray[i];" << endl <<
"		unsigned short j;" << endl <<
"		for (j = 0; j < rs.numFields ; j++)" << endl <<
"		    {	/* lookup the key spec */" << endl <<
"			if (!strcmp(fieldName," << endl <<
"				rs.fieldSpecArray[j].fieldName))" << endl <<
"			    {	/* pack the field and leave inner loop */" << endl <<
"				switch(j)" << endl <<
"				    {" << endl <<
"					default:" << endl <<
"					    cerr << \"Catastrophic screwup in configuration processing\" << endl;" << endl <<
"					    cerr << \"Field index:\" << j <<" << endl <<
"					    \" is out of range in type\" <<" << endl <<
"					    yQtype.TypeName << endl;" << endl;
  outFile << "					    abort();" << endl <<
"					    break;" << endl;
    /* for each field generate the case which does the packing */
    inc = 0;
    for (fieldIter.reset(); fieldIter.next(curNode);inc++)
	{
	    outFile << "					case " <<
	      inc << ":" << endl;
	    if (curNode.Field()->Type() == CDLField::STRING)
	      outFile << 
	      "					    p = ::PackStringKey(p, base.";
	    else if (curNode.Field()->Type() == CDLField::ENUM)
	      outFile << 
	      "					    p = ::Pack(p, (unsigned short)base.";
	    else
	      outFile << 
	      "					    p = ::Pack(p, base.";
	    outFile << curNode.Name();
               // if it's a varrying length field include length param
	    if (curNode.Field()->Type() == CDLField::VARBYTES ||
		curNode.Field()->Type() == CDLField::BYTES ||
                curNode.Field()->Type() == CDLField::STRING ||
	        curNode.Field()->Type() == CDLField::CHARRAY)
	      outFile << ", " <<recName << "_fieldSpecArray[" << inc <<
	         "].fieldSize";
	    outFile << ");" << endl <<
"					    break;" << endl;
	}   /* for (fieldIter.reset(); fieldIter.next(curNode);) */
	outFile <<
"				    } /* switch */" << endl <<
"				break;" << endl <<
"			    }	/* pack the field and leave inner loop */" 
	<< endl <<
"		    }" << endl <<
"	    }" << endl <<
"       unsigned long end_addr = (unsigned long)p;" << endl <<
"       buff_length = end_addr - start_addr;" << endl << 
"	return p;" << endl <<
"    }" << endl <<
"void * Pack(void * buffer, const " <<
recName <<
" & src)" << endl <<
"    {" << endl <<
"	void * p = buffer;" << endl;
   outFile << "	" << recName << "Base& base = *(" << recName << "Base*)(" <<
	recName << ")src;"
   	<< endl;
    inc = 0;
    for (fieldIter.reset(); fieldIter.next(curNode);inc++)
	{
	  if (curNode.Field()->Type() == CDLField::STRING)
	    {
	      outFile << "	p = PackString(p, base." << curNode.Name();
	    }
	  else if (curNode.Field()->Type() == CDLField::ENUM)
	    {
	      outFile << "      p = Pack(p, (unsigned short)base.";
	      outFile << curNode.Name();
	    }
	  else
	    {
	      outFile << "	p = Pack(p, base." << curNode.Name() ;
	    }
	/* if it's a fixed length field include length param */
	  if ( curNode.Field()->Type() == CDLField::BYTES ||
	       curNode.Field()->Type() == CDLField::STRING ||
	       curNode.Field()->Type() == CDLField::CHARRAY)
	    outFile << ", " <<recName << "_fieldSpecArray[" << inc <<
	    "].fieldSize";

	    /* if it is var length give the actual length param */
	  if (curNode.Field()->Type() == CDLField::VARBYTES)
	    outFile << ", base." << curNode.Name() << CDLByteArrayLengthSuffix;
	/* if it's a varbytes field then include flag for packing */
	  if (curNode.Field()->Type() == CDLField::VARBYTES)
	    outFile << ", 1";
	  outFile << ");" << endl;
	}   /* for (fieldIter.reset(); fieldIter.next(curNode);) */
    outFile << "	return p;" << endl <<
"    }" << endl <<
"const void * const Unpack(const void * const buffer, " <<
recName <<
" & src)" << endl <<
"    {" << endl <<
"	" <<
recName <<
"Base & base = *(" <<
recName <<
"Base*)src;" << endl <<
"	const void * p = buffer;" << endl;
  inc = 0;
  for (fieldIter.reset(); fieldIter.next(curNode);inc++)
    {
      if (curNode.Field()->Type() == CDLField::ENUM)
	{
	  outFile << "{" << endl;
	  outFile << "unsigned short tmpShort = (unsigned short)base.";
	  outFile << curNode.Name() << ";" << endl;
	}
      outFile << "	p = Unpack(p, ";
	/* if it's a varrying length field include length param */
      if (curNode.Field()->Type() == CDLField::VARBYTES ||
	  curNode.Field()->Type() == CDLField::BYTES ||
	  curNode.Field()->Type() == CDLField::STRING ||
	  curNode.Field()->Type() == CDLField::CHARRAY)
	{
	  outFile << "(char*)base." << curNode.Name() << ", ";
	  if (curNode.Field()->Type() == CDLField::VARBYTES ||
	      curNode.Field()->Type() == CDLField::BYTES)
	    {
	      outFile << "&base." << curNode.Name();
	      outFile << CDLByteArrayLengthSuffix << ", ";
	    }
	  outFile << recName << "_fieldSpecArray[" << inc <<
	    "].fieldSize";
	}
      else if (curNode.Field()->Type() == CDLField::ENUM)
	{
	  outFile << "tmpShort";
	}
      else
	{
	  outFile << "base." << curNode.Name() ;
	}

	/* if it's a varbytes field then include flag indicating length word */
      if (curNode.Field()->Type() == CDLField::VARBYTES)
	outFile << ", 1";

      outFile << ");" << endl;
      if (curNode.Field()->Type() == CDLField::ENUM)
	{
	  outFile << "}" << endl;
	}
    }   /* for (fieldIter.reset(); fieldIter.next(curNode);) */
    outFile << "	return p;" << endl <<
"    }" << endl;
    return true;
}

/*
 * GenHeader
 *    Generate the header file information for this class, which consists
 * of a definition of the main class we're trying to generate. 
 */

Boolean CDLRqsGenerator::GenHeader(CDLRecord& rec, CDLFileDef& file)

{
  int inc;

  String recName;
  String fileName;  String className;
  String tmpName;

  recName = rec.Name();
  fileName = file.Name();
  className = "RQS";
  className += fileName;

    inclFile << "void * Pack(void * buffer, const " << recName <<
    " & src);" << endl <<
    "const void * const Unpack(const void * const buffer, " <<
    recName << " & src);" << endl <<
    "class " << recName << 
"_YQtype : public YQbase, public RqsBinding" << endl <<
"    { /* manage the relationship between the string name of a generated type" << endl <<
"       * and the rqs type specification */" << endl <<
"public:" << endl <<
"	CDLStatus Create() ;" << endl <<
"	CDLStatus Retrieve(rqs_typeSpec_t & dst) ;" << endl;
  inclFile << "	CDLStatus Destroy() ;" << endl <<
"	" << recName << 
"_YQtype(const char* srvr = DefaultServer);" << endl <<
"	virtual ~" << recName << 
"_YQtype();" << endl <<
"	friend ostream & operator<<(ostream & o," << endl <<
"				    const " << recName << 
"_YQtype &src);" << endl <<
"protected:" << endl <<
"	const String TypeName;" << endl <<
"	const rqs_typeSpec_t & TypeSpecification;" << endl <<
"	/* related classes that need to access the hidden state of this" << endl <<
"	 * class */" << endl <<
"	friend class " << recName << 
"_YQqueue;" << endl <<
"	friend class " << recName << 
"_YQqueueset;" << endl <<
"	friend class " << recName ;
  inclFile << "_YQcursor;" << endl <<
"	friend class " << recName << 
"_YQkey;" << endl <<
"	friend class " << recName << 
"_YQelement;" << endl <<
"    };" << endl <<
"class " << recName << 
"_YQqueue : public YQbase," << endl <<
"			     public RqsBinding" << endl <<
"    {	/* provide operations to access elements via a queue and administer" << endl <<
"	 * the queue */" << endl <<
"public:" << endl <<
"	friend ostream & operator<<(ostream & o," << endl <<
"				    const " << recName << 
"_YQqueue &src);" << endl <<
"	" << recName << 
"_YQqueue(const String & ApplicationName," << endl <<
"			     const char* srvr = DefaultServer);" << endl <<
"	virtual ~" << recName << 
"_YQqueue();" << endl <<
"	/* adminisitrative functions */" << endl <<
"	CDLStatus CreateQueue(const Boolean workAccumulationFlag = true) ;" << endl <<
"	CDLStatus DestroyQueue() ;" << endl <<
"	CDLStatus DeleteAllElements() ;" << endl <<
"	CDLStatus AcquireExclusiveAccess() ;" << endl <<
"	CDLStatus ReleaseExclusiveAccess() ;" << endl <<
"	CDLStatus EnableTransactionalAccess() ;" << endl <<
"       CDLStatus QStat(unsigned long& Qlength, HyperInt& work) ;" <<endl <<

"	/* access to data */" << endl <<
"	CDLStatus Enqueue(/* Given a yaddl object stick it on a queue and return" << endl <<
"			* it's element id */" << endl <<
"		       YQelementID & eid," << endl <<
"		       const " << recName << 
" & src," << endl;
  inclFile << "		       const HyperInt & work" << endl <<
"		       ) ;" << endl <<
"	CDLStatus Dequeue(/* Dequeue from a queue.  Upon succesful completion" << endl <<
"			* delete from rqs or leave orphaned depending on" << endl <<
"			* setting of the dltopt parameter.  If the" << endl <<
"			* immediateReturn flag is true and there is nothing to" << endl <<
"			* dequeue then return without blocking. */" << endl <<
"		       " << recName << 
" & dst," << endl <<
"		       const DeleteOption dltopt," << endl <<
"		       YQelementID & eid," << endl <<
"		       const Boolean immediateReturn," << endl <<
"		       HyperInt & work" << endl <<
"		       ) ;" << endl <<
"	CDLStatus Requeue(/* Take a previously orphaned item and place it on a" << endl <<
"			* new queue.  The value of the item is re-written as I" << endl <<
"			* cannot tell if it has been modified */" << endl <<
"		       const YQelementID & eid," << endl <<
"		       const " << recName << 
" & src," << endl <<
"		       const HyperInt & work" << endl <<
"		       ) ;" << endl <<
"protected:" << endl <<
"	const " << recName << 
"_YQtype & yQtype;" << endl <<
"    };" << endl <<
"class " << recName << 
"_YQqueueset : public YQbase," << endl <<
"				public RqsBinding" << endl <<
"    {	/* provide operations to access elements via a queueset and administer" << endl <<
"	 * the queueset */" << endl;
  inclFile << "public:" << endl <<
"	friend ostream & operator<<(ostream & o," << endl <<
"				    const " << recName << 
"_YQqueueset &src);" << endl <<
"	" << recName << 
"_YQqueueset(const String & ApplicationName," << endl <<
"			        const char* srvr = DefaultServer);" << endl <<
"	virtual ~" << recName << 
"_YQqueueset();" << endl <<
"	/* Administrative stuff */" << endl <<
"	CDLStatus CreateQueueset(List<String> queueApplicationNames," << endl <<
"			      List<unsigned int> queuePriorities) ;" << endl <<
"	CDLStatus DestroyQueueset() ;" << endl <<
"	CDLStatus InsertQueue(const String & QueueApplicationName," << endl <<
"			   unsigned int	Priority);" << endl <<
"	CDLStatus RemoveQueue(const String & QueueApplicationName) ;" << endl <<
"	CDLStatus RemovePriorities(List<unsigned int> priorities);" << endl <<
"	CDLStatus SetServiceLevels(List<unsigned int> serviceLevels," << endl <<
"				List<unsigned int> priorityArray) ;" << endl <<
"	CDLStatus SetInfiniteServiceLevels();" << endl <<
"	CDLStatus SetQueuePriority(const String & queue," << endl <<
"				const unsigned int priority) ;" << endl <<
"	/* access to data */" << endl <<
"	CDLStatus Dequeue(/* Dequeue from a queue.  Upon succesful completion" << endl <<
"			* delete from rqs or leave orphaned depending on" << endl <<
"			* setting of the dltopt parameter.  If the" << endl <<
"			* immediateReturn flag is true and there is nothing to" << endl <<
"			* dequeue then return without blocking. */" << endl <<
"		       " << recName << 
" & dst," << endl;
  inclFile << "		       const DeleteOption dltopt," << endl <<
"		       YQelementID & eid," << endl <<
"		       const Boolean immediateReturn," << endl <<
"		       HyperInt & work" << endl <<
"		       ) ;" << endl <<
"protected:" << endl <<
"	const " << recName << 
"_YQtype & yQtype;" << endl <<
"    };" << endl <<
"class " << recName << 
"_YQcursor : public YQbase," << endl <<
"			      public RqsBinding" << endl <<
"    {	/* provide operations to access elements via a cursor and administer" << endl <<
"	 * the cursor */" << endl <<
"public:" << endl <<
"	friend ostream & operator<<(ostream & o," << endl <<
"				    const " << recName << 
"_YQcursor &src);" << endl <<
"	" << recName << 
"_YQcursor(const String & ApplicationName," << endl <<
"			      const CursorPolicy cp," << endl <<
"			      const ElementLockMode elm," << endl <<
"			      const char* srvr = DefaultServer);" << endl <<
"	" << recName << 
"_YQcursor(const " << recName << 
"_YQcursor & src);" << endl <<
"	virtual ~" << recName << 
"_YQcursor();" << endl <<
"	/* access to data */" << endl <<
"	CDLStatus Next(	" << recName << 
" & dst," << endl <<
"			HyperInt & work);" << endl <<
"	CDLStatus Next(	const unsigned int RequestCount," << endl <<
"			List<" << recName << 
"> & data," << endl;
inclFile << "			List<HyperInt> & work);" << endl <<
"	CDLStatus Reset();" << endl <<
"	CDLStatus DropLock();" << endl <<
"protected:" << endl <<
"	const " << recName << 
"_YQtype & yQtype;" << endl <<
"	rqs_cursor_t cursor;" << endl <<
"    };" << endl <<
"class " << recName << 
"_YQelement : public YQbase," << endl <<
"			       public RqsBinding" << endl <<
"    {	/* provide operations to access elements via a element and administer" << endl <<
"	 * the element */" << endl <<
"public:" << endl <<
"	friend ostream & operator<<(ostream & o," << endl <<
"				    const " << recName << 
"_YQelement &src);" << endl <<
"	" << recName << 
"_YQelement(const String & ApplicationName," << endl <<
"			       const " << recName << 
"_YQtype & yqt," << endl <<
"			       const char* srvr = DefaultServer);" << endl <<
"	virtual ~" << recName << 
"_YQelement();" << endl <<
"	/* access to data */" << endl <<
"	CDLStatus Read(	const YQelementID & eid," << endl <<
"			const ElementLockMode elm," << endl <<
"			const Boolean RetainLock," << endl <<
"			" << recName << 
" & dst) ;" << endl <<
"	CDLStatus Modify(	const YQelementID & eid," << endl <<
"			const " << recName << 
" & src) ;" << endl <<
"	CDLStatus Delete(	const YQelementID & eid," << endl <<
"			const DeleteOption delopt," << endl;
  inclFile << "			" << recName << 
" & dst) ;" << endl <<
"	CDLStatus Unlock(	const YQelementID & eid) ;" << endl <<
"protected:" << endl <<
"	const " << recName << 
"_YQtype & yQtype;" << endl <<
"    };" << endl <<"class " << recName << "_YQkeyIndex" << endl << "{" <<endl <<
"public:" << endl << 
"	enum Index { /*	Index is alwasy zero based montonically increasing by"
  << endl <<
"		      *	one so that they match indexes of rqs data structure" 
 << endl <<
"		      *	as an aid to packing and unpacking */" << endl;
    for (inc = 0; inc < file.NumIndices(); inc++)
	{
	    inclFile << "		     " << 
		file.Index(inc)->Name() << 
		" = " << 
		inc << 
		"," <<
		endl;
	}
    inclFile << 
"		     KeyCount};" << endl << "};" << endl << endl <<

"class " << recName << "_YQkey : public " << recName << "_YQkeyIndex, " << 
"public YQbase," << endl <<
"			   public RqsBinding" << endl <<
"    {	/* provide operations to access elements via a key and administer" << endl <<
"	 * the key */" << endl <<
"public:" << endl <<

"	friend ostream & operator<<(ostream & o," << endl <<
"				    const " << recName << "_YQkey &src);" << endl <<
"	" << recName << "_YQkey(" << endl <<
"			   const char* srvr = DefaultServer);" << endl <<
"	virtual ~" << recName << "_YQkey();" << endl <<
"	/* access to data */" << endl <<
"	CDLStatus Read(	const Index," << endl;
  inclFile << "			const " << recName << " & keyValue," << endl <<
"			const ElementLockMode elm," << endl <<
"			const unsigned int maxReturnCount," << endl <<
"			List<YQelementID> & ids," << endl <<
"			List<" << recName << "> & values, " << endl <<
"                       char* QueueName = \"\");" << endl <<  
"	CDLStatus Read(	const List<Index>," << endl <<
"			const List<" << recName << "> & keyValues," << endl <<
"			const ElementLockMode elm," << endl <<
"			const unsigned int maxReturnCount," << endl <<
"			List<YQelementID> & ids," << endl <<
"			List<" << recName << "> & values) ;" << endl <<
"protected:" << endl <<
"       unsigned long buff_length;" << endl <<
"	const " << recName << "_YQtype & yQtype;" << endl <<
"	void * Pack(void *, const " << recName << " &, const Index);" << endl <<
"    };" << endl;


  return true;
}

