/*
 * CDLSfsGenerator:
 *
 *    The classes generated by this generator pass most of the work up to 
 * the SFSFileBase class. Most of the work of the class this generates
 * is concerned with translating the objects generated by CDL into the
 * appropriate types for use by SFS.
 *
 * cdlsfsgenerator.C,v 1.18 1994/04/07 19:26:18 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993-1994 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <dce/idlbase.h>
#include <cdl/cdlerrors.H>
#include <cdlsfsgenerator.H>
#include <cdlclassgenerator.H>
#include <cdlfield.H>
#include <cdlindexdef.H>
#include <cdlindexfielddef.H>
#include <cdlsuffixes.H>
#include <unistd.h>

#ifdef _AIX
#include <List.c>

#pragma define (Const_listiter<FieldNode>)
#pragma define (List<FieldNode>)
#pragma define (Listiter<FieldNode>)
#pragma define (lnnk_ATTLC<FieldNode>)
#endif

/* 
 * CDLSfsGenerator:
 *    This is the main class.
 */

CDLSfsGenerator::CDLSfsGenerator(const ArgList& args)
   : CDLDataGenerator(args)
{
  mainName = "";
  inclName = "";
}

CDLSfsGenerator::~CDLSfsGenerator()

{
}

/* 
 * Start
 *    Create the output files and write there whatever we need to put
 * at the beginning of every file, like #include statements.
 */

Boolean CDLSfsGenerator::Start(const String& fileName)

{
  String symbolName;
  String baseName;
  
  MakeFileName(fileName, mainName, "_cdl_sfs.C");
  MakeFileName(fileName, inclName, "_cdl_sfs.H");
  outFile.open(mainName);
  if (outFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1102, (const char*)mainName);
      return false;
    }
  inclFile.open(inclName);
  if (inclFile.fail())
    {
      CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1102, (const char*)inclName);
      return false;
    }
  else
    {
      MakeFileName(fileName, symbolName, "_CDL_SFS_H_INCLUDED");
      MakeFileName(fileName, baseName, "_cdl_file.H");
      inclFile << "/*" << endl;
      inclFile << " * Class generated by Citi-DDL:" << endl;
      inclFile << " * cdlsfsgenerator.C,v 1.18 1994/04/07 19:26:18 gjb Exp" << endl;
      inclFile << " */\n" << endl;
      inclFile << "#ifndef " << symbolName << endl;
      inclFile << "#define " << symbolName << "\n" << endl;
      inclFile << "#include <" << baseName << ">" << endl;
      inclFile << "extern \"C\" {" << endl;
      inclFile << "#include <sfs/sfs.h>" << endl;
      inclFile << "}" << endl;
      inclFile << "#include <cdl/sfsfilebase.H>\n" << endl;
      
      outFile << "/*" << endl;
      outFile << " * Class generated by Citi-DDL:" << endl;
      outFile << " * cdlsfsgenerator.C,v 1.18 1994/04/07 19:26:18 gjb Exp" << endl;
      outFile << " */\n" << endl;
      outFile << "#include <" << inclName << ">" << endl;
      outFile << "#include <cdl/cdldefs.H>" << endl;
      outFile << "#include <cdl/sfsbases.H>\n" << endl;

      return true;
    }
}

/*
 * End
 *    Finish up the output files and close them.
 */ 

Boolean CDLSfsGenerator::End()

{
  inclFile << "#endif" << endl;
  inclFile << flush;
  inclFile.close();
  outFile << flush;
  outFile.close();
  return true;
}

/* 
 * DeleteFiles
 *    Close and remove the output files. This is used when parsing fails
 * and files should be deleted.
 */

void CDLSfsGenerator::DeleteFiles()

{
  inclFile.close();
  outFile.close();
  unlink(inclName);
  unlink(mainName);
}

/*
 * GetSize
 *    This figures out how much space a particular field type will use
 * in SFS, and thus the value for the "dataSize" and "bufferSize" fields
 * of the various SFS structures. 
 *    The sizes of various types are guaranteed in SFS to be a specific 
 * number of bytes. Hence, we hardcode those sizes here. However,
 * "enum" isn't an SFS type, so we cast it to a short, since enums are
 * declared in the IDL file, and IDL won't allow values over 65,536.
 */

int CDLSfsGenerator::GetSize(CDLField& field) const

{
  int retVal;
  switch (field.Type())
    {
    case CDLField::REAL:
      retVal = 8;
      break;
    case CDLField::HYPER:
      retVal = 8;
      break;
    case CDLField::LONG:
      retVal = 4;
      break;
    case CDLField::SHORT:
    case CDLField::CHAR:
    case CDLField::ENUM:
      retVal = 2;
      break;
    case CDLField::BYTES:
      retVal = field.Length();
      break;
    case CDLField::VARBYTES:
      retVal = field.Length() + 4;
      break;
    case CDLField::STRING:
      retVal = field.Length() + 1;
      break;
    case CDLField::CHARRAY:
      retVal = field.Length();
      break;
    case CDLField::RECORD:
      retVal = -1;
      break;
    }
  return retVal;
}

/*
 * GetMaxSize
 *    Do the same, but for the fields in a particular index only.
 */

int CDLSfsGenerator::GetMaxIndexSize(CDLIndexDef* index, 
				     List<FieldNode>& fields) const

{
  int inc;
  String fieldName;
  CDLIndexFieldDef* curField;
  FieldNode testNode;
  FieldNode curNode;
  Listiter<FieldNode> fieldIter(fields);
  int retVal = 0;

  for (inc = 0; inc < index->NumFields(); inc++)
    {
      curField = index->Field(inc);
      testNode.VariableName(curField->Name());
      fieldIter.reset();
      if (!fieldIter.find_next(testNode))
	return 0;
      else
	{
	  fieldIter.next(curNode);
	  retVal += GetSize(*(curNode.Field()));
	}
    }
  return retVal;
}

/*
 * BuildFieldList
 *    Build a list of all the files in a record, and expand all the nested
 * records as well. The resulting field names, stored in FieldNode objects,
 * name each record from the top level separated by '.', like this: 
 * "record1.record2.field".
 *    This function uses two internal lists because SFS requires that variable-
 * length fields be placed after fixed length fields. So, when building the list
 * we build two lists, then append them to get the final list of fields.
 */

Boolean CDLSfsGenerator::BuildFieldList(CDLRecord& rec, CDLFileDef& file,
					List<FieldNode>& out)

{
  List<FieldNode> varFields;
  List<FieldNode> nonVarFields;

  CDLReport.Report(ErrorLevels::TraceMsg,
		   "Recursively expanding list of fields...\n");
  ExpandOneRecord(&rec, file, "", varFields, nonVarFields);
  out = nonVarFields;
  out.put(varFields);
  CDLReport.Report(ErrorLevels::TraceMsg,
		   "Done with recursive field list.\n");
  return true;
}

Boolean CDLSfsGenerator::ExpandOneRecord(CDLRecord* rec, CDLFileDef& file,
					 const String& name,
					 List<FieldNode>& varFields,
					 List<FieldNode>& nonVarFields)
{
  CDLField* curField;
  CDLField* curLengthField;
  FieldNode* newField;
  FieldNode* newLengthField;
  String specialName;
  int inc;
  int arrayPos;
  int realArrayPos;
  Boolean isArray;
  String thisFieldName;
  String varName;
  String lengthName;

  if (file.Type() == SEQUENTIAL)
    specialName = file.PrimaryIndex()->Field(0)->Name();
  else
    specialName = "";
  for (inc = 0; inc < rec->NumFields(); inc++)
    {
      curField = rec->Field(inc);
      isArray = (curField->ArraySize() > 1);
      if ((file.Type() != SEQUENTIAL) ||
	  (curField->Name() != specialName))
	{
	  for (arrayPos = 0; arrayPos < curField->ArraySize(); arrayPos++)
	    {
	      if (isArray)
		{
		  realArrayPos = arrayPos;
		}
	      else
		{
		  realArrayPos = -1;
		}
	      GenWholeFieldName('.', curField->Name(), name, 32, 
				thisFieldName, realArrayPos);
	      GenWholeFieldName('.', curField->Name(), name, 0,
				varName, realArrayPos);
	      if (IsRecord(curField->Type()))
		{
		  thisFieldName += ".";
		  ExpandOneRecord(curField->Record(), file, varName,
				  varFields, nonVarFields);
		}
	      else
		{
		  CDLReport.VReport(ErrorLevels::TraceMsg,
				    "SFS field name: %s var name: %s\n",
				    (const char*)thisFieldName,
				    (const char*)varName);
		  newField = new FieldNode(thisFieldName, curField);
		  newField->VariableName(varName);
		  if (curField->Type() == CDLField::BYTES)
		    {
		      lengthName = curField->Name() + CDLByteArrayLengthSuffix;
		      curLengthField = new CDLField(CDLField::LONG, 
						    CDLField::UNSIGNED, 
						    lengthName, 1);
		      GenWholeFieldName('.', lengthName, name, 32,
					thisFieldName, realArrayPos);
		      GenWholeFieldName('.', lengthName, name, 0,
					varName, realArrayPos);
		      CDLReport.VReport(ErrorLevels::TraceMsg,
					"SFS field name: %s var name: %s\n",
					(const char*)thisFieldName,
					(const char*)varName);
		      newLengthField = new FieldNode(thisFieldName, 
						     curLengthField);
 		      newLengthField->VariableName(varName);
		      nonVarFields.put(*newLengthField);
		      newField->LengthFieldName(varName);
		      delete curLengthField;
		      delete newLengthField;
		    }
		  else if (curField->Type() == CDLField::VARBYTES)
		    {
		      // This should set the length field to the IDL field name
		      lengthName = curField->Name() + CDLByteArrayLengthSuffix;
		      GenWholeFieldName('.', lengthName, name, 0,
					varName, realArrayPos);
		      newField->LengthFieldName(varName);
		    }
		  if (curField->Type() == CDLField::VARBYTES)
		    varFields.put(*newField);
		  else
		    nonVarFields.put(*newField);
		  delete newField;
		}
	    }
	}
    }
  return true;
}

/* 
 * Generate
 *    Generate all the files by calling the other generation functions 
 * in turn.
 */

Boolean CDLSfsGenerator::Generate(CDLRecord& rec, CDLFileDef& file)

{
  Boolean retVal;
  retVal = CheckStructure(rec, file);
  if (retVal)
    retVal = GenSubclasses(rec, file);
  if (retVal)
    retVal = GenHeader(rec, file);
  if (retVal)
    retVal = GenMain(rec, file);
  if (retVal)
    retVal = GenModifiers(rec, file);
  return retVal;
}

/*
 * CheckStructure
 *    Verify that the indices are structured properly for use by SFS. For
 * instance, clustered files must have a primary index, and the primary index
 * for relative files must have only one field of type unsigned long.
 * Perhaps some of this checking should go at a higher level.
 */

Boolean CDLSfsGenerator::CheckStructure(CDLRecord& rec, CDLFileDef& file)

{
  Boolean retVal = true;
  CDLIndexDef* primary;
  CDLIndexDef* curIndex;
  CDLField* field;
  int inc;
  int numFields;
  String primaryName;
  String specialName;

  if (file.Type() == HASHED)
    CDLReport.ReportMsg(ErrorLevels::InfoMsg, 1200);
  if ((file.Type() == CLUSTERED) || (file.Type() == HASHED))
    {
      if (file.PrimaryIndex() == (CDLIndexDef*)0)
	{
	  CDLReport.ReportMsg(ErrorLevels::ErrorMsg, 1201);
	  retVal = false;
	}
    }
  else if (file.Type() == RELATIVE)
    {
      if (file.PrimaryIndex() == (CDLIndexDef*)0)
	retVal = false;
      else
	{
	  primary = file.PrimaryIndex();
	  if ((primary->NumFields() != 1) || (primary->Duplicates()))
	    retVal = false;
	  else
	    {
	      field = rec.Find(primary->Field(0)->Name());
	      if ((field == (CDLField*)0) || 
		  (field->Type() != CDLField::LONG) ||
		  ((field->Type() == CDLField::LONG) && 
		   (field->Modifier() != CDLField::UNSIGNED)))
		retVal = false;
	    }
	}
      if (!retVal)
	{	
	  CDLReport.ReportMsg(ErrorLevels::ErrorMsg, 1202);
	}
    }
  else if (file.Type() == SEQUENTIAL)
    {
      if (file.PrimaryIndex() == (CDLIndexDef*)0)
	retVal = false;
      else
	{
	  primary = file.PrimaryIndex();
	  if ((primary->NumFields() != 1) || (primary->Duplicates()))
	    retVal = false;
	  else
	    {
	      field = rec.Find(primary->Field(0)->Name());
	      if ((field == (CDLField*)0) ||
		  (field->Type() != CDLField::HYPER) ||
		  ((field->Type() == CDLField::HYPER) &&
		   (field->Modifier() != CDLField::UNSIGNED)))
		retVal = false;
	    }
	}
      if (!retVal)
	{
	  CDLReport.ReportMsg(ErrorLevels::ErrorMsg, 1204);
	}
    }
  if (retVal && ((file.Type() == RELATIVE) || (file.Type() == SEQUENTIAL)))
    {
      primaryName = file.PrimaryIndex()->Name();
      specialName = file.PrimaryIndex()->Field(0)->Name();
      numFields = file.NumIndices();
      for (inc = 0; retVal && (inc < numFields); inc++)
	{
	  curIndex = file.Index(inc);
	  if ((curIndex->Name() != primaryName) &&
	      (curIndex->Find(specialName) != (CDLIndexFieldDef*)0))
	    retVal = false;
	}
      if (!retVal)
	{
	  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1203, (const char*)specialName);
	}
    }
  return retVal;
}

/*
 * GenFieldPacking
 *    Generate the appropriate code to copy the field in question into
 * the packed array. We use memcpy for this, and put everything in-line
 * for efficiency.
 */

Boolean CDLSfsGenerator::GenFieldPacking(CDLField& curField,
					 const String& prefix,
					 const String& arrayName,
					 const String& countName,
					 const String& varName,
				         Boolean padStrings,
					 const String& lengthFieldName)
{
  int dataLen = GetSize(curField);
  String address;
  String realVarName;

  address = "&(" + arrayName;
  address += "[";
  address += countName;
  address += "])";
  realVarName = prefix + varName;

  switch(curField.Type())
    {
    case CDLField::VARBYTES:
      outFile << indent << "memcpy(" << address << ", (char*)&";
      outFile << prefix << lengthFieldName << ", 4);" << endl;
      outFile << indent << countName << " += 4;" << endl;
      outFile << indent << "memcpy(" << address << ", " << realVarName;
      outFile << ", (unsigned int)" << prefix << lengthFieldName;
      outFile << ");" << endl;
      outFile << indent << countName << " += (unsigned int)";
      outFile << prefix << lengthFieldName << ";" << endl;
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << indent << "memcpy(" << address << ", " << realVarName;
      outFile << ", " << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      break;
    case CDLField::STRING:
      if (padStrings)
	{
	  outFile << indent << "memcpy(" << address << ", " << realVarName;
	  outFile << ", " << dataLen << ");" << endl;
	  outFile << indent << countName << " += " << dataLen << ";" << endl;
	}
      else
	{
	  outFile << indent << "strncpy((char*)" << address << ", (char*)";
	  outFile << realVarName << ", " << dataLen << ");" << endl;
	  outFile << indent << countName << " += (strlen((char*)" << realVarName;
	  outFile << ") + 1);" << endl;
	}
      break;
    case CDLField::CHAR:
    case CDLField::ENUM:
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "unsigned short tmpShort = (unsigned short)";
      outFile << realVarName << ";" << endl;
      outFile << indent << "memcpy(" << address << ", (char*)&tmpShort, ";
      outFile << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      Outdent();
      outFile << indent << "}" << endl;
      break;
    default:
      outFile << indent << "memcpy(" << address << ", (char*)&" << realVarName;
      outFile << ", " << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      break;
    }
  return true;
}

/*
 * GenFieldUnpacking
 *    Generate the appropriate code to copy the field in question into
 * the packed array. We use memcpy for this, and put everything in-line
 * for efficiency.
 */

Boolean CDLSfsGenerator::GenFieldUnpacking(CDLField& curField,
					   const String& prefix,
					   const String& arrayName,
					   const String& countName,
					   const String& varName,
					   Boolean padStrings,
					   const String& lengthFieldName)
{
  int dataLen = GetSize(curField);
  String address;
  String realVarName;

  address = "&(" + arrayName;
  address += "[";
  address += countName;
  address += "])";
  realVarName = prefix + varName;

  switch(curField.Type())
    {
    case CDLField::VARBYTES:
      outFile << indent << "memcpy((char*)&" << prefix << lengthFieldName;
      outFile << ", " << address << ", 4);" << endl;
      outFile << indent << countName << " += 4;" << endl;
      outFile << indent << "memcpy(" << realVarName << ", " << address;
      outFile << ", (unsigned int)" << prefix << lengthFieldName;
      outFile << ");" << endl;
      outFile << indent << countName << " += (unsigned int)";
      outFile << prefix << lengthFieldName << ";" << endl;
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << indent << "memcpy(" << realVarName << ", " << address << ", ";
      outFile << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      break;
    case CDLField::STRING:
      if (padStrings)
	{
	  outFile << indent << "memcpy(" << realVarName << ", " << address;
	  outFile << ", " << dataLen << ");" << endl;
	  outFile << indent << countName << " += " << dataLen << ";" << endl;
	}
      else
	{
	  outFile << indent << "strncpy((char*)" << realVarName << ", (char*)";
	  outFile << address << ", " << dataLen << ");" << endl;
	  outFile << indent << countName << " += (strlen((char*)" << address;
	  outFile << ") + 1);" << endl;
	}
      break;
    case CDLField::ENUM:
      outFile << indent << "{" << endl;	
      Indent();
      outFile << indent << "unsigned short tmpShort;" << endl;
      outFile << indent << "memcpy((char*)&tmpShort, " << address;
      outFile << ", " << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      outFile << indent << realVarName << " = (" << curField.RealTypeName();
      outFile << ")tmpShort;" << endl;
      Outdent();
      outFile << indent << "}" << endl;
      break;
    case CDLField::CHAR:
      outFile << indent << "{" << endl;	
      Indent();
      outFile << indent << "unsigned short tmpShort;" << endl;
      outFile << indent << "memcpy((char*)&tmpShort, " << address;
      outFile << ", " << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      outFile << indent << realVarName << " = (";
      if (curField.Modifier() == CDLField::UNSIGNED)
	outFile << "unsigned ";
      outFile << "char)tmpShort;" << endl;
      Outdent();
      outFile << indent << "}" << endl;
      break;
    default:
      outFile << indent << "memcpy((char*)&" << realVarName << ", " << address;
      outFile << ", " << dataLen << ");" << endl;
      outFile << indent << countName << " += " << dataLen << ";" << endl;
      break;
    }
  return true;
}

/*
 * GenFieldSpec
 *    In a similar manner to GenField, set the fields of a an SFS field
 * specification structure to refer to a particuar CDL field.
 */

Boolean CDLSfsGenerator::GenFieldSpec(CDLField& curField, 
				      const String& prefix,
				      const String& sfsFieldName,
				      int ent)
{
  int dataLen;
  dataLen = GetSize(curField);

  outFile << indent << prefix << "[" << ent << "].fieldName = \"";
  outFile << sfsFieldName << "\";" << endl;
  outFile << indent << prefix << "[" << ent << "].fieldType = ";
  switch(curField.Type())
    {
    case CDLField::REAL:
      outFile << "sfs_double;" << endl;
      break;
    case CDLField::BYTES:
    case CDLField::CHARRAY:
      outFile << "sfs_byteArray;" << endl;
      break;
    case CDLField::VARBYTES:
      outFile << "sfs_varLenByteArray;" << endl;
      break;
    case CDLField::STRING:
      outFile << "sfs_string;" << endl;
    break;
    case CDLField::HYPER:
      if (curField.Modifier() == CDLField::SIGNED)
	outFile << "sfs_signedInt64;" << endl;
      else
	outFile << "sfs_unsignedInt64;" << endl;
      break;
    case CDLField::LONG:
      if (curField.Modifier() == CDLField::SIGNED)
	outFile << "sfs_signedInt32;" << endl;
      else
	outFile << "sfs_unsignedInt32;" << endl;
      break;
    case CDLField::SHORT:
    case CDLField::CHAR:
      if (curField.Modifier() == CDLField::SIGNED)
	outFile << "sfs_signedInt16;" << endl;
      else
	outFile << "sfs_unsignedInt16;" << endl;
      break;
    case CDLField::ENUM:
      outFile << "sfs_unsignedInt16;" << endl;
      break;
    case CDLField::RECORD:
      break;
    }
  outFile << indent << prefix << "[" << ent << "].fieldSize = ";
  outFile << dataLen << ";" << endl;
  outFile << indent << prefix << "[" << ent;
  outFile << "].collatingLanguage = (char*)0;" << endl;
  return true;
}

/*
 * GenIndexField
 *    Set the fields of a secondary index field specification to refer
 * to the appropriate field in the index.
 */

Boolean CDLSfsGenerator::GenIndexField(CDLIndexFieldDef& curIField,
				       const String& prefix, int ent)
{
  String shortFieldName;

  GenWholeFieldName('.', curIField.Name(), "", 32, shortFieldName);
  outFile << indent << prefix << "[" << ent << "].fieldName = \"";
  outFile << shortFieldName << "\";" << endl;
  outFile << indent << prefix << "[" << ent << "].indexFieldOrdering = ";
  if (curIField.Direction() == BACKWARDS)
    outFile << "sfs_descending;" << endl;
  else
    outFile << "sfs_ascending;" << endl;
  return true;
}

/*
 * GenSubclasses
 *    Generate the definitions and implementations for the subclasses of
 * SFSBaseRecord, SFSBaseKey, SFSBaseSecondary and SFSBaseFile. These subclasses
 * turn the appropriate record or index specifier for this CDL record into
 * a base class object that may be used more easily by SFS.
 */
      
Boolean CDLSfsGenerator::GenSubclasses(CDLRecord& rec, CDLFileDef& file)

{
  int inc;
  int ent;
  int recursiveCount;
  int maxPackedSize;
  CDLField* curField;
  CDLIndexDef* curIndex;
  CDLIndexFieldDef* curIField;
  String recName;
  String recBaseName;
  String fileName;
  String className;
  String tmpName;
  List<FieldNode> recursiveFields;
  Listiter<FieldNode> fieldIter(recursiveFields);
  FieldNode curNode;
  FieldNode testNode;
  
  recName = rec.Name();
  recBaseName = rec.Name();
  recBaseName += "Base";
  fileName = file.Name();
  className = "SFS";
  className += fileName;
  className += "Record";

  BuildFieldList(rec, file, recursiveFields);
  recursiveCount = recursiveFields.length();

/* Class: SFSFileRecord */
  outFile << indent << "class " << className << " : public SFSBaseRecord" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "public:" << endl;
  Indent();
  outFile << indent << className << "();" << endl;
  outFile << indent << "virtual ~" << className << "();" << endl;
  outFile << indent << "CDLStatus Pack(" << recName << "&);" << endl;
  outFile << indent << "CDLStatus UnPack(" << recName;
  outFile << "&);" << endl;
  Outdent();
  outFile << indent << "};\n" << endl;
  
/* SFSFileRecord::SFSFileRecord */
  outFile << indent << className << "::" << className << "()" << endl;
  outFile << indent << ": SFSBaseRecord()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  maxPackedSize = GetMaxSize(recursiveFields);
  outFile << indent << "rec.format = sfs_contiguous;" << endl;
  outFile << indent << "rec.buffer = (sfs_pointer_t)malloc(";
  outFile << maxPackedSize << ");" << endl;
  outFile << indent << "rec.bufferLength = " << maxPackedSize << ";" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFileRecord::~SFSFileRecord */
  outFile << indent << className << "::~" << className << "()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

/* SFSFileRecord::Pack */

  outFile << indent << "CDLStatus " << className << "::Pack(";
  outFile << recName << "& r)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << recBaseName << "* base = r;" << endl;
  outFile << indent << "unsigned int packPos = 0;" << endl;
  outFile << indent << "unsigned char* buf = (unsigned char*)rec.buffer;";
  outFile << endl;
  for (fieldIter.reset(); fieldIter.next(curNode); )
    {
      GenFieldPacking(*(curNode.Field()), "base->", "buf", "packPos", 
		      curNode.VariableName(), true, 
		      curNode.LengthFieldName());
    }
  outFile << indent << "rec.dataLength = (unsigned long)packPos;" << endl;
  outFile << indent << "return CheckPacking(packPos);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFileRecord::UnPack */
  outFile << indent << "CDLStatus " << className << "::UnPack(";
  outFile << recName << "& r)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << recBaseName << "* base = r;" << endl;
  outFile << indent << "unsigned long dataPos = 0;" << endl;
  outFile << indent << "unsigned char* buf = (unsigned char*)rec.buffer;";
  outFile << endl;
  for (fieldIter.reset(); fieldIter.next(curNode); )
    {
      GenFieldUnpacking(*(curNode.Field()), "base->", "buf", "dataPos",
			curNode.VariableName(), true,
			curNode.LengthFieldName());
   }
  outFile << indent << "return CheckUnpacking(dataPos);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* Class SFSFileKey */
  className = "SFS";
  className += fileName;
  className += "Key";
  outFile << indent << "class " << className << " : public SFSBaseKey" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "public:" << endl;
  Indent();
  outFile << indent << className << "(" << fileName << "::Index);" << endl;
  outFile << indent << "virtual ~" << className << "();" << endl;
  outFile << indent << "CDLStatus Pack(" << recName << "&);" << endl;
  Outdent();
  outFile << indent << "private:" << endl;
  Indent();
  outFile << indent << fileName << "::Index whichIndex;" << endl;
  Outdent();
  outFile << "};\n" << endl;

/* SFSFileKey::SFSFileKey */
  outFile << indent << className << "::" << className << "(";
  outFile << fileName << "::Index index)" << endl;
  outFile << ": SFSBaseKey()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "whichIndex = index;" << endl;
  outFile << indent << "key.keyFormat = sfs_packed;" << endl;
  outFile << indent << "key.keyRecord.format = sfs_contiguous;" << endl;
  outFile << indent << "switch(whichIndex)" << endl;
  outFile << indent << "{" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      outFile << indent << "case " << fileName << "::" << tmpName.upper();
      outFile << ":" << endl;
      Indent();
      if ((file.Type() != SEQUENTIAL) || 
	  (tmpName != file.PrimaryIndex()->Name()))
	{
	  maxPackedSize = GetMaxIndexSize(curIndex, recursiveFields);
	  outFile << indent << "indexName = \"" << curIndex->Name();
	  outFile << "\";" << endl;
	  outFile << indent << "key.keyRecord.buffer = (sfs_pointer_t)";
	  outFile << "malloc(" << maxPackedSize << ");" << endl;
	  outFile << indent << "key.keyRecord.bufferLength = ";
	  outFile << maxPackedSize << ";" << endl;
	}
      else  
      {
        outFile << indent << "indexName = \"Primary\";" << endl;
	outFile << indent << "key.keyRecord.buffer = (sfs_pointer_t)malloc(8);";
	outFile << endl;
        outFile << indent << "key.keyRecord.bufferLength = 8;" << endl;
      }
      outFile << indent << "break;" << endl;
      Outdent();
    }
  outFile << indent << "}" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFileKey::~SFSFileKey */
  outFile << indent << className << "::~" << className << "()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

/* SFSFileKey::Pack */
  outFile << indent << "CDLStatus " << className << "::Pack(";
  outFile << recName << "& r)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << recName << "Base* base = r;" << endl;
  outFile << indent << "unsigned int packPos = 0;" << endl;
  outFile << indent;
  outFile << "unsigned char* buf = (unsigned char*)key.keyRecord.buffer;";
  outFile << endl;
  outFile << indent << "switch (whichIndex)" << endl;
  outFile << indent << "{" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      outFile << indent << "case " << fileName << "::" << tmpName.upper();
      outFile << ":" << endl;
      Indent();
      if ((file.Type() != SEQUENTIAL) ||
	  (tmpName != file.PrimaryIndex()->Name()))
         { 
	  for (ent = 0; ent < curIndex->NumFields(); ent++)
	    {
	      curIField = curIndex->Field(ent);
	      testNode.VariableName(curIField->Name());
	      fieldIter.reset();
	      if (!fieldIter.find_next(testNode))
		{
		  CDLReport.VReportMsg(ErrorLevels::ErrorMsg, 1105,
                                       (const char*)curIField->Name(),
                                       (const char*)rec.Name());
                  return false;
                }
              else
                {
		  fieldIter.next(curNode);
		  curField = curNode.Field();
		  GenFieldPacking(*curField, "base->", "buf", 
				  "packPos", curNode.VariableName(), false,
				  curNode.LengthFieldName());
		}
	    }
        }
      else 
      {
	String field_name;
        curIField = curIndex->Field(0);
        field_name = curIField->Name();
        outFile << indent << "memcpy(&(buf[packPos]), (char*)&base->";
        outFile << field_name << ", 8);" << endl;
        outFile << indent << "packPos += 8;" << endl;
      } 

      outFile << indent << "break;" << endl;
      Outdent();
    }
  outFile << indent << "}" << endl;
  outFile << indent;
  outFile << "key.keyRecord.dataLength = (unsigned long)packPos;" << endl;
  outFile << indent << "return CheckPacking(packPos);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* Class SFSFileSecondary */
  className = "SFS";
  className += fileName;
  className += "Secondary";
  outFile << indent << "class " << className;
  outFile << " : public SFSBaseSecondary" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "public:" << endl;
  Indent();
  outFile << indent << className << "(" << fileName << "::Index);" << endl;
  outFile << indent << "virtual ~" << className << "();" << endl;
  Outdent();
  outFile << "};\n" << endl;

/* SFSFileSecondary::SFSFileSecondary */
  outFile << indent << className << "::" << className << "(" << fileName;
  outFile << "::Index index)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "switch(index)" << endl;
  outFile << indent << "{" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      outFile << indent << "case " << fileName << "::" << tmpName.upper();
      outFile << ":" << endl;
      Indent();
      if (tmpName != file.PrimaryIndex()->Name())
	{
	  outFile << indent << "indexName = \"";
	  outFile << curIndex->Name() << "\";" << endl;
	  if (curIndex->Duplicates())
	    outFile << indent << "spec.unique = 0;" << endl;
	  else
	    outFile << indent << "spec.unique = 1;" << endl;
	  outFile << indent << "spec.numFields = " << curIndex->NumFields();
	  outFile << ";" << endl;
	  outFile << indent << "spec.fieldSpecArrayP = (sfs_indexFieldSpec_t*)";
	  outFile << "malloc(sizeof(sfs_indexFieldSpec_t) * ";
	  outFile << curIndex->NumFields() << ");" << endl;
	  for (ent = 0; ent < curIndex->NumFields(); ent++)
	    {
	      curIField = curIndex->Field(ent);
	      GenIndexField(*curIField, "spec.fieldSpecArrayP", ent);
	    }
	}
      outFile << indent << "break;" << endl;
      Outdent();
    }
  outFile << indent << "}" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFileSecondary::~SFSFileSecondary */
  outFile << indent << className << "::~" << className << "()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

/* Class SFSFileFileSpec */
  className = "SFS";
  className += fileName;
  className += "FileSpec";
  outFile << indent << "class " << className;
  outFile << " : public SFSBaseFileSpec" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "public:" << endl;
  Indent();
  outFile << indent << className << "();" << endl;
  outFile << indent << "virtual ~" << className << "();" << endl;
  Outdent();
  outFile << "};\n" << endl;

/* SFSFileFileSpec::SFSFileFileSpec */
  outFile << indent << className << "::" << className << "()" << endl;
  outFile << indent << ": SFSBaseFileSpec(\"" << recName << "\")" << endl;
  outFile << indent << "{" << endl;
  Indent();
  switch(file.Type())
    {
    case CLUSTERED:
    case HASHED:
      outFile << indent << "spec.fileOrganization = sfs_btreeClustered;" << endl;
      break;
    case SEQUENTIAL:
      outFile << indent << "spec.fileOrganization = sfs_entrySequenced;" << endl;
      break;
    case RELATIVE:
      outFile << indent << "spec.fileOrganization = sfs_relative;" << endl;
      break;
    }
  outFile << indent << "spec.primaryIndexName = \"Primary\";" << endl;
  outFile << indent << "spec.maxNumberRecords = SFS_NATURAL_RECORD_LIMIT;" << endl;
  outFile << indent << "spec.recordSpec.numFields = " << recursiveCount;
  outFile << ";" << endl;
  outFile << indent << "spec.recordSpec.fieldSpecArrayP = ";
  outFile << "(sfs_recordFieldSpec_t*)malloc(sizeof(sfs_recordFieldSpec_t) * ";
  outFile << recursiveCount << ");" << endl;
  ent = 0;
  for (fieldIter.reset(); fieldIter.next(curNode); )
    {
      GenFieldSpec(*(curNode.Field()), "spec.recordSpec.fieldSpecArrayP",
	       curNode.Name(), ent++);
    }
  curIndex = file.PrimaryIndex();
  if (curIndex == (CDLIndexDef*)0)
    return false;
  if (curIndex->Duplicates())
    outFile << indent << "spec.primaryIndexSpec.unique = 0;" << endl;
  else
    outFile << indent << "spec.primaryIndexSpec.unique = 1;" << endl;
  outFile << indent << "spec.primaryIndexSpec.numFields = ";
  outFile << curIndex->NumFields() << ";" << endl;
  outFile << indent << "spec.primaryIndexSpec.fieldSpecArrayP = ";
  outFile << "(sfs_indexFieldSpec_t*)malloc(sizeof(sfs_indexFieldSpec_t) * ";
  outFile << curIndex->NumFields() << ");" << endl;
  for (ent = 0; ent < curIndex->NumFields(); ent++)
    {
      curIField = curIndex->Field(ent);
      GenIndexField(*curIField, "spec.primaryIndexSpec.fieldSpecArrayP", ent);
    }
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFileFileSpec::~SFSFileFileSpec */
  outFile << indent << className << "::~" << className << "()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

  return true;
}

/*
 * GenHeader
 *    Generate the header file information for this class, which consists
 * of a definition of the main class we're trying to generate. 
 */

Boolean CDLSfsGenerator::GenHeader(CDLRecord& rec, CDLFileDef& file)

{
  int inc;
  CDLField* curField;
  String recName;
  String fileName;  String className;
  String tmpName;

  recName = rec.Name();
  fileName = file.Name();
  className = "SFS";
  className += fileName;
  inclFile << indent << "class " << className;
  inclFile << " : public " << fileName << ", public SFSFileBase\n" << endl;
  inclFile << indent << "{" << endl;
  inclFile << "public:" << endl;
  Indent();
  inclFile << indent << className << "();" << endl;
  inclFile << indent << "virtual ~" << className << "();" << endl;
  inclFile << indent << "virtual CDLStatus OpenFile(const String& dbName, ";
  inclFile << "const String& fileName," << endl;
  inclFile << indent << indent << "unsigned short flags = 0);" << endl;
  inclFile << indent << "virtual CDLStatus CloseFile();" << endl;
  inclFile << indent << "virtual CDLStatus CreateFile(const String& dbName, ";
  inclFile << "const String& fileName, const String& volName, "; 
  inclFile << "unsigned long numRecs, Boolean makeIndices = true);" << endl;
  inclFile << indent << "virtual CDLStatus DestroyFile();" << endl;
  inclFile << indent << "virtual CDLStatus EmptyFile();" << endl; 
  if (file.HasIndices())
    {
      inclFile << indent << "virtual CDLStatus Select(" << fileName;
      inclFile << "::Index, " << recName << "* start, ";
      inclFile << recName << "* end, CDLIndexDir);" << endl;
    }
  inclFile << indent << "virtual CDLStatus Insert(" << recName;
  inclFile << "& newRec);" << endl;
  inclFile << indent << "virtual CDLStatus InsertList(List<" << recName << ">&);";
  inclFile << endl;
  inclFile << indent << "virtual CDLStatus ReadCurrent(CDLReadLock, ";
  inclFile << "CDLRecordDir, " << recName << "& out);" << endl;
  if (file.HasUniqueIndices())
    {
      inclFile << indent << "virtual CDLStatus ReadUnique(CDLReadLock, ";
      inclFile << fileName << "::Index, " << recName << "& key, ";
      inclFile << recName << "& out);" << endl;
    }
  if (file.HasIndices())
    {
      inclFile << indent << "virtual CDLStatus ReadAll(CDLReadLock, ";
      inclFile << fileName << "::Index, " << recName;
      inclFile << "& key, CDLIndexDir, List<";
      inclFile << recName << ">& out, unsigned int maxHits = 0);" << endl;
      inclFile << indent << "virtual CDLStatus ReadAll(CDLReadLock, ";
      inclFile << fileName << "::Index, " << recName << "* start," << endl;
      inclFile << indent << indent << recName << "* end, ";
      inclFile << "CDLIndexDir, List<" << recName << ">& out, ";
      inclFile << "unsigned int maxHits = 0);" << endl;
    }
  inclFile << indent << "virtual CDLStatus DeleteCurrent(CDLRecordDir);";
  inclFile << endl;
  if (file.HasUniqueIndices())
    {
      inclFile << indent << "virtual CDLStatus DeleteUnique(" << fileName;
      inclFile << "::Index, " << recName << "& key);" << endl;
    }
  if (file.HasIndices())
    {
      inclFile << indent << "virtual CDLStatus DeleteAll(" << fileName;
      inclFile << "::Index, " << recName << "& key);" << endl;
      inclFile << indent << "virtual CDLStatus DeleteAll(" << fileName;
      inclFile << "::Index, " << recName << "* start," << endl;
      inclFile << indent << indent << recName << "* end);" << endl;
    }
  inclFile << indent << "virtual CDLStatus UpdateCurrent(CDLRecordDir, ";
  inclFile << recName << "& newVal);" << endl;
  inclFile << indent << "virtual CDLStatus SaveContext(void*& ctxt);" << endl;
  inclFile << indent << "virtual CDLStatus RestoreContext(void* ctxt);" << endl;
  if (file.HasUniqueIndices())
    {
      inclFile << indent << "virtual CDLStatus UpdateUnique(" << fileName;
      inclFile << "::Index, " << recName << "& key, ";
      inclFile << recName << "& newVal);" << endl;

      for (inc = 0; inc < rec.NumFields(); inc++)	// Do modifies & increments
	{
	  curField = rec.Field(inc);
	  if (IsAtomic(curField->Type()) && (curField->ArraySize() == 1))
	    {
	      CDLClassGenerator::GetTypeName(curField, tmpName, false);
	      inclFile << indent << "virtual CDLStatus Modify_" << curField->Name();
	      inclFile << "(" << fileName << "::Index, ";
	      inclFile << recName << "& key, " << tmpName << " newVal, ";
	      inclFile << recName << "& out);" << endl;
	      inclFile << indent << "virtual CDLStatus Increment_";
	      inclFile << curField->Name() << "(" << fileName;
	      inclFile << "::Index, " << recName << "& key, ";
	      inclFile << tmpName << " incrementBy, ";
	      inclFile << recName << "& out,";
	      inclFile << "Boolean incrementLock = false);" << endl;
	    }
	}
    }
  inclFile << indent << "static CDLStatus StartBatch();" << endl;
  inclFile << indent << "static CDLStatus EndBatch();" << endl;
  inclFile << indent << "virtual CDLStatus ActivateIndex(";
  inclFile << fileName << "::Index, const String& volume, ";
  inclFile << recName << "* nullKey = (" << recName << "*)0);" << endl;
  inclFile << indent << "virtual CDLStatus DeactivateIndex(";
  inclFile << fileName << "::Index);" << endl;
  inclFile << indent << "virtual CDLStatus DestroyIndex(";
  inclFile << fileName << "::Index);" << endl;
  inclFile << indent << "virtual CDLStatus ActivateAllIndices";
  inclFile << "(const String& volume);" << endl;
  inclFile << indent << "virtual CDLStatus DeactivateAllIndices();" << endl;
  inclFile << indent << "virtual CDLStatus DestroyAllIndices();" << endl;
  Outdent();
  inclFile << indent << "};\n" << endl;
  return true;
}

/*
 * GenMain
 *     Generate most of the function bodies for the functions defined by
 * GenHeader. Some of this could be done with templates, but then we wouldn't
 * be able to write specific members to set and increment individual fields.
 */

Boolean CDLSfsGenerator::GenMain(CDLRecord& rec, CDLFileDef& file)

{
  int inc;
  CDLIndexDef* curIndex;
  String recName;
  String fileName;
  String className;
  String tmpName;
  String esnName;

  recName = rec.Name();
  fileName = file.Name();
  className = "SFS";
  className += fileName;
  if (file.Type() == SEQUENTIAL)
    esnName = file.PrimaryIndex()->Field(0)->Name();
  else
    esnName = "";
 
/* SFSFile::SFSFile */
  outFile << indent << className << "::" << className << "()" << endl;
  outFile << indent << indent << ": " << fileName << "(), SFSFileBase()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

/* SFSFile::~SFSFile */
  outFile << indent << className << "::~" << className << "()" << endl;
  outFile << indent << "{" << endl;
  outFile << indent << "}\n" << endl;

/* SFSFile::CreateFile */
  outFile << indent << "CDLStatus " << className;
  outFile << indent << "::CreateFile(const String& dbName, const String& file, ";
  outFile << "const String& vol, unsigned long numRecs, ";
  outFile << "Boolean makeIndices)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal;" << endl;
  outFile << indent << "unsigned long bytes = sizeof(" << recName;
  outFile << ") * numRecs;" << endl;
  outFile << indent << className << "FileSpec fSpec;" << endl;
  outFile << indent << "retVal = ICreate(fSpec, dbName, file, vol, bytes);";
  outFile << endl;
  outFile << indent << "if (makeIndices && (retVal == CDL_SUCCESS))" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "retVal = OpenFile(dbName, file, ";
  outFile << "CDL_EXCLUSIVE_OPEN | CDL_NONTRANSACTIONAL_OPEN);" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      if (curIndex->BuildMe() && (tmpName.upper() != "PRIMARY"))
	{
	  outFile << indent << "if (retVal == 0)" << endl;
	  Indent();
	  outFile << indent << "retVal = ActivateIndex(" << fileName << "::";
	  outFile << tmpName.upper() << ", vol);" << endl;
	  Outdent();
	}
    }
  outFile << indent << "CloseFile();" << endl;
  Outdent();
  outFile << indent << "}" << endl;
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << "}\n" << endl;

/* SFSFile::OpenFile */
  outFile << indent << "CDLStatus " << className << "::OpenFile(const String& ";
  outFile << "dbName, const String& fileName, unsigned short flags)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return IOpenFile(dbName, fileName, ";
  outFile << "\"" << recName << "\", flags);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::CloseFile */
  outFile << indent << "CDLStatus " << className << "::CloseFile()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return ICloseFile();" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::Select */
  if (file.HasIndices())
    {
      outFile << indent << "CDLStatus " << className << "::Select(" << fileName;
      outFile << indent <<"::Index index, " << recName << "* start, ";
      outFile << indent << recName << "* end, CDLIndexDir dir)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
      outFile << indent << className << "Key* sk;" << endl;
      outFile << indent << className << "Key* ek;" << endl;
      outFile << indent << className << "Key tmpKey(index);" << endl;
      outFile << indent << "String iName = tmpKey.Name();" << endl;
      outFile << indent << "if (start != (" << recName << "*)0)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "sk = new " << className;
      outFile << "Key(index);" << endl;
      outFile << indent << "retVal = sk->Pack(*start);" << endl;
      Outdent();
      outFile << indent << "}" << endl;
      outFile << indent << "else" << endl;
      Indent();
      outFile << indent << "sk = (" << className << "Key*)0;" << endl;
      Outdent();
      outFile << indent << "if (end != (" << recName << "*)0)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "ek = new " << className;
      outFile << "Key(index);" << endl;
      outFile << indent << "if (retVal == 0)" << endl;
      Indent();
      outFile << indent << "retVal = ek->Pack(*end);" << endl;
      Outdent();
      Outdent();
      outFile << indent << "}" << endl;
      outFile << indent << "else" << endl;
      Indent();
      outFile << indent << "ek = (" << className << "Key*)0;" << endl;
      Outdent();
      outFile << indent << "if (retVal == 0)" << endl;
      Indent();
      outFile << indent << "retVal = ISelect(iName, sk, ek, dir);" << endl;
      Outdent();
      outFile << indent << "if (sk != (" << className << "Key*)0)" << endl;
      Indent();
      outFile << indent << "delete sk;" << endl;
      Outdent();
      outFile << indent << "if (ek != (" << className << "Key*)0)" << endl;
      Indent();
      outFile << indent << "delete ek;" << endl;
      Outdent();
      outFile << indent << "return retVal;" << endl;
      Outdent();
      outFile << "}\n" << endl;
    }

/* SFSFile::Insert */
  outFile << indent << "CDLStatus " << className << "::Insert(";
  outFile << recName << "& rec)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
  outFile << indent << className << "Record newRec;" << endl;
  outFile << indent << "retVal = newRec.Pack(rec);" << endl;

  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
  Indent();
  if (file.Type() == SEQUENTIAL)
    {
      CDLIndexDef* curInd;  
      curInd = file.PrimaryIndex();
      outFile << indent << "{" <<endl;
      Indent();
      outFile << indent << "ExistingTransaction tran;" << endl;
      outFile << indent << "retVal = IInsert(newRec);" << endl;
      outFile << indent << "if (!IsBatching(tran))" << endl;
      Indent();
      outFile << indent << "{" << endl;
      Outdent();
      outFile << indent << "sfs_esn_t esn;" << endl;
      outFile << indent << "idl_uhyper_int esnInt;" << endl;
      outFile << indent << "IGetEsn(esn);" << endl;
      outFile << indent << "memcpy((void*)&esnInt, (const void*)&esn, sizeof(sfs_esn_t));" << endl;
      CDLIndexFieldDef* curIField;
      String field_name;
      curIField = curInd->Field(0);
      field_name = curIField->Name();
      outFile << indent << "rec." << field_name << "(esnInt);" << endl; 
      Outdent();
      outFile << indent << "}" << endl;
      Outdent();
      outFile << indent << "}" << endl;
    }
  else
      outFile << indent << "retVal = IInsert(newRec);" << endl;
  Outdent();
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << "}\n" << endl;

  outFile << indent << "CDLStatus " << className << "::InsertList(List<";
  outFile << recName << ">& recs)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
  outFile << indent << recName << "* newRec;" << endl;
  outFile << indent << className << "Record* newBase;" << endl;
  outFile << indent << "StartBatch();" << endl;
  outFile << indent << "Listiter<" << recName << "> iter(recs);" << endl;
  outFile << indent << "while ((retVal == CDL_SUCCESS) && iter.next(newRec))" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "newBase = new " << className;
  outFile << "Record();" << endl;
  outFile << indent << "retVal = newBase->Pack(*newRec);" << endl;
  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
  Indent();
  outFile << indent << "retVal = IInsert(*newBase);" << endl;
  Outdent();
  outFile << indent << "delete newBase;" << endl;
  Outdent();
  outFile << indent << "}" << endl;
  outFile << indent << "EndBatch();" << endl;
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::ReadCurrent */
  outFile << indent << "CDLStatus " << className;
  outFile << "::ReadCurrent(CDLReadLock lock, CDLRecordDir dir, ";
  outFile << recName << "& out)" << endl;
  outFile << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal;" << endl;
  outFile << indent << className << "Record rec;" << endl;
  outFile << indent << "retVal = IReadCurrent(lock, rec, dir);" << endl;
  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
  Indent();
  outFile << indent << "retVal = rec.UnPack(out);" << endl;
  Outdent();
  if (file.Type() == SEQUENTIAL)
    {
      outFile << indent << "sfs_esn_t esn;" << endl;
      outFile << indent << "idl_uhyper_int esnInt;" << endl;
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "IGetEsn(esn);" << endl;
      outFile << indent << "memcpy((void*)&esnInt, (const void*)&esn, ";
      outFile << "sizeof(sfs_esn_t));" << endl;
      outFile << indent << "out." << esnName << "(esnInt);" << endl;
      Outdent();
      outFile << indent << "}" << endl;
    }
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
/* SFSFile::ReadUnique */
  if (file.HasUniqueIndices())
    {
      outFile << indent << "CDLStatus " << className;
      outFile << "::ReadUnique(CDLReadLock lock, " << fileName;
      outFile << "::Index index, " << recName << "& k, ";
      outFile << recName << "& out)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
      outFile << indent << className << "Key key(index);" << endl;
      outFile << indent << className << "Record rec;" << endl;
      outFile << indent << "retVal = key.Pack(k);" << endl;
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = IReadUnique(lock, key, rec);" << endl;
      Outdent();
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = rec.UnPack(out);" << endl;
      Outdent();
      if (file.Type() == SEQUENTIAL)
	{
	  outFile << indent << "sfs_esn_t esn;" << endl;
	  outFile << indent << "idl_uhyper_int esnInt;" << endl;
	  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
	  outFile << indent << "{" << endl;
	  Indent();
	  outFile << indent << "IGetEsn(esn);" << endl;
	  outFile << indent << "memcpy((void*)&esnInt, (const void*)&esn, ";
	  outFile << "sizeof(sfs_esn_t));" << endl;
	  outFile << indent << "out." << esnName << "(esnInt);" << endl;
	  Outdent();
	  outFile << indent << "}" << endl;
	}
      outFile << indent << "return retVal;" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;
    }
  
/* SFSFile::ReadAll */
  if (file.HasIndices())
    {
      outFile << indent << "CDLStatus " << className;
      outFile << "::ReadAll(CDLReadLock lock, " << fileName;
      outFile << "::Index index, " << recName;
      outFile << "& k, CDLIndexDir dir, List<" << recName;
      outFile << ">& out, unsigned int maxHits)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "return ReadAll(lock, index, &k, &k, ";
      outFile << "dir, out, maxHits);" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;

      outFile << indent << "CDLStatus " << className;
      outFile << "::ReadAll(CDLReadLock lock, " << fileName;
      outFile << "::Index index, " << recName << "* start, " << recName;
      outFile << "* end, CDLIndexDir dir, List<";
      outFile << recName << ">& out, unsigned int maxHits)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      if (file.Type() == SEQUENTIAL)
	{
	  outFile << indent << "sfs_esn_t esn;" << endl;
	  outFile << indent << "idl_uhyper_int esnInt;" << endl;
	}
      outFile << indent << "CDLStatus retVal;" << endl;
      outFile << indent << "unsigned int numHits = 0;" << endl;
      outFile << indent << className << "Record* readRec;" << endl;
      outFile << indent << recName << " tmpRec;" << endl;
      outFile << indent << "retVal = Select(index, start, end, dir);" << endl;
      outFile << indent << "while ((retVal == CDL_SUCCESS) && ";
      outFile << "((maxHits == 0) || (numHits < maxHits)))" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "readRec = new " << className;
      outFile << "Record();" << endl;
      outFile << indent << "retVal = IReadCurrent(lock, *readRec, NEXT);" << endl;
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = readRec->UnPack(tmpRec);" << endl;
      Outdent();
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "numHits++;" << endl;
      outFile << indent << "out.put(tmpRec);" << endl;
      if (file.Type() == SEQUENTIAL)
	{
	  outFile << indent << "IGetEsn(esn);" << endl;
	  outFile << indent << "memcpy((void*)&esnInt, (const void*)&esn, ";
	  outFile << "sizeof(sfs_esn_t));" << endl;
	  outFile << indent << "tmpRec." << esnName << "(esnInt);" << endl;
	}
      outFile << indent << "delete readRec;" << endl;
      Outdent();
      outFile << indent << "}" << endl;
      Outdent();
      outFile << indent << "}" << endl;
      outFile << indent << "if ((retVal == CDL_END_OF_RANGE) && ";
      outFile << "(out.length() == 0))" << endl;
      Indent();
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "ExistingTransaction tran;" << endl;
      outFile << indent << "String errMsg;" << endl;
      outFile << indent << "CDLReport.GetMsg(10023, errMsg);" << endl;
      outFile << indent << "retVal = CDL_KEY_NOT_FOUND;" << endl;
      outFile << indent << "tran.Abort(errMsg, (const int)retVal, CDLId);";
      outFile << endl;
      Outdent();
      outFile << indent << "}" << endl;
      Outdent();
      outFile << indent << "else if (retVal == CDL_END_OF_RANGE)" << endl;
      Indent();
      outFile << indent << "retVal = CDL_SUCCESS;" << endl;
      Outdent();
      outFile << indent << "return retVal;" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;
    }

/* SFSFile::ActivateIndex */
  outFile << indent << "CDLStatus " << className << "::ActivateIndex(";
  outFile << fileName << "::Index index, const String& volume, ";
  outFile << recName << "* nullKey)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << className << "Secondary ind(index);" << endl;
  outFile << indent << "SFSBaseKey* bKey = (SFSBaseKey*)0;" << endl;
  outFile << indent << "if (nullKey != (" << recName << "*)0)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << className << "Key excluded(index);" << endl;
  outFile << indent << "excluded.Pack(*nullKey);" << endl;
  outFile << indent << "bKey = &excluded;" << endl;
  Outdent();
  outFile << indent << "}" << endl;
  outFile << indent << "return IActivateIndex(ind, volume, bKey);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::DeactivateIndex */
  outFile << indent << "CDLStatus " << className << "::DeactivateIndex(";
  outFile << fileName << "::Index index)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << className << "Secondary ind(index);" << endl;
  outFile << indent << "return IDeactivateIndex(ind);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::DestroyIndex */
  outFile << indent << "CDLStatus " << className << "::DestroyIndex(";
  outFile << fileName << "::Index index)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << className << "Secondary ind(index);" << endl;
  outFile << indent << "return IDestroyIndex(ind);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::ActivateAllIndices */
  outFile << indent << "CDLStatus " << className;
  outFile << "::ActivateAllIndices(const String& volume)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      if (tmpName.upper() != "PRIMARY")
	{
	  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
	  Indent();
	  outFile << indent << "retVal = ActivateIndex(" << fileName << "::";
	  outFile << tmpName.upper() << ", volume);" << endl;
	  Outdent();
	}
    }
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::DeactivateAllIndices */
  outFile << indent << "CDLStatus " << className;
  outFile << "::DeactivateAllIndices()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      if (tmpName.upper() != "PRIMARY")
	{
	  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
	  Indent();
	  outFile << indent << "retVal = DeactivateIndex(" << fileName << "::";
	  outFile << tmpName.upper() << ");" << endl;
	  Outdent();
	}
    }
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::DestroyAllIndices */
  outFile << indent << "CDLStatus " << className;
  outFile << "::DestroyAllIndices()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
  for (inc = 0; inc < file.NumIndices(); inc++)
    {
      curIndex = file.Index(inc);
      tmpName = curIndex->Name();
      if (tmpName.upper() != "PRIMARY")
	{
	  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
	  Indent();
	  outFile << indent << "retVal = DestroyIndex(" << fileName << "::";
	  outFile << tmpName.upper() << ");" << endl;
	  Outdent();
	}
    }
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

  return true;
}

/*
 * GenModifiers
 *    Contiues the work of GenMain. Here, we generate the function bodies that
 * delete and update entire records, plus the functions that modify and
 * increment individual fields.
 */
  
Boolean CDLSfsGenerator::GenModifiers(CDLRecord& rec, CDLFileDef& file)

{
  int inc;
  CDLField* curField;
  String recName;
  String fileName;
  String className;
  String tmpName;
  String recBaseName;
  
  recName = rec.Name();
  recBaseName = rec.Name();
  recBaseName += "Base";
  fileName = file.Name();
  className = "SFS";
  className += fileName;
  
/* SFSFile::DeleteCurrent */
  outFile << indent << "CDLStatus " << className;
  outFile << "::DeleteCurrent(CDLRecordDir dir)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return IDeleteCurrent(dir);" << endl;
  Outdent();
  outFile << "}\n" << endl;
  
/* SFSFile::DeleteUnique */
  if (file.HasUniqueIndices())
    {
      outFile << indent << "CDLStatus " << className;
      outFile << "::DeleteUnique(" << fileName;
      outFile << "::Index index, " << recName << "& k)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
      outFile << indent << className << "Key key(index);" << endl;
      outFile << indent << "retVal = key.Pack(k);" << endl;
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = IDeleteUnique(key);" << endl;
      Outdent();
      outFile << indent << "return retVal;" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;
    }
  
/* SFSFile::DeleteAll */
  if (file.HasIndices())
    {
      outFile << indent << "CDLStatus " << className << "::DeleteAll(" << fileName;
      outFile << "::Index index, " << recName << "& k)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "return DeleteAll(index, &k, &k);" << endl;
      Outdent();
      outFile << "}\n" << endl;
      
      outFile << indent << "CDLStatus " << className << "::DeleteAll(" << fileName;
      outFile << "::Index index, " << recName << "* start, ";
      outFile << recName << "* end)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "CDLStatus retVal;" << endl;
      outFile << indent << "retVal = Select(index, start, end, FORWARDS);" << endl;
      outFile << indent << "while (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = DeleteCurrent(NEXT);" << endl;
      Outdent();
      outFile << indent << "if (retVal == CDL_END_OF_RANGE)" << endl;
      Indent();
      outFile << indent << "retVal = CDL_SUCCESS;" << endl;
      Outdent();
      outFile << indent << "return retVal;" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;
    }
  
/* SFSFile::UpdateCurrent */
  outFile << indent << "CDLStatus " << className;
  outFile << "::UpdateCurrent(CDLRecordDir dir, " << recName << "& n)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "CDLStatus retVal;" << endl;
  outFile << indent << className << "Record newRec;" << endl;
  outFile << indent << "retVal = newRec.Pack(n);" << endl;
  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
  Indent();
  outFile << indent << "retVal = IUpdateCurrent(newRec, dir);" << endl;
  Outdent();
  outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
  Indent();
  outFile << indent << "retVal = newRec.UnPack(n);" << endl;
  Outdent();
  outFile << indent << "return retVal;" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
/* SFSFile::UpdateUnique */
  if (file.HasUniqueIndices())
    {
      outFile << indent << "CDLStatus " << className << "::UpdateUnique(";
      outFile << fileName << "::Index index, " << recName;
      outFile << "& k, " << recName << "& n)" << endl;
      outFile << indent << "{" << endl;
      Indent();
      outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
      outFile << indent << className << "Key key(index);" << endl;
      outFile << indent << className << "Record rec;" << endl;
      outFile << indent << "retVal = key.Pack(k);" << endl;
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = rec.Pack(n);" << endl;
      Outdent();
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = IUpdateUnique(key, rec);" << endl;
      Outdent();
      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
      Indent();
      outFile << indent << "retVal = rec.UnPack(n);" << endl;
      Outdent();
      outFile << indent << "return retVal;" << endl;
      Outdent();
      outFile << indent << "}\n" << endl;
    }
  
/* SFSFile::SaveContext */
  outFile << indent << "CDLStatus " << className;
  outFile << "::SaveContext(void*& context)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return ISaveContext(&context);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/* SFSFile::RestoreContext */
  outFile << indent << "CDLStatus " << className;
  outFile << "::RestoreContext(void* context)" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return IRestoreContext(context);" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;

/*
  * Modifiers: One Modify_<fieldName> and Increment_<fieldName> for each
  * atomic-typed field. 
  */
  
  if (file.HasUniqueIndices())
    {
      for (inc = 0; inc < rec.NumFields(); inc++)
	{
	  curField = rec.Field(inc);
	  int fieldSize;
	  if (IsAtomic(curField->Type()) && (curField->ArraySize() == 1))
	    {
	      CDLClassGenerator::GetTypeName(curField, tmpName, false);
	      fieldSize = GetSize(*curField);
	      outFile << indent << "CDLStatus " << className << "::Modify_";
	      outFile << curField->Name() << "(" << fileName << "::Index index, ";
	      outFile << recName << "& k, " << tmpName << " n, " << recName;
	      outFile << "& out)" << endl;
	      outFile << indent << "{" << endl;
	      Indent();
	      outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
	      outFile << indent << recBaseName << "* base = out;" << endl;
	      outFile << indent << className << "Key key(index);" << endl;
	      if (curField->Type() == CDLField::ENUM)
		{
		  outFile << indent << "unsigned short inShort =";
		  outFile << "(unsigned short)n;" << endl;
		  outFile << indent << "unsigned short outShort;" << endl;
		  outFile << indent << "sfs_pointer_t valPtr = ";
		  outFile << "(sfs_pointer_t)&inShort;" << endl;
		  outFile << indent << "sfs_pointer_t outPtr = ";
		  outFile << "(sfs_pointer_t)&outShort;" << endl;
		}
	      else
		{
		  outFile << indent << "sfs_pointer_t valPtr = ";
		  outFile << "(sfs_pointer_t)&n;" << endl;
		  outFile << indent << "sfs_pointer_t outPtr = ";
		  outFile << "(sfs_pointer_t)&(base->";
		  outFile << curField->Name() << ");" << endl;
		}
	      outFile << indent << "retVal = key.Pack(k);" << endl;
	      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
	      Indent();
	      outFile << indent << "retVal = IModifyField(key, \"";
	      outFile << curField->Name();
	      outFile << "\", valPtr, outPtr, " << fieldSize << ");" << endl;
	      if (curField->Type() == CDLField::ENUM)
		{
		  outFile << indent << "base->" << curField->Name();
		  outFile << " = (" << curField->RealTypeName();
		  outFile << ")outShort;" << endl;
		}
	      Outdent();
	      outFile << indent << "return retVal;" << endl;
	      Outdent();
	      outFile << indent << "}\n" << endl;
	      
	      outFile << indent << "CDLStatus " << className << "::Increment_";
	      outFile << curField->Name() << "(" << fileName << "::Index index, ";
	      outFile << recName << "& k, " << tmpName << " n, " << recName;
	      outFile << "& out, Boolean incrementLock)" << endl;
	      outFile << indent << "{" << endl;
	      Indent();
	      outFile << indent << "CDLStatus retVal = CDL_SUCCESS;" << endl;
	      outFile << indent << recBaseName << "* base = out;" << endl;
	      outFile << indent << className << "Key key(index);" << endl;
	      if (curField->Type() == CDLField::ENUM)
		{
		  outFile << indent << "unsigned short inShort =";
		  outFile << "(unsigned short)n;" << endl;
		  outFile << indent << "unsigned short outShort;" << endl;
		  outFile << indent << "sfs_pointer_t valPtr = ";
		  outFile << "(sfs_pointer_t)&inShort;" << endl;
		  outFile << indent << "sfs_pointer_t outPtr = ";
		  outFile << "(sfs_pointer_t)&outShort;" << endl;
		}
	      else
		{
		  outFile << indent << "sfs_pointer_t valPtr = ";
		  outFile << "(sfs_pointer_t)&n;" << endl;
		  outFile << indent << "sfs_pointer_t outPtr = ";
		  outFile << "(sfs_pointer_t)&(base->";
		  outFile << curField->Name() << ");" << endl;
		}
	      outFile << indent << "retVal = key.Pack(k);" << endl;
	      outFile << indent << "if (retVal == CDL_SUCCESS)" << endl;
	      Indent();
	      outFile << indent << "retVal = IIncrementField(key, \"";
	      outFile << curField->Name();
	      outFile << "\", valPtr, outPtr, " << fieldSize;
	      outFile << ", incrementLock);" << endl;
	      if (curField->Type() == CDLField::ENUM)
		{
		  outFile << indent << "base->" << curField->Name();
		  outFile << " = (" << curField->RealTypeName();
		  outFile << ")outShort;" << endl;
		}
	      Outdent();
	      outFile << indent << "return retVal;" << endl;
	      Outdent();
	      outFile << indent << "}\n" << endl;
	    }
	}
    }
  
/* SFSFile::DestroyFile */
  outFile << indent << "CDLStatus " << className << "::DestroyFile()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return IDestroyFile();" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
/* SFSFile::EmptyFile */
  outFile << indent << "CDLStatus " << className << "::EmptyFile()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return IEmptyFile();" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
/* SFSFile::StartBatch */
  outFile << indent << "CDLStatus " << className << "::StartBatch()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return SFSFileBase::IStartBatch();" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
/* SFSFile::EndBatch */
  outFile << indent << "CDLStatus " << className << "::EndBatch()" << endl;
  outFile << indent << "{" << endl;
  Indent();
  outFile << indent << "return SFSFileBase::IEndBatch();" << endl;
  Outdent();
  outFile << indent << "}\n" << endl;
  
  return true;
}

	  
