/*
 * CDLSfsGenerator:
 *
 *    This generator produces a subclass of the file class that works 
 * with SFS. It is a subclass of the SFSFileBase class, which contains 
 * functions shared by all the file subclasses.
 *    FieldNode is a class used internally to store an association between
 * a field and its recursively-expanded name when traversing the stack
 * of nested record definitions.
 *
 * cdlsfsgenerator.H,v 1.7 1994/02/01 22:33:29 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef CDL_SFS_GENERATOR_H
#define CDL_SFS_GENERATOR_H

#include <cdlgenerator.H>
#include <objdce/objtranparams.H>
#include <cdlfield.H>
#include <Path.h>
#include <List.h>

class CDLSfsGenerator : public CDLDataGenerator

{
public:
  CDLSfsGenerator(const ArgList&);
  virtual ~CDLSfsGenerator();
  virtual Boolean Start(const String& fileName);
  virtual Boolean Generate(CDLRecord&, CDLFileDef&);
  virtual Boolean End();
  virtual void DeleteFiles();
private:
  Boolean CheckStructure(CDLRecord&, CDLFileDef&);
  Boolean GenFieldSpec(CDLField&, const String& prefix, 
		       const String& fieldName, int entryNum);
  Boolean GenIndexField(CDLIndexFieldDef&, const String& pfx, int entryNum);
  Boolean GenFieldPacking(CDLField&, const String& prefix,
			  const String& arrayName, const String& countName, 
			  const String& varName, Boolean padStrings,
			  const String& lengthFieldName);
  Boolean GenFieldUnpacking(CDLField&, const String& prefix,
			    const String& arrayName, const String& countName, 
			    const String& varName, Boolean padStrings,
			    const String& lengthFieldName);
  Boolean GenSubclasses(CDLRecord&, CDLFileDef&);
  Boolean GenModifiers(CDLRecord&, CDLFileDef&);
  Boolean GenHeader(CDLRecord&, CDLFileDef&);
  Boolean GenMain(CDLRecord&, CDLFileDef&);
  virtual int GetSize(CDLField&) const;
  int GetMaxIndexSize(CDLIndexDef*, List<FieldNode>&) const;
  Boolean BuildFieldList(CDLRecord&, CDLFileDef&, List<FieldNode>& out);
  Boolean ExpandOneRecord(CDLRecord*, CDLFileDef&, const String& name,
			  List<FieldNode>& varFields,
			  List<FieldNode>& nonVarFields);

  ofstream inclFile;
  String mainName;
  String inclName;
};

#endif







