/*
 * parselist.H
 *
 *    This file contains routines that maintain a list of strings. This
 * can be thought of as an object, but it has a non-object-oriented interface.
 * The lex and yacc routines use this list to maintain a list of tokens they
 * are parsing.
 *    This file also has definitions for the stack of included files. It lets
 * us push things on the list (and thus switch the input buffer to that file)
 * and pop them off.
 *
 *  parselist.H,v 1.6 1994/04/07 19:26:40 gjb Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#ifndef PARSE_LIST_H
#define PARSE_LIST_H

#include <cdl/cdldefs.H>
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <pthread.h>

extern void AddToParseList(
#ifdef __STDC__
			   const char* str
#endif
			 );
extern const char* GetFromParseList(
#ifdef __STDC__
				    const int index
#endif
				  );
extern const char* GetParseListEnd(
#if __STDC__
				   void
#endif
				 );
extern void ClearParseList(
#if __STDC__
			   void
#endif
			 );
extern int ParseListLength(
#if __STDC__
			   void
#endif
			 );
extern void PushIncludeFile(
#ifdef __STDC__
			    void*, const char*, int
#endif
		     );
extern void PopIncludeFile(
#ifdef __STDC__
			   void**, char**, int*
#endif
			  );
extern Boolean IsIncluding(
#if __STDC__
			   void
#endif
			 );
extern void ClearIncludeFiles(
#if __STDC__
			      void
#endif
			    );
extern void AddIncludeName(
#ifdef __STDC__
			   const char* str
#endif
			 );
extern int CheckIncludeName(
#ifdef __STDC__
			    const char* str
#endif
			  );
extern const char* GetIncludeName(
#ifdef __STDC__
				  const int index
#endif
				);
extern void ClearIncludeNames(
#if __STDC__
			      void
#endif
			    );
extern int NumIncludeNames(
#if __STDC__
			   void
#endif
			 );
extern void AddMainIncludeName(
#ifdef __STDC__
			       const char* str
#endif
			     );
extern const char* GetMainIncludeName(
#ifdef __STDC__
				      const int inc
#endif
				    );
extern void ClearMainIncludeNames(
#if __STDC__
				  void
#endif
				);
extern int NumMainIncludeNames(
#if __STDC__
			       void
#endif
			     );
FILE* OpenIncludeFile(
#ifdef __STDC__
		  const char* name, char* outName, char* outInName
#endif
		);
void MakeIncludeList(
#ifdef __STDC__
		     int argc, const char* const* argv
#endif
		   );
void SetCurrentFile(
#ifdef __STDC__
		    const char* fileName
#endif
		  );
void SetCurrentLine(
#ifdef __STDC__
		    int lineNo
#endif
		  );
const char* GetCurrentFile(
#if __STDC__
		     void
#endif
		   );
int GetCurrentLine(
#if __STDC__
		   void
#endif
		 );
void CDLPError(
#ifdef __STDC__
	       const char* errStr
#endif
	     );
void CDLPNumError(
#ifdef __STDC__
		  const int msgNum
#endif
		);
void CDLPWarning(
#ifdef __STDC__
		 const char* errStr
#endif
	       );
void CDLPNumWarning(
#ifdef __STDC__
		  const int msgNum
#endif
		);
#ifdef __cplusplus
}
#endif

#endif
