Punctuation 	[\"\'\[\]\;\:\{\}\,\=] 
Nonpunctuation 	[^\"\'\[\]\;\:\{\}\,\=] 
Digit		[0-9] 
Blank 		[ \t]
Blanks 		[ \t\n]* 
Nonblank 	[^ \t\n] 
TextChar	[^\"\'\[\]\;\:\{\}\,\=\t\n ] 
AllTextChar	[^\"\'\[\]\;\:\{\}\,\=]
Hash		[\#]

%{
/*
 * sfslex.l
 *
 *    This is the lexical analyzer for the auxiliary file for SFS. The file
 * this analyzer parses holds information on which files to build and which
 * indices to put in the file.
 *
 * sfslex.l,v 1.3 1993/12/03 14:44:27 lcp Exp
 */



    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


extern char* yytext;

#include <sfsparse.h>
#include <sfsparsedefs.H>
#include <parselist.H>

extern void CDLSfsCount();

static int newlineCount;

void CDLSfsGotToken(n)

char* n;

{
  if (yydebug)
    printf("%s: -->%s<--, line %i\n", n, yytext, CDLSfsCurrentLine);
}

void CDLSfsCount()

{
}

void CDLSfsBuffer()

{
  register char* curChar;

  for (curChar = yytext; *curChar != '\0'; curChar++)
    if (*curChar == '\n')
      newlineCount++;
  CDLSfsCurrentLine += newlineCount;
  SetCurrentLine(CDLSfsCurrentLine);
  newlineCount = 0;
}

%}

%%

{Hash}{Blank}+{Digit}+{Blank}+\"{TextChar}+\" {
  int newLineNo;
  sscanf(yytext, "# %i", &newLineNo);
  CDLSfsGotToken("Line directive");
  CDLSfsCurrentLine = newLineNo - 1;	/* Other stuff counts newlines */
  SetCurrentLine(CDLSfsCurrentLine);
}

{Hash}{Blank}+{Digit}+ {
  int newLineNo;
  sscanf(yytext, "# %i", &newLineNo);
  CDLSfsGotToken("Line directive");
  CDLSfsCurrentLine = newLineNo - 1;	/* Other stuff counts newlines */
  SetCurrentLine(CDLSfsCurrentLine);
}

{Punctuation}  {
    CDLSfsGotToken("Punctuation");
    CDLSfsBuffer();
    return (int)(yytext[yyleng - 1]);
  }

{Punctuation}{Nonpunctuation} {
    int firstChar;

    firstChar = (int)yytext[0];
    yyless(1);
    CDLSfsGotToken("Punctuation + Nonpunctuation");
    CDLSfsBuffer();
    return firstChar;
  }

{Punctuation}{Nonblank} {
    register int inc;
    int firstChar;

    firstChar = (int)yytext[0];
    yyless(1);
    CDLSfsGotToken("Punctuation + Nonblank");
    CDLSfsBuffer();
    return firstChar;
  }

index {
    CDLSfsGotToken("IndexSpec");
    CDLSfsBuffer();
    return IndexSpec;
  }
   
file {
    CDLSfsGotToken("FileSpec");
    CDLSfsBuffer();
    return FileSpec;
  }

of {
    CDLSfsGotToken("OfSpec");
    CDLSfsBuffer();
    return OfSpec;
  }

clustered {
    fileType = CLUSTERED;
    CDLSfsGotToken("FileTypeSpec");
    CDLSfsBuffer();
    return FileTypeSpec;
  }

relative {
    fileType = RELATIVE;
    CDLSfsGotToken("FileTypeSpec");
    CDLSfsBuffer();
    return FileTypeSpec;
  }

sequential {
    fileType = SEQUENTIAL;
    CDLSfsGotToken("FileTypeSpec");
    CDLSfsBuffer();

    return FileTypeSpec;
  }

hashed {
    fileType = CLUSTERED;
    CDLSfsGotToken("FileTypeSpec");
    CDLSfsBuffer();
    return FileTypeSpec;
  }

duplicates {
    duplicates = 1;
    CDLSfsGotToken("DuplicateSpec");
    CDLSfsBuffer();
    return DuplicateSpec;
  }

noduplicates {
    duplicates = 0;
    CDLSfsGotToken("DuplicateSpec");
    CDLSfsBuffer();
    return DuplicateSpec;
  }

ascending {
    direction = FORWARDS;
    CDLSfsGotToken("DirectionSpec");
    CDLSfsBuffer();
    return DirectionSpec;
  }

descending {
    direction = BACKWARDS;
    CDLSfsGotToken("DirectionSpec");
    CDLSfsBuffer();
    return DirectionSpec;
  }

language {
    hasLang = 1;
    CDLSfsGotToken("LanguageSpec");
    CDLSfsBuffer();
    return LanguageSpec;
  }

acl_object {
    hasACL = 1;
    CDLSfsGotToken("ACLSpec");
    CDLSfsBuffer();
    return ACLSpec;
  }

build {
    buildIt = 1;
    CDLSfsGotToken("BuildSpec");
    CDLSfsBuffer();
    return BuildSpec;
  }

nobuild {
    buildIt = 0;
    CDLSfsGotToken("BuildSpec");
    CDLSfsBuffer();
    return BuildSpec;
  }

documentation {
    CDLSfsGotToken("DocumentationSpec");
    CDLSfsBuffer();
    return DocumentationSpec;
  }

{Digit}+ {
    AddToParseList(yytext);
    CDLSfsGotToken("Integer");
    CDLSfsBuffer();
    return Integer;
  }

{TextChar}+ {
    AddToParseList(yytext);
    CDLSfsGotToken("TextString");
    CDLSfsBuffer();
    return TextString;
  }

{Blanks}  {
    CDLSfsBuffer();
  }

%%
