/*
 * sfsparse.y:
 *    This file defines the parser that parses the files that define
 * which SFS files to create and what indices to create for them.
 *
 * sfsparse.y,v 1.3 1993/12/03 14:44:30 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


%{
#include <stdio.h>
#include <string.h>
#include <sfsparse.h>
#include <sfsparsedefs.H>
#include <parselist.H>

extern void CDLSfsCount();
%}

%token Integer TextString IndexSpec FileSpec FileTypeSpec DuplicateSpec
%token DirectionSpec LanguageSpec ACLSpec BuildSpec DocumentationSpec OfSpec
%start WholeFile

%%

empty		: /* Nothing */
		;
IndexField	: TextString FieldIndexMods
  		{
		  CDLSfsAddIndexField(GetFromParseList(0)); /* Check parse list */
		  ClearParseList();
		}
		;
IndexFieldList	: IndexFieldList ',' IndexField
		| IndexField
		;
Language	: LanguageSpec '=' TextString
		{
		  ACLName = strdup(GetFromParseList(ParseListLength() - 1));
		}
		;
ACL		: ACLSpec '=' TextString
	        {
		  langName = strdup(GetFromParseList(ParseListLength() - 1));
		}
		;
WrongMod	: TextString
		| TextString '=' TextString
		;
TopIndexMod 	: DirectionSpec
		| DuplicateSpec
		| BuildSpec
		| WrongMod
		{
		  CDLPNumError(1114);
		  CDLSfsFailed = 1;
		}
		;
FieldIndexMod 	: DirectionSpec
		| Language
		| WrongMod
		{
		  CDLPNumError(1115);
		  CDLSfsFailed = 1;
		}
		;
TopModList	: TopModList ',' TopIndexMod
		| TopIndexMod
		;
FieldModList	: FieldModList ',' FieldIndexMod
		| FieldIndexMod
		;
TopIndexMods	: '[' TopModList ']'
		| TextString
		{
		  CDLPNumError(1114);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
  		| empty
		;
FieldIndexMods	: '[' FieldModList ']'
		| TextString
		{
		  CDLPNumError(1115);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
  		| empty
		;
IndexStart	: IndexSpec TextString TopIndexMods
		{
		  CDLSfsStartIndex(GetFromParseList(0));	/* Checks list */
		  ClearParseList();
		}
Index		: IndexStart '{' IndexFieldList '}' ';'
		{
		  CDLSfsEndIndex();
		}
		| error ';'
		{
		  ClearParseList();
		  CDLSfsFailed = 1;
		  yyerrok;
		}
		;
IndexList	: IndexList Index
		| Index
		;
FileMod		: FileTypeSpec
		| DuplicateSpec
		| ACL
		| WrongMod
		{
		  CDLPNumError(1116);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
		;
FileModList	: FileModList ',' FileMod
		| FileMod
		;
FileMods	: '[' FileModList ']'
		| empty
		;
FileStart	: FileSpec TextString OfSpec TextString FileMods
		{
		  CDLSfsStartFile(GetFromParseList(0), GetFromParseList(1));
		  ClearParseList();		/* Checks list */
		}
		| FileSpec TextString TextString TextString FileMods
		{
		  CDLPNumError(1117);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
		| FileSpec TextString TextString FileMods
		{
		  CDLPNumError(1117);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
		| FileSpec TextString FileMods
		{
		  CDLPNumError(1117);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
		| FileSpec TextString 
		{
		  CDLPNumError(1117);
		  CDLSfsFailed = 1;
		  ClearParseList();
		}
		;
FileDefinition 	: FileStart '{' IndexList '}' ';'
		{
		  CDLSfsEndFile();
		  ClearParseList();
		}
		| error ';'
		{
		  ClearParseList();
		  CDLSfsFailed = 1;
		  yyerrok;
		}
		;
Item		: FileDefinition
		;
ItemList	: ItemList Item
		| Item
		;
WholeFile	: ItemList
		;

%%

