/*
 * sfsparsewrap.C
 *
 *    These functions are called by the parser in a similar manner to the
 * functions in mainparsewrap.C. However, some of these functions 
 * (CDLSfsStartFile, CDLSfsStartIndex and CDLSfsAddIndexField) must also
 * check the contents of the parse list for optional modifiers as well as
 * just the arguments passed them.
 *
 * sfsparsewrap.C,v 1.3 1993/12/03 14:44:35 lcp Exp
 */

    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <iostream.h>
#include <parselist.H>
#include <sys/param.h>
#include <sfsparsedefs.H>
#include <cdlauxparser.H>
#include <cdlfiledef.H>
#include <cdlindexdef.H>
#include <cdlindexfielddef.H>
#include <string.h>

/* Globals */

int CDLSfsCurrentLine;			// Current line in parser
int CDLSfsFailed;
CDLFileType fileType;
int duplicates;
CDLIndexDir direction;
int hasLang;
char* langName;
int hasACL;
char* ACLName;
int buildIt;

static char CDLAuxFileName[MAXPATHLEN + 1];
static CDLAuxParser* theParser;
static CDLFileDef* newFile;
static CDLIndexDef* newIndex;
static CDLIndexFieldDef* newField;
static CDLIndexDir indexDir;

void CDLSfserror(char* errStr)

{
  CDLPError(errStr);
}

void CDLSfsSetParser(CDLAuxParser* p)

{
  theParser = p;
}

void CDLSfsSetName(const char* fName)

{
  strncpy(CDLAuxFileName, fName, MAXPATHLEN);
  CDLAuxFileName[MAXPATHLEN] = '\0';
  SetCurrentFile(fName);
}

CDLAuxParser* CDLSfsGetParser()

{
  return theParser;
}

static void SetDefaults()

{
  fileType = CLUSTERED;	// btree files are default
  duplicates = 0;	// No duplicates allowed in indices or files
  direction = DEFAULT;	// Indices are made ascending later
  hasLang = 0;		// String fields have no collating language
  buildIt = 1;		// Indices are built
  if (langName != (char*)0)
    {
      free(langName);;
      langName = (char*)0;
    }
  hasACL = 0;		// Files have no ACL checking on
  if (ACLName != (char*)0)
    {
      free(ACLName);
      ACLName = (char*)0;
    }
}

void CDLSfsStartFile(const char* fileName, const char* recName)

{
  if (hasACL == 1)
    newFile = new CDLFileDef(fileName, recName, fileType, duplicates, ACLName);
  else
    newFile = new CDLFileDef(fileName, recName, fileType, duplicates, "");
  SetDefaults();
}

void CDLSfsEndFile()

{
  if (!theParser->AddFile(newFile))	// The parser will delete it later
    CDLSfsFailed = 1;
}

void CDLSfsStartIndex(const char* indexName)

{	
  if (direction == DEFAULT)
    direction = FORWARDS;
  indexDir = direction;
  newIndex = new CDLIndexDef(indexName, direction, duplicates, buildIt);
  SetDefaults();
}

void CDLSfsEndIndex()

{
  if (!newFile->AddIndex(newIndex))
    CDLSfsFailed = 1;
}

void CDLSfsAddIndexField(const char* fieldName)

{
  if (direction == DEFAULT)
    direction = indexDir;
  if (hasLang)
    newField = new CDLIndexFieldDef(fieldName, direction, langName);
  else
    newField = new CDLIndexFieldDef(fieldName, direction, "");
  if (!newIndex->AddField(newField))
    CDLSfsFailed = 1;
  SetDefaults();
}


