    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <cdl/pack.H>
#include <memory.h>
unsigned long FieldSizeTotal(const rqs_recordSpec_t & rs)
/* return the length of the buffer required to hold all the data of the
 * specified type.  This means that in the case of varrying byte arrays the
 * header length must be included
 */
    {
	unsigned long retval = 0;
	unsigned short i;
	for (i = 0; i < rs.numFields; i++)
	    retval += FieldSizeTotal(rs.fieldSpecArray[i]);
	return retval;
    }
unsigned long FieldSizeTotal(const rqs_typeSpec_t & rts, int key)
/* return the length of the buffer required to hold all the data of the
 * specified type and key.  This means that in the case of varrying byte
 * arrays the header length must be included.  Return (unsigned long)-1
 * if the key is invalid.
 */
    {
	if (key < 0 || key >= rts.numKeys) return (unsigned long)-1;
	unsigned long retval = 0;
	rqs_keySpec_t & rks = rts.keySpecArray[key];
	int i = 0;
	for (i = 0; i < rks.numFields;i++)
	    {	/* for each field in the key */
		const char * const fieldName = rks.fieldNameArray[i];
		unsigned int j;
		for (j = 0; j < rts.recordSpec.numFields;j++)
		    {	/* lookup field spec by name */
			const rqs_recordFieldSpec_t & rfs =
				rts.recordSpec.fieldSpecArray[j];
			if (!strcmp(fieldName, rfs.fieldName))
			    {	/* increment and escape loop */
                                       retval += FieldSizeTotal(rfs);    
				       break;  
			    }
		    }
	    }
	return retval;
    }
unsigned long FieldSizeTotal(const rqs_recordFieldSpec_t & rfs)
/* return the length of the buffer required to hold all the data of the
 * specified type.  
 */
    {
	unsigned long retval = 0;
	retval += rfs.fieldSize;
	return retval;
    }

void *Pack(void * buffer, const unsigned char & src)
{
// chars are actually stored as shorts! so, assign it to a short to
// get the extension and then copy to the buffer
  unsigned short * retval = (unsigned short *)buffer;
  unsigned short extended_src = src;
  memcpy(retval,&extended_src,sizeof(extended_src));
  retval++;
  return retval;
}

void *Pack(void * buffer, const signed char & src)
{
  // same as above
  signed short * retval = (signed short *)buffer;
  //*retval = src;
  signed short ext_src = src;
  memcpy(retval, &ext_src, sizeof(ext_src));
  retval++;
  return retval;
}

void *Pack(void * buffer, const unsigned short & src)
{
  unsigned short * retval = (unsigned short *)buffer;
  memcpy(retval,&src,sizeof(src));
  retval++;
  return retval;
}

void *Pack(void * buffer, const signed short & src)
{
  signed short * retval = (signed short *)buffer;
  memcpy(retval, &src, sizeof(src));
  retval++;
  return retval;
}

void *Pack(void * buffer, const unsigned long & src)
{
  unsigned long * retval = (unsigned long *)buffer;
  memcpy(retval, &src, sizeof(src));
  retval++;
  return retval;
}
void *Pack(void * buffer, const signed long & src)
{
  signed long * retval = (signed long * )buffer;
  memcpy(retval, &src, sizeof(src));
  retval++;
  return retval;
}
void *Pack(void * buffer, const ndr_hyper_int_rep & src)
{
  ndr_hyper_int_rep * retval = (ndr_hyper_int_rep *)buffer;
  memcpy(retval, &src, sizeof(src));
  retval++;
  return retval;
}
void *Pack(void * buffer, const ndr_uhyper_int_rep & src)
{
  ndr_uhyper_int_rep * retval = (ndr_uhyper_int_rep *)buffer;
  memcpy(retval, &src, sizeof(src));
  retval++;
  return retval;
}
void *Pack(void * buffer, const float & src)
{
  float * retval = (float *)buffer;
  memcpy(retval,&src,sizeof(src));
  retval++;
  return retval;
}
void *Pack(void * buffer, const double & src)
{
  double* retval = (double *)buffer;
  memcpy(retval,&src,sizeof(src));
  retval++;
  return retval;
}


void *PackString(void * buffer, const unsigned char* s, 
			  const unsigned long maxLen )
{
  int len = strlen((const char *)s);
  unsigned long l = maxLen;
  char * p = (char *)buffer;
  strncpy(p, (const char *)s, l);
  char * retval = (char *)p + len;
  *retval = '\0'; // null-terminate again just in case
  retval = (char*)buffer + maxLen;
  return retval;
}


void *PackStringKey(void * buffer, const unsigned char* s, 
                              const unsigned long maxLen )

{
  int len = strlen((const char *)s);
  char * p = (char *)buffer;
  if(len > maxLen)    len = (int)maxLen;   
  strncpy(p, (const char *)s, len);
  char * retval = (char *)p + len; 
  *retval = '\0'; // null-terminate again just in case
  retval++;
  return retval;
}

void *Pack( void * buffer, 
	    const unsigned char * const src, 
	    const unsigned long len, 
	    const int IncludeLength)
{
  unsigned long * p = (unsigned long *)buffer;
  if (IncludeLength)
    {
      memcpy(p, &len, sizeof(unsigned long int));
      p++;
    }
  memcpy((char *)p, src, (int)len);
  char * q = (char *)p + len;
  return (q);
}

void *Unpack(const void * const buffer, unsigned char & dst)
{
// chars are stored as shorts !!
  unsigned short * retval = (unsigned short *)buffer;
  memcpy(&dst,retval,sizeof(dst));
  // we copied one byte but incremented the pointer by the sizeof a short!
  retval++;
  return retval;
}

void *Unpack(const void * const buffer, signed char & dst)
{
  // chars are stored as shorts !!
  signed short * retval = (signed short *)buffer;
  signed short tmp;
  memcpy(&tmp,retval,sizeof(tmp));
  dst = tmp;
  retval++;
  return retval;
}

void *Unpack(const void * const buffer, unsigned short & dst)
{
  unsigned short * retval = (unsigned short *)buffer;
  memcpy(&dst,retval,sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, signed short & dst)
{
  signed short * retval = (signed short *)buffer;
  memcpy(&dst,retval,sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, unsigned long & dst)
{
  unsigned long * retval = (unsigned long *)buffer;
  memcpy(&dst,retval,sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, signed long & dst)
{
  signed long * retval = (signed long * )buffer;
  memcpy(&dst,retval,sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, ndr_hyper_int_rep & dst)
{
  ndr_hyper_int_rep * retval = (ndr_hyper_int_rep *)buffer;
  memcpy(&dst, retval, sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, ndr_uhyper_int_rep & dst)
{
  ndr_uhyper_int_rep * retval = (ndr_uhyper_int_rep *)buffer;
  memcpy(&dst, retval, sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, float & dst)
{
  float * retval = (float *)buffer;
  memcpy(&dst,retval,sizeof(dst));
  retval++;
  return retval;
}
void *Unpack(const void * const buffer, double & dst)
{
  double * retval = (double *)buffer;
  memcpy(&dst,retval,sizeof(dst));
  retval++;
  return retval;
}

/* unpack byte arrays */
void *Unpack(const void * const buffer, char * dst,
	     idl_ulong_int* returnLength, 
	     const unsigned long maxLen /* length of target buffer */,
	     const int IncludeLength)
{
  char * retval = (char*)buffer;
  unsigned long len;
  if (IncludeLength)
    {
      memcpy(&len, retval, sizeof (len));
      if (len > maxLen)
	len = maxLen;  // just in case!!!
      retval += sizeof(len);
    }
  else
    {
      len = maxLen;
    }
  *returnLength = len;
  memcpy(dst, retval, (int)len);
  retval += len;
  return retval;
}
	     

/* unpack strings*/
void *Unpack(const void * const buffer, char * dst,
	     const unsigned long maxLen /* length of target buffer */
	     )
{
  char * retval = (char*)buffer;
    
  memcpy(dst, retval, maxLen);
  retval += maxLen;
  return retval;
}


