    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <iostream.h>
#include <memory.h>
#include <cdl/rqsutil.H>

Map<String, rqs_serverHandle_t>* 
    RqsBinding::bindings = (Map<String, rqs_serverHandle_t>*)0;

#ifdef _AIX
#include <Map.c>

#pragma define (Map<String,char*>)
#pragma define (Mapiter<String,char*>)
#pragma define (Mapnode_ATTLC<String,char*>)
#endif

RqsBinding::RqsBinding()
    {
	handle = (rqs_serverHandle_t)0;
	server = "";
	if (bindings == (Map<String, rqs_serverHandle_t>*)0)
	  {
	    bindings = new Map<String, rqs_serverHandle_t>(0);
	  }
    }
RqsBinding::RqsBinding(const String & newServer)
    {
	if (bindings == (Map<String, rqs_serverHandle_t>*)0)
	  {
	    bindings = new Map<String, rqs_serverHandle_t>(0);
	  }
	SetServer(newServer);
    }
RqsBinding::~RqsBinding(){}
int RqsBinding::Bound() const
    {
	return(handle != (rqs_serverHandle_t)0);
    }
void RqsBinding::Boundless() 
    {
	if (!Bound())
	    Bind(Server());
	while (!Bound())
	    {
		Bind(Server());
		cerr << "Warning! rqs server\"" << Server() << 
		"\" not responding" << endl;
		sleep(5);
	    }
    }
void RqsBinding::SetServer(const String & newServer)
    {
	server = newServer;
	Bind(newServer);
    }
ostream & operator<<(ostream & o, const RqsBinding & b)
    {
	o << b.handle << endl << b.server << endl;
	return o;
    }
rqs_serverHandle_t RqsBinding::Bind(const String & s)
    {/*	Lookup a server handle by name.  If the server name is in the list of
      *	bindings then returns it's handle else bind to the server.  if the
      *	binding failed return null else add the binding to the list and return
      *	it's value */
	handle = (*bindings)[s]; // get the binding for server s
	if (handle != 0)   // then it was already on the list
	    return handle;
		
	// otherwise, get a binding and put the new one on the list
	const char * serverName = s;
	rqs_status_t sts = rqs_GetServerHandle((char*)serverName, &handle);
	if (sts != RQS_SUCCESS)
	    handle = (rqs_serverHandle_t )0;
	else
	    {
		(*bindings)[s] = handle;
	    }
	return handle;
    }
const String RqsBinding::Find(const rqs_serverHandle_t q) const
    {/*	return the name of the server the corresponds to a binding handle.  If
      *	the handle is not in ouir list then return an empty string */
	String retval;
	Mapiter<String, rqs_serverHandle_t> iter(*bindings);
	while (iter.next())
	    if (iter.value() == q)
	    {
		retval = iter.key();
		break;
	    }
	return retval;
    }
void RqsBinding::Remove(const String & s)
    {	/* remove the element of the bindings list that corresponds to the
	 * specified server name.  Free the memory allocated to the handle. */
	handle = (*bindings)[s];
	if (handle != 0)
	    {
	    bindings->remove(s);
	    rqs_FreeServerHandle(handle);
	    }
    }
void RqsBinding::Remove(const rqs_serverHandle_t q)
    {	/* remove the element of the bindings list that is equal to the
	 * specified handle.  Free the memory allocated to the handle */
	Mapiter<String, rqs_serverHandle_t> iter(*bindings);
	while (iter.next())
	    if (iter.value() == q)
	    {
		bindings->remove(iter.key());
		rqs_FreeServerHandle(iter.value());
		break;
	    }
    }
YQelementID::YQelementID(){}
YQelementID::YQelementID(const String &src)
    {
	rqs_StringToElementId((char *)(const char *)src, &id);
    }
YQelementID::~YQelementID(){}
YQelementID & YQelementID::operator=(const String & src)
    {
	rqs_StringToElementId((char *)(const char *)src, &id);
	return *this;
    }
YQelementID::operator String() const
    {
	String retval;
	char * p;
	rqs_elementId_t* id = (rqs_elementId_t*)this;
	rqs_status_t s = rqs_ElementIdToString(id, &p);
	if (s == RQS_SUCCESS)
	    {
		retval = p;
		rqs_Free(p);
	    }
	else
	    {
		retval = "Error in conversion of Element ID to String";
	    }
	return retval;
    }

ostream& operator<<(ostream& o, const YQelementID& e)
{
   o << "YQElementID: " << (String)e;
   return o;
}

