


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


/* static non-local stuff for class YQbase */
#include <iostream.h>
#include <cdl/yqbase.H>
extern "C" {
#include <threadTid/threadTid.h>
}
#include <objdce/objtranparams.H>

const char* YQbase::DefaultServer = 0;
static pthread_once_t YQbaseInited = pthread_once_init;
static pthread_once_t RQSClientInited = pthread_once_init;

ostream & operator<<(ostream & o, const rqs_serverHandle_t src)
    {
	o << src << endl;
	return o;
    }
ostream & operator<<(ostream &o, const YQbase & src)
    {
	o << src.DefaultServer << endl << 
	src.aName << endl << src.RqsStatusValue << endl;
	return o;
    }

static void YQbaseOnetimeInit()

{
  YQbase::DefaultServer = getenv("ENCINA_RQS_SERVER");
}

YQbase::YQbase(const String& ApplicationName) : aName(ApplicationName)

{
  CDLFirstInit();
  if (pthread_once(&YQbaseInited, YQbaseOnetimeInit) != 0)
    {
      perror("Couldn't initialize RQS client. Exiting");
      exit(1);
    }
  if (ObjtranMainMode != 0)
    {
      if (pthread_once(&RQSClientInited, rqs_ClientInit) != 0)
	{
	  perror("Couldn't initialize RQS client. Exiting");
	  exit(1);
	}
    }
}
     
CDLStatus YQbase::ConvertStatus(const rqs_status_t err) const
{
  CDLStatus ret;

  switch (err)
    {
    case RQS_ALREADY_QSET_MEMBER:
    case RQS_NOT_QSET_MEMBER:
    case RQS_SUCCESS:
      ret = CDL_SUCCESS;
      break;

    case RQS_CURSOR_END_OF_QUEUE:
      ret = CDL_END_OF_QUEUE;
      break;

    case RQS_EMPTY_QUEUE:
    case RQS_TIMEOUT_EMPTY_DEQUEUE:
    case RQS_EMPTY_QSET:
      ret = CDL_RECORD_NOT_FOUND;
      break;

    default:
      ret = CDL_SFS_ERROR;
      break;
    }

  return ret;
}

CDLStatus YQbase::ThrowError(const int msgNum, CDLStatus err,
			     Transaction& tran, ErrorLevels::ReportingLevels level) const
{
  CDLReport.ReportMsg(level, msgNum);
  String message;
  if (!CDLReport.GetMsg(msgNum, message))
    message = "Internal YADDL/RQS error";
  if (tran != TRAN_TID_NULL)
    tran.Abort(message, (const int)err, CDLId);
  return CDL_SUCCESS;
}

CDLStatus YQbase::CheckError(const int msgNum, const rqs_status_t err,
                                  Transaction& tran, ErrorLevels::ReportingLevels level) const
{
  if (err != RQS_SUCCESS)
    {
      CDLStatus retVal = ConvertStatus(err);
      if (NoAbort(retVal))
        return retVal;                          // A non-fatal error
      else
        {
          char statusBuf[ENCINA_MAX_STATUS_STRING_SIZE];

          encina_StatusToString(err, ENCINA_MAX_STATUS_STRING_SIZE, statusBuf);
          CDLReport.VReportMsg(level, msgNum, statusBuf);
          if (tran != TRAN_TID_NULL)
            tran.Abort(statusBuf, (const int)retVal, CDLId);
          return retVal;
        }
    }
  else
    return CDL_SUCCESS;
}

/*
 * NoAbort
 *    If the error message passed is non-fatal, we should not abort the
 * transaction.
 */

Boolean YQbase::NoAbort(CDLStatus err) const


{
  Boolean retVal;
  switch (err)
    {
    case CDL_RECORD_NOT_FOUND:
    case CDL_END_OF_QUEUE:
      retVal = true;
      break;

    default:
      retVal = false;
      break;
    }
  return retVal;
}


/*
 * RetryError
 *    Return true if the error message indicates the operation should
 * be retried.
 */

Boolean YQbase::RetryError(const rqs_status_t err) const

{
  Boolean retVal;

  switch (err)
    {
    case RQS_COMMUNICATION_ERROR:
      retVal = true;
      break;
    default:
      retVal = false;
      break;
    }
  return retVal;
}
